/* $Id: class1.cpp,v 1.2 1995/09/23 21:07:56 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	class1.cpp

	The minimal Rx program defining and exporting a class.
	It defines a class Class1 with a method Class1::hello(),
	which appropriately enough also says hello.

	The application defines an AutoCAD command CLASS1
	which invokes Class1::hello through the Rx mechanisms.

	Jeremy Tammik, Autodesk ESC, 94-08-23
*/
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <aced.h> // AcEdStuff
#include <adslib.h>
#include "class1.h"

// Class1 implementation:
//
ACRX_CONS_DEFINE_MEMBERS(Class1, AcRxObject, 0);

extern "C" Class1 * newClass1()
{
	return new Class1;
}

void
Class1::hello()
{
	ads_printf("Class1::hello: Hi there!\n");
}

void
class1()
{
	ads_printf("This is class1() in Class1 ...\n");

	Class1 *h = new Class1;
	ads_printf("Survived creation of a Class1 object. "
		"Let's call its hello function:\n");
	if(!h) {
		ads_printf("Unfortunately, it's a null pointer ...\n");
		return;
	}
	h->hello();
	delete h;
	ads_printf("Survived deletion of Class1 object.\n");
}

void
initApp()
{
	ads_printf("Class1 Rx Sample -- initApp\n");

	/* Step 1:

	   Add classes to the runtime-identifiable class hierarchy.

	   This call was defined by use of the ACRX_CLASS() macro on
	   Class1 and registers its class objects with Rx.
	*/
	Class1::rxInit();

	/* Step 2:

	   Build class relationships before proceeding.	 Use after
	   making a set of init<Classname>() calls which add a contiguous
	   growth onto the inheritance tree.
	*/
	acrxBuildClassHierarchy();

	/* Step 3:

	   Do any other initialization you need.  The point is that it is
	   best to do the classes first, since the rest of the initialization
	   is usually concerned with them.

	   This example application adds a new object to the system dictionary
	   and registers a service name, needed for symbolic function
	   export,	and a handy object for verifying the presence of this
	   application.

	*/
	//acrxSysRegistry()->atPut( "Hello4_Server", new Class1Imp() );

	acrxRegisterService(SERVICE);

	acedRegCmds->addCommand("CLASS1_COMMANDS", "CLASS1", "CLASS1",
					ACRX_CMD_MODAL, &class1);
}

void unloadApp()
{
	ads_printf("Class1 Rx Sample -- kUnloadAppMsg\n");
	acedRegCmds->removeGroup("CLASS1_COMMANDS");

    // remove Class1 from the AcRx runtime tree
    //
	deleteAcRxClass(Class1::desc());
}

extern "C" AcRx::AppRetCode 
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
	switch (msg) {
	case AcRx::kInitAppMsg:
	    acrxDynamicLinker->unlockApplication(pkt);
        initApp();
	    break;
	case AcRx::kUnloadAppMsg:
	    unloadApp();
	    break;
	case AcRx::kLoadADSMsg:
		ads_printf("Class1 Rx Sample -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf("Class1 Rx Sample -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf("Class1 Rx Sample -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}

