/* $Id: class2.cpp,v 1.3 1995/09/23 21:13:39 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	class2.cpp

	The minimal Rx program defining and exporting a class.
	It defines a class Class2 with a method Class2::hello(),
	which appropriately enough also says hello.

	In Class2, we split the class definition into two
	parts: the API class Class2, and the corresponding
	implementation class Class2i.

	The application defines an AutoCAD command CLASS2
	which invokes Class2::hello through the Rx mechanisms.

	Jeremy Tammik, Autodesk ESC, 94-08-23
*/
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <aced.h> // AcEdStuff
#include <adslib.h>
#include "class2i.h"

// Class2 API class implementation:
//
ACRX_NO_CONS_DEFINE_MEMBERS(Class2, AcRxObject);

// Class2i implementation class implementation:
//
#ifdef USE_RXCLASS

ACRX_CONS_DEFINE_MEMBERS(Class2i, Class2, 0);

#endif

void
Class2i::hello()
{
	ads_printf("Class2i::hello: Hi there!\n");
}

// pseudo constructor for API class Class2:
//
Class2 * newClass2()
{
	return new Class2i();
}

// class2 - user command interface:
//
void
class2()
{
	ads_printf("This is class2() in Class2 ...\n");

	Class2 *h = newClass2();
	ads_printf("Survived creation of a Class2 object. "
		"Let's call its hello function:\n");
	if(!h) {
		ads_printf("Unfortunately, it's a null pointer ...\n");
		return;
	}
	h->hello();
	delete h;
	ads_printf("Survived deletion of Class2 object.\n");
}

void
initApp()
{
	ads_printf("Class2 Rx Sample -- initApp\n");

	/* Step 1:

	   Add classes to the runtime-identifiable class hierarchy.

	   This call was defined by use of the ACRX_CLASS() macro on
	   Class2 and registers its class objects with Rx.
	*/
	Class2::rxInit();

#ifdef USE_RXCLASS

	Class2i::rxInit();

#endif
	/* Step 2:

	   Build class relationships before proceeding.	 Use after
	   making a set of init<Classname>() calls which add a contiguous
	   growth onto the inheritance tree.
	*/
	acrxBuildClassHierarchy();

	/* Step 3:

	   Do any other initialization you need.  The point is that it is
	   best to do the classes first, since the rest of the initialization
	   is usually concerned with them.

	   This example application adds a new object to the system dictionary
	   and registers a service name, needed for symbolic function
	   export,	and a handy object for verifying the presence of this
	   application.

	*/

	acrxRegisterService(SERVICE);

	acedRegCmds->addCommand("CLASS2_COMMANDS", "CLASS2", "CLASS2",
					ACRX_CMD_MODAL, &class2);
}

void
unloadApp()
{
	ads_printf("Class2 Rx Sample -- kUnloadAppMsg\n");
	acedRegCmds->removeGroup("CLASS2_COMMANDS");

    // remove Class2 from the AcRx runtime tree
    //
	deleteAcRxClass(Class2::desc());
}

extern "C" AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
	switch (msg) {
	case AcRx::kInitAppMsg:
	    acrxDynamicLinker->unlockApplication(pkt);
        initApp();
        break;
	case AcRx::kUnloadAppMsg:
	    unloadApp();
	    break;
	case AcRx::kLoadADSMsg:
		ads_printf("Class2 Rx Sample -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf("Class2 Rx Sample -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf("Class2 Rx Sample -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}

