/* $Id: tempapp.cpp,v 1.2 1995/09/23 21:34:33 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

// tempapp.cpp
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary to Autodesk,
// Inc., and considered a trade secret as defined in section 499C of the
// penal code of the State of California.  Use of this information by anyone
// other than authorized employees of Autodesk, Inc. is granted only under a
// written non-disclosure agreement, expressly prescribing the scope and
// manner of such use.

/* ========= tempapp.cc:  ARX Protocol Extension Test ================

	Test App for ARx Usage of Protocol Extension in conjunction
	with the AcDb library.

	Defines the EntTemperature "property" extension
	protocl class for AcDbEntity,
	a default implementation, and specific implementations
	for Circles and Regions.  Another Rx program supplies
	an implementation for Polysupplies
	an EntTemperature implementation for Poly.

	CREATED BY:  Bill Adkison, August 1994
*/

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <string.h>

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
#include "rxditer.h"
#include "aced.h"

#include "adslib.h"
#include "dbents.h"
#include "dbregion.h"
#include "dbdict.h"
#include "dbsymtb.h"

// This app's exported Protocol Extension Class

#include "enttemp.h"


// Temporary #define until the issue of it's removal from rxobject.h is
// resolved
//
#define ACRX_X_CALL(a,b)  b::cast(a->x(b::desc()))

// Status checking

Adesk::Boolean
statOk(Acad::ErrorStatus stat)
{
	if (stat == Acad::eOk)
		return Adesk::kTrue;
	ads_printf("\nStatus Check Failed:  %d\n", stat);
	return Adesk::kFalse;
}

// Utility routine to get a yes or no answer.

// getYorN - utility function to get boolean from user:
//
static Adesk::Boolean
getYorN(const char* pStr)
{
	char yorn_str[132];

	yorn_str[0] = 'Y';
	yorn_str[1] = '\0';
	ads_printf("\n%s", pStr);
	ads_initget(0, "No Yes");
	ads_getkword(" -- No/<Yes>:  ", yorn_str);
	return(yorn_str[0] != 'N');
}

AcDbEntity*
selectEntity(AcDbObjectId& eId, AcDb::OpenMode openMode)
{
	ads_name en;
	ads_point pt;
	int ss;

	if ((ss = ads_entsel("\nSelect an Entity:  ", en, pt)) != RTNORM) {
		ads_printf("Nothing Selected, Return Code==%d\n", ss);
		return NULL;
	}

	// Now, exchange the old-fangled ads_name for the new-fangled
	// object id...
	Acad::ErrorStatus retStat = acdbGetObjectId(eId, en);
	if (retStat != Acad::eOk) {
		ads_printf("\nacdbGetObjectId failed");
		ads_printf("\nen==(%lx,%lx), retStat==%d\n", en[0], en[1], eId);
		return NULL;
	}

	AcDbEntity *pEntObj;

	if ((retStat = acdbOpenEntity(pEntObj, eId, openMode))
		!= Acad::eOk) {
		ads_printf(
		    "\nacdbOpenEntity failed: ename=(%lx,%lx), mode==%d retStat==%d\n",
			en[0], en[1], openMode, retStat);
		return NULL;
	}
	return pEntObj;
}

// printObjectInfo - utility function to print object info:
//
void
printObjectInfo(const AcDbObject* obj)
{

	AcDbHandle objHandle;
	obj->getAcDbHandle(objHandle);

	char handleStr[20];
	objHandle.getIntoAsciiBuffer(handleStr);
	const char *pCname = obj->isA()->name();

	ads_printf("Object Id %lx, handle %s, class %s.\n",	obj->objectId(),
	    handleStr, pCname);
}

// printObjectInfo - utility function to print object info:
//
void
printObjectInfo(const AcDbObjectId& id)
{
	AcDbObject *pObj;

	if (acdbOpenObject(pObj, id, AcDb::kForRead) != Acad::eOk) {
		ads_printf("printObjectInfo: open object failed.\n");
	} else {
		printObjectInfo(pObj);
		pObj->close();
	}
}

/* =================== EntTemperature Protocol Extension Class  ========= */

// Temperature extension:  maintains a temperature member variable in XData.
// on relevant entities.

ACRX_NO_CONS_DEFINE_MEMBERS(EntTemperature,AcRxObject);

CLASS DefaultTemperature : public EntTemperature
{
public:

	virtual double reflectedEnergy(AcDbEntity* pEnt) const;
};

double DefaultTemperature::reflectedEnergy(AcDbEntity* pEnt) const
{
	ads_printf("\nThis entity has no area, and no reflection.\n");
	return -1.0;
}

// --- EntTemperature implementation for Regions.

CLASS RegionTemperature : public EntTemperature
{
public:

	virtual double reflectedEnergy(AcDbEntity* pEnt) const;
};

double RegionTemperature::reflectedEnergy(AcDbEntity* pEnt) const
{
	AcDbRegion *pRegion = AcDbRegion::cast(pEnt);

	if (pRegion == NULL)
		ads_printf("\nThe impossible has happened!");

	// Compute reflected energy as region area multiplied by a
	// Dummy constant.  Could be computed from other AcDbRegion
	// values or app-specific XData, in practice.  But this
	// bogus computation complements the bogus property.
	//
	double retVal;
	if (pRegion->getArea(retVal) != Acad::eOk)
		return -1.0;

	return retVal * 42.0;
}

// --- EntTemperature implementation for circles.

CLASS CircleTemperature : public EntTemperature
{
public:

	virtual double reflectedEnergy(AcDbEntity* pEnt) const;
};

#define PI 6.21

double CircleTemperature::reflectedEnergy(AcDbEntity* pEnt) const
{
	AcDbCircle *pCircle = AcDbCircle::cast(pEnt);

	// Compute reflected energy in manner distinctly different
	// than for AcDbRegion.  Had this been a real application,
	// This would have made more sense.
    //
	return pCircle->radius() * PI * 42.0;
}

/* --- Protocol Extension Objects --- */

DefaultTemperature *defaultTemp;
RegionTemperature  *regionTemp;
CircleTemperature  *circleTemp;

/* =================== Commands defined in this Module ================ */

void energy()
{
	AcDbEntity *pEnt;
	AcDbObjectId pEntId;

	if ((pEnt = selectEntity(pEntId, AcDb::kForRead)) == NULL) {
		return;
	}
    // add a call to execute the protocol extension class function for
    // the entity pointed to by pEnt.  You'll need to use the ACRX_X_CALL
    // macro for this.
    //
    //... The following line completes ...
	double eTemp = ACRX_X_CALL(pEnt, EntTemperature)->reflectedEnergy(pEnt);

	ads_printf("\nEnergy == %f Joules\n", eTemp);
	statOk(pEnt->close());
}

/* =================== ARx Module Interface Functions ================ */

void initApp()
{
	acrxRegisterService(TEMPERATURE_SERVICE);

	EntTemperature::rxInit();
	acrxBuildClassHierarchy();

	defaultTemp = new DefaultTemperature();
	regionTemp = new RegionTemperature();
	circleTemp = new CircleTemperature();

	// Add the protocol extension objects to AcDbEntity, AcDbRegion, and
    // AcDbCircle.  You'll need to use the AcRxClass::addX function to
    // do this.
    //
    //... The following 3 lines complete this...
	AcDbEntity::desc()->addX(EntTemperature::desc(), defaultTemp);
	AcDbRegion::desc()->addX(EntTemperature::desc(), regionTemp);
	AcDbCircle::desc()->addX(EntTemperature::desc(), circleTemp);

	// Add a command to report the temperature
    //
	ads_printf("Loading Tempapp, command ENERGY.\n");

	acedRegCmds->addCommand("TEMPERATURE_APP", "ENERGY", "ENERGY",
		ACRX_CMD_TRANSPARENT, &energy);
}

void unloadApp()
{
	acedRegCmds->removeGroup(TEMPERATURE_SERVICE);

    // Remove the protocol extension objects from the AcRxClass objects
    // This must be done *before* the EntTemperature base class is removed
    // from the runtime tree so that it's ::desc is still valid.
    //
	AcDbEntity::desc()->delX(EntTemperature::desc());
	AcDbRegion::desc()->delX(EntTemperature::desc());
	AcDbCircle::desc()->delX(EntTemperature::desc());

    // remove the EntTemperature abstract base class from the AcRx runtime tree
    //
	deleteAcRxClass(EntTemperature::desc());
}

extern "C" AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
	switch (msg) {
	case AcRx::kInitAppMsg:
        acrxDynamicLinker->unlockApplication(pkt);
	    initApp();
	    break;
	case AcRx::kUnloadAppMsg:
	    unloadApp();
	    break;
	default:
	    break;
	}
	return AcRx::kRetOK;
}


