/* $Id: jblob.h,v 1.8 1995/09/23 21:39:59 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

//
// jblob.h - jblob entity
//
// Jeremy Tammik, Autodesk ESC, 94-08-10
//
// Based on bcurve.h and bcurve.cpp by 
// Peter Janson, 10. May 1994.
//
#ifndef _JBLOB_H
#define _JBLOB_H

#include "dbents.h"
#include "acgi.h"

class Jblob : public  AcDbEntity
{
public:
    ACRX_DECLARE_MEMBERS(Jblob);
	Jblob();
	virtual	~Jblob();

	virtual Adesk::Boolean		worldDraw(AcGiWorldDraw* mode);
	virtual Acad::ErrorStatus	dwgInFields(AcDbDwgFiler* filer);
	virtual Acad::ErrorStatus	dwgOutFields(AcDbDwgFiler* filer) const;
	virtual Acad::ErrorStatus	dxfInFields(AcDbDxfFiler* filer);
	virtual Acad::ErrorStatus	dxfOutFields(AcDbDxfFiler* filer) const;

	// AcDbEntity overrides
	//
	virtual Acad::ErrorStatus	explode(AcDbVoidPtrArray& entitySet) const;
	virtual void				list() const;
	virtual Acad::ErrorStatus	getOsnapPoints(
        AcDb::OsnapMode	osnapMode,
        int gsSelectionMark,
        const AcGePoint3d& pickPoint,
        const AcGePoint3d& lastPoint,
        const AcGeMatrix3d&	viewXform,
        AcGePoint3dArray& snapPoints,
        AcDbIntArray& geomIds) const;
    virtual Acad::ErrorStatus	getGripPoints(
	    AcGePoint3dArray& gripPoints,
	    AcDbIntArray& osnapMasks,
	    AcDbIntArray& geomIds) const;
    virtual Acad::ErrorStatus	moveGripPointsAt(
	    const AcDbIntArray& indices,
	    const AcGeVector3d&	offset);
    virtual Acad::ErrorStatus	getStretchPoints(
	    AcGePoint3dArray& stretchPoints) const;
    virtual Acad::ErrorStatus	moveStretchPointsAt(
	    const AcDbIntArray& indices,
	    const AcGeVector3d&	offset);
    virtual Acad::ErrorStatus	transformBy(const AcGeMatrix3d& xform);

	// Jblob specific protocol
	//
	void endpoint(AcGePoint3d& p) const;
	void setEp(const AcGePoint3d& p); // endpoint

	void center(AcGePoint3d& p) const;
	void setBp(const AcGePoint3d& p); // blob center

	void radius(double& r) const;
	void setBr(const double r); // blob radius

private:

	Acad::ErrorStatus osnapNear(const AcGePoint3d&	pickPoint,
	    AcGePoint3dArray&	snapPoints) const;
	Acad::ErrorStatus osnapEnd(const AcGePoint3d&	pickPoint,
		AcGePoint3dArray& snapPoints) const;
	Acad::ErrorStatus osnapMid(const AcGePoint3d&	pickPoint,
		AcGePoint3dArray& snapPoints) const;
	Acad::ErrorStatus osnapCen(const AcGePoint3d&	pickPoint,
		AcGePoint3dArray& snapPoints) const;
	Acad::ErrorStatus osnapQuad(const AcGePoint3d&	pickPoint,
		AcGePoint3dArray& snapPoints) const;

	// data members
	//
	AcGePoint3d  mp;      // one endpoint
	AcGePoint3d  mpblob;  // other endpoint == blob center
	double       mrblob;  // blob radius
	AcGeVector3d mnormal; // blob normal vector

};



inline void Jblob::endpoint(AcGePoint3d& p) const
	{p = mp;}

inline void Jblob::setEp(const AcGePoint3d& p) 
	{ mp = p; } // endpoint

inline void Jblob::center(AcGePoint3d& p) const
	{p = mpblob;}

inline void Jblob::setBp(const AcGePoint3d& p) 
	{ mpblob = p; } // blob center

inline void Jblob::radius(double& r) const
	{r = mrblob;}

inline void Jblob::setBr(const double r)       
	{ mrblob = r; } // blob radius

#endif // _JBLOB_H
