/* $Id: rxhelloc.cpp,v 1.1 1994/10/18 16:27:28 edwardb Exp $ */
/* 
      (C) Copyright 1993-1994 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	rxhelloc.cpp

	Client module of RxHello3.

	The minimal Rx program defining and exporting a C function.
	It says 'hello' both at initModule and syncModule.
	It defines a function HelloFunc which appropriately enough
	says hello.

	We have the two modules server and client.

	The client application defines an AutoCAD command HELLO3
	which invokes HelloFunc through the Rx mechanisms.

	Jeremy Tammik, Autodesk ESC, 93-11-19
*/
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <aced.h> // AcEdStuff
#include <adslib.h>
#include "hello3.h"

// global flag tells us whether HelloService is available:

// hello3 - main action here, AutoCAD command:
//
// currently, we assume that once a service has been
// detected , it will remain available for our lifetime.
// If this is not the case, we need to check each time
// ... or did I head someone mention something about
// a reactor for this?

void hello3 ()
{
	static int hello_service_ok = 0;

	ads_printf ("This is hello3() in RxHello3 ...\n");
	if (!hello_service_ok) {
		hello_service_ok = acrxServiceIsRegistered (RX_SERVICE);
		ads_printf ("%s has %sbeen registered.\n", RX_SERVICE,
			hello_service_ok ? "" : "NOT ");
	}
	if (hello_service_ok) {
		HelloFunc();
	}
	else {
		ads_printf ("No HelloService available.\n");
	}
}

void initApp()
{
	ads_printf ("Rx Hello 3 Client -- initApp\n");

	// add the command to the dictionary:

	acedRegCmds->addCommand( "HELLO3_COMMANDS", "HELLO3", 
		"HELLO3", ACRX_CMD_MODAL, &hello3 );
}

void unloadApp()
{
	ads_printf( "Rx Hello 3 -- unloadApp\n");
	acedRegCmds->removeGroup( "HELLO3_COMMANDS" );
}

#ifdef IN_THE_OLD_DAYS_WE_STILL_HAD_SYNCMODULE_CALLS
void syncModule ()
{
	ads_printf ("Rx Hello 3 Client -- syncModule\n");
	hello_service_ok = acrxServiceIsRegistered (RX_SERVICE);
	ads_printf ("%s has %sbeen registered.\n", RX_SERVICE,
		hello_service_ok ? "" : "NOT ");
}
#endif

extern "C" AcRx::AppRetCode acrxEntryPoint (AcRx::AppMsgCode msg, void*)
{
	switch(msg) {
	case AcRx::kInitAppMsg:
		initApp ();
		break;
	case AcRx::kUnloadAppMsg:
		unloadApp ();
		break;
	case AcRx::kLoadADSMsg:
		ads_printf ("Rx Hello 3 Client -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf ("Rx Hello 3 Client -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf ("Rx Hello 3 Client -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}
