/* $Id: rxhello4.cpp,v 1.1 1994/10/18 16:29:18 edwardb Exp $ */
/* 
      (C) Copyright 1993-1994 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	rxhello4.cpp

	The minimal Rx program defining and exporting a class.
	It says 'hello' both at initModule and syncModule.
	It defines a class HelloClass with a method HelloClass::hello(),
	which appropriately enough also says hello.

	We have the three modules server, api library and client.

	The client application defines an AutoCAD command HELLO4
	which invokes HelloClass::hello through the API lib and the
	Rx mechanisms.

	Jeremy Tammik, Autodesk ESC, 93-11-15
*/
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <adslib.h>
#include "helloimp.h" // RX_SERVICE

void initModule()
{
	ads_printf ("Rx Hello 4 Server -- initModule\n");

	/* Step 1:

	   Add classes to the runtime-identifiable class hierarchy.

	   This call was defined by use of the ACRX_CLASS() macro on
	   HelloClass and registers its class objects with Rx.
	*/
	initHelloClass();

#ifdef USE_RXCLASS

	initHelloClassImp();

#endif

	/* Step 2:

	   Build class relationships before proceeding.	 Use after
	   making a set of init<Classname>() calls which add a contiguous
	   growth onto the inheritance tree.
	*/
	acrxBuildClassHierarchy();

	/* Step 3:

	   Do any other initialization you need.  The point is that it is
	   best to do the classes first, since the rest of the initialization
	   is usually concerned with them.

	   This example application adds a new object to the system dictionary
	   and registers a service name, needed for symbolic function
	   export,	and a handy object for verifying the presence of this
	   application.

	*/
	acrxSysRegistry()->atPut ("Hello4_Server", new HelloClassImp());

	acrxRegisterService (RX4_SERVICE);
}

void syncModule ()
{
	ads_printf ("Rx Hello 4 Server -- syncModule\n");
}

extern "C" AcRx::AppRetCode acrxEntryPoint (AcRx::AppMsgCode msg, void*)
{
	switch(msg) {
	case AcRx::kInitAppMsg:
		initModule ();
		break;
	case AcRx::kUnloadAppMsg:
		ads_printf ("Rx Hello 4 -- kUnloadAppMsg\n");
		break;
	case AcRx::kLoadADSMsg:
		ads_printf ("Rx Hello 4 -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf ("Rx Hello 4 -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf ("Rx Hello 4 -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}
