/* $Id: rxhellod.cpp,v 1.1 1994/10/18 16:29:42 edwardb Exp $ */
/* 
      (C) Copyright 1993-1994 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	rxhellod.cpp

	Client module of RxHello4.

	The absolute minimal Rx program defining and exporting
	a class. It says 'hello' both at initModule and syncModule.
	It defines a class Hello with a method Hello::hello(),
	which appropriately enough says hello.

	We have the three modules server, api library and client.

	The client application defines an AutoCAD command HELLO4
	which invokes Hello::hello through the API lib and the
	Rx mechanisms.

	Jeremy Tammik, Autodesk ESC, 93-11-15
*/
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <aced.h> // AcEdStuff
#include <adslib.h>
#include "hello4.h"

// global flag tells us whether HelloService is available:

static int hello_service_ok = 0;

// hello4 - main action here, AutoCAD command:

void hello4 ()
{
	ads_printf ("This is hello4() in RxHello4 ...\n");
	if (!hello_service_ok) {
		hello_service_ok = acrxServiceIsRegistered (RX4_SERVICE);
		ads_printf ("%s has %sbeen registered.\n", RX4_SERVICE,
			hello_service_ok ? "" : "NOT ");
	}
	if (hello_service_ok) {
		HelloClass *h = newHelloClass();
		ads_printf ("Survived creation of a hello object. How exciting.\n"
			"Let's call its hello function:\n");
		if (!h) {
			ads_printf ("Unfortunately, it's a null pointer ...\n");
			return;
		}
		h->hello();
		delete h;
		ads_printf ("Survived deletion of HelloClass object. End of test.\n");
	}
	else {
		ads_printf ("No %s available.\n", RX4_SERVICE);
	}
}

// rxobfun - AutoCAD command to demo Rx object utility functions:

void rxobfun ()
{
	ads_printf ("This is rxobfun() in RxHello4 ...\n");
	if (!hello_service_ok) {
		hello_service_ok = acrxServiceIsRegistered (RX4_SERVICE);
		ads_printf ("%s has %sbeen registered.\n", RX4_SERVICE,
			hello_service_ok ? "" : "NOT ");
	}
	if (hello_service_ok) {
		HelloClass *h = newHelloClass();
		ads_printf ("Survived creation of a hello object. How exciting.\n"
				"Let's call its hello function:\n");
		if (!h) {
			ads_printf ("Unfortunately, it's a null pointer ...\n");
			return;
		}
		h->hello();
		if (h->isKindOf (HelloClass::desc()))
			ads_printf ("It knows it is a kind of HelloClass.\n");
		if (!h->isKindOf (AcRxDictionary::desc()))
			ads_printf ("It knows it is not a kind of AcRxDictionary.\n");
		AcRxClass *cObj = h->desc();
		if (cObj) {
			ads_printf ("Descriptor object was returned, and its getName "
				"method returns %s.\n", cObj->getName()->cpyToAscii());
		}
		else {
			ads_printf ("Null descriptor object.\n");
		}
		delete h;
		ads_printf ("Survived deletion of HelloClass object. End of test.\n");
	}
	else {
		ads_printf ("No %s available.\n", RX4_SERVICE);
	}
}

void initApp()
{
	ads_printf ("Rx Hello 4 Client -- initModule\n");

	acedRegCmds->addCommand ( "HELLO4_COMMANDS", "HELLO4", "HELLO4",
					ACRX_CMD_MODAL, &hello4 );
	acedRegCmds->addCommand ( "HELLO4_COMMANDS", "RXOBFUN", "RXOBFUN",
					ACRX_CMD_MODAL, &rxobfun );
}

void unloadApp()
{
	ads_printf( "Rx Hello 4 -- unloadApp\n");
	acedRegCmds->removeGroup( "HELLO4_COMMANDS" );
}

#ifdef IN_THE_OLD_DAYS_WE_STILL_HAD_SYNCMODULE_CALLS
void syncModule ()
{
	ads_printf ("Rx Hello 4 Client -- syncModule\n");
	hello_service_ok = acrxServiceIsRegistered (RX4_SERVICE);
	ads_printf ("%s has %sbeen registered.\n", RX4_SERVICE,
		hello_service_ok ? "" : "NOT ");
}
#endif

extern "C" AcRx::AppRetCode acrxEntryPoint (AcRx::AppMsgCode msg, void*)
{
	switch(msg) {
	case AcRx::kInitAppMsg:
		initApp ();
		break;
	case AcRx::kUnloadAppMsg:
		unloadApp ();
		break;
	case AcRx::kLoadADSMsg:
		ads_printf ("Rx Hello 4 Client -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf ("Rx Hello 4 Client -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf ("Rx Hello 4 Client -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}
