/* $Id: rxhelloe.cpp,v 1.3 1995/09/06 13:47:29 elevy Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	rxhelloe.cpp

	Client module of RxHello5.

	Here we define an AutoCAD command HELLO5 which instantiates
	a HelloClass object (defined by RxHello4) and invokes the
	protocol extension added to it by RxHello5.

	Jeremy Tammik, Autodesk ESC, 93-11-22
*/
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <aced.h> // AcEdStuff
#include <adslib.h>
#include "../rxhell04/hello4.h"
#include "hello5.h"

// global flags tell us whether required services are available:

static int hello4_service_ok = 0;
static int hello5_service_ok = 0;

// hello5 - main action here, AutoCAD command:

void hello5 ()
{
	ads_printf ("This is hello5() in RxHello5 ...\n");
	if (!hello4_service_ok) {
		hello4_service_ok = acrxServiceIsRegistered (RX4_SERVICE);
		ads_printf ("%s has %sbeen registered.\n", RX4_SERVICE,
			hello4_service_ok ? "" : "NOT ");
	}
	if (!hello5_service_ok) {
		hello5_service_ok = acrxServiceIsRegistered (RX5_SERVICE);
		ads_printf ("%s has %sbeen registered.\n", RX5_SERVICE,
			hello5_service_ok ? "" : "NOT ");
	}
	if (hello4_service_ok && hello5_service_ok) {
		HelloClass *h = newHelloClass();

		ads_printf ("Survived creation of HelloClass object.\n"
			"Let's call the HelloClass hello function:\n");
		if (!h) {
			ads_printf ("Unfortunately, it's a null pointer ...\n");
			return;
		}
		h->hello();

		// now for the protocol extension ...
		// what it should (in an ideal world) be is simply a
		// call to h->hello_pex(), but we must deal with some
		// syntactic Rx sugar:

		ads_printf ("Now for the HelloClass protocol extension function:\n");
		ACRX_X_CALL (h, HelloPex)->hello_pex();
		delete h;
		ads_printf ("Survived deletion of the object. End of test.\n");
	}
	else {
		ads_printf ("At least one of %s and %s are not available.\n",
			RX4_SERVICE, RX5_SERVICE);
	}
}

void initModule()
{
	ads_printf ("Rx Hello 5 Client -- initModule\n");

        acedRegCmds->addCommand( "HELLO5_COMMANDS", "HELLO5", "HELLO5",
                                  ACRX_CMD_MODAL, &hello5 );
}

#ifdef IN_THE_OLD_DAYS_WE_STILL_HAD_SYNCMODULE_CALLS
void syncModule ()
{
	ads_printf ("Rx Hello 5 Client -- syncModule\n");
	hello4_service_ok = acrxServiceIsRegistered (RX4_SERVICE);
	ads_printf ("%s has %sbeen registered.\n", RX4_SERVICE,
		hello4_service_ok ? "" : "NOT ");
	hello5_service_ok = acrxServiceIsRegistered (RX5_SERVICE);
	ads_printf ("%s has %sbeen registered.\n", RX5_SERVICE,
		hello5_service_ok ? "" : "NOT ");
}
#endif

extern "C" AcRx::AppRetCode acrxEntryPoint (AcRx::AppMsgCode msg, void*)
{
	switch(msg) {
	case AcRx::kInitAppMsg:
		initModule ();
		break;
	case AcRx::kUnloadAppMsg:
		ads_printf ("Rx Hello 5 Client -- kUnloadAppMsg\n");
		break;
	case AcRx::kLoadADSMsg:
		ads_printf ("Rx Hello 5 Client -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf ("Rx Hello 5 Client -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf ("Rx Hello 5 Client -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}

