/* $Id: notfytst.cpp,v 1.3 1995/09/06 13:52:13 elevy Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/* Next available MSG number is	  1 */
/*
	  (C) Copyright 1982-1992,1994,1995 by Autodesk, Inc.

******************************************************************************
*																			 *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the		 *
* penal code of the State of California.  Use of this information by anyone	 *
* other than authorized employees of Autodesk, Inc. is granted only under a	 *
* written non-disclosure agreement, expressly prescribing the scope and		 *
* manner of such use.														 *
*																			 *
******************************************************************************/

/* ========= notfytst.cc:  Rx Notification Test/Example ================

	Test App for AcEditorReactor and AcDLinkerReactor usage, to
	obtain notification for:

	- editor events, specifically drawing load,
	  system variable change and command invocation events.

	- temporary "program exit" notification, too.

	- dynamic linker events, specifically application loading and unloading.

	CREATED BY:	 Bill Adkison, October 1993

*/

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
//#include "rxditer.h"
//#include "rxascii.h"
#include "aced.h"

#include "adslib.h"

extern "C" {
// locally defined entry point invoked by Rx.
AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt);
}

// Utility routine to get a yes or no anser.

static Adesk::Boolean getYorN(const char* pStr)
{
	char yorn_str[132];

	// specific prompt.
	ads_printf("\n%s", pStr);

	ads_initget(0, "No Yes");

	yorn_str[0] = 'N';
	yorn_str[1] = '\0';

	switch (ads_getstring(Adesk::kFalse, " -- No/<Yes>:	 ", yorn_str)) {
	case RTKWORD:
		ads_getinput(yorn_str);
		/* Deliberate fallthrough */
		default:
		break;
	}
	return (!((yorn_str[0] == 'N') || (yorn_str[0] == 'n')));
}

/* =================== Commands defined in this Module ================ */

//********* AcRxDLinkerReactor --  Application Load Notification Example

CLASS AppReactor : public AcRxDLinkerReactor {

  public:

	virtual void rxAppLoaded(const char* moduleName);

};

void AppReactor::rxAppLoaded(const char* moduleName)
{
	ads_printf("\nAppReactor:  Rx application '%s' loaded.\n", moduleName);
}

AppReactor * ar1 = NULL;

/* "WATCH_APPS" -- Add or remove a reactor to the acrxDynamicLinker
	system object. */

void watch_apps()
{
	if (getYorN("Start Watching App Loads")) {
		acrxDynamicLinker->addReactor(ar1);
		ads_printf("  Added Reactor to acrxDynamicLinker.\n");
	} else {
		acrxDynamicLinker->delReactor(ar1);
		ads_printf("  Removed Reactor from acrxDynamicLinker.\n");
	}
}

//********* AcEditorReactor --	System Variable Change Notification Example


CLASS SysVarReactor : public AcEditorReactor {

  public:

	virtual void sysVarChanged(const char * varName, Adesk::Boolean success);

	virtual void sysVarWillChange(const char * varName);
};

void SysVarReactor::sysVarWillChange(const char* varName)
{
	ads_printf("\nSysVarReactor: System Variable '%s' will be changed.\n",
			   varName);
}

void SysVarReactor::sysVarChanged(const char* varName, Adesk::Boolean success)
{
	ads_printf("\nSysVarReactor: System Variable '%s' was %schanged.\n",
			   varName, (success ? "" : "not successfully"));
}

SysVarReactor * svr1 = NULL;


/* "WATCH_SYSVARS" -- Add or remove a reactor to the acedEditor system
	object to monitor system variables. */

void watch_sysvars()
{
	if (getYorN("Start Watching System Variables")) {
		acedEditor->addReactor(svr1);
		ads_printf("  Added Sysvar Reactor to acedEditor.\n");
	} else {
		acedEditor->delReactor(svr1);
		ads_printf("  Removed Sysvar Reactor from acedEditor.\n");
	}
}

//********* AcEditorReactor --	Dwg Load Notification Example


CLASS DwgLoadReactor : public AcEditorReactor {

  public:

	virtual void dwgLoaded(Adesk::Boolean success);
	virtual void dwgWillLoad();
};

void DwgLoadReactor::dwgLoaded(Adesk::Boolean success)
{
	ads_printf("\nDwgLoadReactor: Drawing was %sloaded.\n",
			   (success ? "" : "not "));
	if (success)
	ads_printf("Use GETVAR/DWGNAME to determine drawing name\n");
}

void DwgLoadReactor::dwgWillLoad()
{
	ads_printf("\nDwgLoadReactor: Drawing load commencing.\n");
	ads_printf("Use GETVAR/DWGNAME to determine drawing name\n");
	ads_printf("To monitor drawing load, attach reactor to acdbCurDwg\n");
}

DwgLoadReactor * dwgr1 = NULL;


/* "WATCH_DWGLOADS" -- Add or remove a reactor to the acedEditor system
	object to monitor drawing load operations. */

void watch_dwgloads()
{
	if (getYorN("Start Watching Drawing Loads")) {
		acedEditor->addReactor(dwgr1);
		ads_printf("  Added DwgLoad Reactor to acedEditor.\n");
	} else {
		acedEditor->delReactor(dwgr1);
		ads_printf("  Removed DwgLoad Reactor from acedEditor.\n");
	}
}

//********* AcEditorReactor --	Command Event Notification Example


CLASS CommandReactor : public AcEditorReactor {

  public:

	virtual void commandWillStart(const char * cmdStr);

	virtual void commandEnded(const char * cmdStr);

	virtual void commandCancelled(const char * cmdStr);

	virtual void commandFailed(const char * cmdStr);

};

void CommandReactor::commandWillStart(const char* cmdStr)
{
	ads_printf("\nCommandReactor: Command '%s' will begin.\n",
			   cmdStr);
}

void CommandReactor::commandEnded(const char* cmdStr)
{
	ads_printf("\nCommandReactor: Command '%s' ended normally.\n",
			   cmdStr);
}

void CommandReactor::commandCancelled(const char* cmdStr)
{
	ads_printf("\nCommandReactor: Command '%s' was cancelled.\n",
			   cmdStr);
}

void CommandReactor::commandFailed(const char* cmdStr)
{
	ads_printf("\nCommandReactor: Command '%s' failed.\n",
			   cmdStr);
}

CommandReactor * cmdr1 = NULL;


/* "WATCH_COMMANDS" -- Add or remove a reactor to the acedEditor system
			   object to monitor commands.

*/

void watch_commands()
{
	if (getYorN("Start Watching Commands")) {
		acedEditor->addReactor(cmdr1);
		ads_printf("  Added Sysvar Reactor to acedEditor.\n");
	} else {
		acedEditor->delReactor(cmdr1);
		ads_printf("  Removed Sysvar Reactor from acedEditor.\n");
	}
}

/* =================== Rx Module Interface Functions ================ */

static AcRxDictionary* notifyTestCmds = NULL;

// Initialization Function

void initModule()
{
	ar1 = new AppReactor();
	svr1 = new SysVarReactor();
	cmdr1 = new CommandReactor();
	dwgr1 = new DwgLoadReactor();

	// Add some commands to test the MI objects defined in miserv.so

    acedRegCmds->addCommand("NOTIFY_TEST",
                            "WATCH_APPS",
                            "WATCH_APPS",
                            ACRX_CMD_TRANSPARENT,
                            &watch_apps);
    acedRegCmds->addCommand("NOTIFY_TEST",
                            "WATCH_SYSVARS",
                            "WATCH_SYSVARS",
                            ACRX_CMD_TRANSPARENT,
                            &watch_sysvars);
    acedRegCmds->addCommand("NOTIFY_TEST",
                            "WATCH_DWGLOADS",
                            "WATCH_DWGLOADS",
                            ACRX_CMD_TRANSPARENT,
                            &watch_dwgloads);
    acedRegCmds->addCommand("NOTIFY_TEST",
                            "WATCH_COMMANDS",
                            "WATCH_COMMANDS",
                            ACRX_CMD_TRANSPARENT,
                            &watch_commands);
}

AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void*) {
	switch(msg) {
	case AcRx::kInitAppMsg:
		initModule();
		break;
	case AcRx::kUnloadAppMsg:
		acedRegCmds->removeGroup("NOTIFY_TEST");
		default:
		break;
	}
	return AcRx::kRetOK;
}

