/* $Id: rxhellof.cpp,v 1.3 1995/09/06 13:52:17 elevy Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
	rxhellof.cpp

	Another client module of RxHello4.

	This one differs from the original RxHello4 client, RxHelloD,
	in that it uses a notification scheme to find out whether
	the server, RxHello4, is loaded or unloaded.

	Now, initModule, besides defining the command HELLO6 to
	use the RxHello4 services, creates a new HelloReactor
	object and attaches it to the dynamic linker with
	'acrxDynamicLinker->addReactor (hr);'

	For the rest of the documentation on RxHello4 and RxHelloD,
	see the RxHello4 directory.

	Jeremy Tammik, Autodesk ESC, 93-12-06
*/

#include <string.h> // strnicmp
#include <rxobject.h>
#include <rxregsvc.h> // register services
#include <aced.h> // AcEdStuff
#include <rxdlinkr.h>
//#include <rxditer.h>
//#include <rxascii.h>
#include <adslib.h>
#include "../rxhell04/hello4.h"
#include <adeskabb.h>

// global flag tells us whether HelloService is available:

static Boolean helloServiceOk = kFalse;

///////////////////////////////////////////
// HelloReactor to set HelloService flag:
//
// Is this really based on *module* name?
// It should be *service* name!

CLASS HelloReactor : public AcRxDLinkerReactor
{
public:
	virtual void rxAppLoaded( const char* moduleName );
	virtual void rxAppUnLoaded( const char* moduleName );

private: // helper function
	void doit ( const char* moduleName, Boolean isLoad );
};

static HelloReactor * hr = NULL;

void HelloReactor::doit ( const char* moduleName, Boolean isLoad )
{
	ads_printf("\nHelloReactor: Rx application '%s' being %sloaded.\n",
		moduleName, (isLoad?"":"un"));

	if (strstr (moduleName, "RXHELLO4") != 0) { // assuming upper case!
		helloServiceOk = isLoad;
	}
}

void HelloReactor::rxAppLoaded( const char* moduleName )
{
	doit ( moduleName, kTrue );
}

void HelloReactor::rxAppUnLoaded( const char* moduleName )
{
	doit ( moduleName, kFalse );
}

////////////////////////////////////////////////
// hello6 - main action here, AutoCAD command:

void hello6 ()
{
	ads_printf ( "This is hello6() in RxHello6 ...\n" );
	if ( helloServiceOk ) {
		HelloClass *h = newHelloClass();
		ads_printf ("Survived creation of a hello object. How exciting.\n"
			"Let's call its hello function:\n");
		if (!h) {
			ads_printf ("Unfortunately, it's a null pointer ...\n");
			return;
		}
		h->hello();
		delete h;
		ads_printf ("Survived deletion of HelloClass object. End of test.\n");
	}
	else {
		ads_printf ( "No %s available.\n", RX4_SERVICE );
	}
}

void initModule()
{
	hr = new HelloReactor();

	ads_printf ("Rx Hello 6/4 Client -- initModule\n");

    acedRegCmds->addCommand("HELLO6_COMMANDS", "HELLO6", "HELLO6",
                             ACRX_CMD_TRANSPARENT, &hello6);

	acrxDynamicLinker->addReactor (hr);
	ads_printf("Added HelloReactor to acrxDynamicLinker.\n");
}

#ifdef IN_THE_OLD_DAYS_WE_STILL_HAD_SYNCMODULE_CALLS
void syncModule ()
{
	ads_printf ("Rx Hello 6/4 Client -- syncModule\n");
	hello_service_ok = acrxServiceIsRegistered (RX4_SERVICE);
	ads_printf ("%s has %sbeen registered.\n", RX4_SERVICE,
		hello_service_ok ? "" : "NOT ");
}
#endif

extern "C" AcRx::AppRetCode acrxEntryPoint (AcRx::AppMsgCode msg, void*)
{
	switch (msg) {
	case AcRx::kInitAppMsg:
		initModule ();
		break;
	case AcRx::kUnloadAppMsg:
		ads_printf ("Rx Hello 6/4 Client -- kUnloadAppMsg\n");
		break;
	case AcRx::kLoadADSMsg:
		ads_printf ("Rx Hello 6/4 Client -- kLoadADSMsg\n");
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf ("Rx Hello 6/4 Client -- kUnloadADSMsg\n");
		break;
	default:
		ads_printf ("Rx Hello 6/4 Client -- <unknown message %d>\n", msg);
		break;
	}
	return AcRx::kRetOK;
}
