#ifndef AC_ACCMD_H
#define AC_ACCMD_H 1
/* $Id: accmd.h,v 1.16 1995/10/26 16:53:18 neb Exp $ */
/*

   (C) Copyright 1993-1995 by Autodesk, Inc.
  
   The information contained herein is confidential, proprietary
   to Autodesk,  Inc.,  and considered a trade secret as defined
   in section 499C of the penal code of the State of California.
   Use of  this information  by  anyone  other  than  authorized
   employees of Autodesk, Inc.  is granted  only under a written
   non-disclosure agreement,  expressly  prescribing  the  scope
   and manner of such use.
  
   DESCRIPTION:  Definitions for the AutoCAD native command definition
       		 mechanism.
*/

#if defined(__cplusplus)
#include "dbmain.h"
#endif

#define ACRX_CMD_MODAL  	0x00000000
#define ACRX_CMD_TRANSPARENT	0x00000001
#define ACRX_CMD_USEPICKSET	0x00000002
#define ACRX_CMD_REDRAW	        0x00000004
#define ACRX_CMD_INPROGRESS	0x00000400

#define ACRX_MAX_CMDNAME_LEN  	64

#define ACRX_COMMAND_DOCK  	"ACAD_REGISTERED_COMMANDS"

typedef void (*AcRxFunctionPtr) ();

struct AcEdCommandStruc {
    AcRxFunctionPtr fcnAddr;
    long            flags;
    void	    *app;
};


#if defined(__cplusplus)

#include "rxobject.h"

class AcRxDictionary;

class AcEdCommand: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcEdCommand);

    // --- AcEdCommand protocol

    virtual Adesk::Int32 commandFlags() const = 0;

    virtual AcRxFunctionPtr functionAddr() const = 0;

    virtual void *commandApp() const = 0;

    //  --- Extended AcEdCommand protocol

    virtual const char* globalName() const = 0;

    virtual const char* localName() const = 0;

};

// This class was added to support the extension of the 
// classes related to AcEdCommand, and for the purposes
// of allowing commands to have a global (untranslated)
// name, and a local (translated) name.
// This class would not be used otherwise.

class AcEdCommandIterator: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcEdCommandIterator);

    virtual Adesk::Boolean done() const = 0;
    virtual Adesk::Boolean next() = 0;
    virtual const AcEdCommand* command() const = 0;
    virtual const char* commandGroup() const = 0;
};

#define acedRegCmds \
AcEdCommandStack::cast(acrxSysRegistry()->at(ACRX_COMMAND_DOCK))

class AcEdCommandStack: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcEdCommandStack);

    // --- AcEdCommandStack protocol

    //  --- Extended AcEdCommandStack protocol

    virtual Adesk::Boolean atPut(
                                   const char * cmdGroupName,
                                   AcRxDictionary * & cmdsGlobal,
                                   AcRxDictionary * & cmdsLocal) = 0;

    virtual Acad::ErrorStatus addCommand(
                                   const char * cmdGroupName,
                                   const char * cmdGlobalName,
                                   const char * cmdLocalName,
                                   Adesk::Int32 commandFlags,
                                   AcRxFunctionPtr FunctionAddr) = 0;

    virtual AcEdCommandIterator * iterator() = 0;

    virtual AcEdCommand * lookupGlobalCmd(const char * cmdName) = 0;

    virtual AcEdCommand * lookupLocalCmd(const char * cmdName) = 0;

    virtual AcEdCommand * lookupCmd(
                                   const char * cmdName,
                                   Adesk::Boolean globalFlag) = 0;

    virtual AcEdCommand * lookupCmd(
                                   const char * cmdName,
                                   Adesk::Boolean globalFlag,
                                   Adesk::Boolean allGroupsFlag) = 0;

    virtual Acad::ErrorStatus popGroupToTop(
                                   const char * cmdGroupName) = 0;

    virtual Acad::ErrorStatus removeCmd(
                                   const char * cmdGroupName,
                                   const char * cmdGlobalName) = 0;

    virtual Acad::ErrorStatus removeGroup(
                                   const char * groupName) = 0;


};

#endif

#endif
