/* Next available MSG number is   1 */
/* $Id: actrans.h,v 1.9 1995/09/21 11:28:42 jamesr Exp $ */
#ifndef AC_ACTRANS_H_
#define AC_ACTRANS_H_  1

/*
      (C) Copyright 1982-1992,1994,1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
   actrans.h
  
   DESCRIPTION:  Header for Transaction Modeler.
*/

#include "rxnames.h"
#include "aced.h"
#include "dbptrar.h"

/* Name of Registered Service Object for constructing AutoCAD-specific
   instances.
*/

#define AC_TRANSACTION_MANAGER_OBJ    "AcTransactionManager"


//****************************************************************************
//
// Transaction Management
//
//****************************************************************************


class AcTransaction: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcTransaction);
    virtual Acad::ErrorStatus getObject(AcDbObject*&      pObject, 
                                     AcDbObjectId      objectId,    
                                     AcDb::OpenMode    mode,            
                                     Adesk::Boolean    openErasedObject =    
                                                       Adesk::kFalse) = 0; 

    virtual Acad::ErrorStatus markObjectPageable(AcDbObject* pObject) = 0;
};


// Reactor for transaction management.

class AcTransactionReactor: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcTransactionReactor);

    virtual void transactionStarted (int& numTransactions);
    virtual void transactionEnded   (int& numTransactions);
    virtual void transactionAborted (int& numTransactions);
    virtual void endCalledOnOutermostTransaction (int& numTransactions);
 
};


#define actrTransactionManager  \
AcTransactionManager::cast(acrxSysRegistry()->at(AC_TRANSACTION_MANAGER_OBJ))

class AcTransactionManager: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcTransactionManager);

    virtual AcTransaction*      startTransaction() = 0;
    virtual Acad::ErrorStatus   endTransaction  () = 0;
    virtual Acad::ErrorStatus   abortTransaction() = 0;

    virtual int                 numActiveTransactions() = 0;
    virtual AcTransaction*      topTransaction() = 0;
    virtual Acad::ErrorStatus   addNewlyCreatedDBRObject(AcDbObject* pObject, 
                                     Adesk::Boolean add = Adesk::kTrue) = 0;
    virtual Acad::ErrorStatus   getAllObjects(AcDbVoidPtrArray& objects) = 0;

    virtual Acad::ErrorStatus   queueForGraphicsFlush() = 0;
    virtual Acad::ErrorStatus   enableGraphicsFlush(
				     Adesk::Boolean doEnable) = 0;
    virtual void                flushGraphics() = 0;
 
    virtual void addReactor(AcTransactionReactor *pReactor) = 0;
    virtual void removeReactor(AcTransactionReactor *pReactor) = 0;

    virtual Acad::ErrorStatus getObject(AcDbObject*&      pObject, 
                                     AcDbObjectId      objectId,    
                                     AcDb::OpenMode    mode,            
                                     Adesk::Boolean    openErasedObject =    
                                                       Adesk::kFalse) = 0; 

    virtual Acad::ErrorStatus markObjectPageable(AcDbObject* pObject) = 0;
};

#endif
