#ifndef GEOMENT_DBBODY_H
#define GEOMENT_DBBODY_H
//
// $Id: dbbody.h,v 1.23 1995/09/30 15:26:35 jamesr Exp $
//
//*****************************************************************************
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//*****************************************************************************
//
// The AcDbBody class is the interface class for representing
// general ACIS bodies inside AutoCAD.  All the functionality
// supported through this class is implemented in the class
// AcDbImpBody and its base classes.

#include "dbmain.h"
#include "dbsubeid.h"

class AcDbBody: public  AcDbEntity
{
public:
    ACDB_DECLARE_MEMBERS(AcDbBody);
	     AcDbBody();
    virtual ~AcDbBody();

    virtual Adesk::Boolean    isNull       () const;
    virtual void              list         () const;

    virtual void*             body         () const;
    virtual Acad::ErrorStatus setBody      (const void* modelerBody);

    virtual AcDbSubentId       internalSubentId (void*           ent) const;
    virtual void*           internalSubentPtr(const AcDbSubentId& id) const;

#ifdef BINARY_ACIS_FILES

    static  Acad::ErrorStatus acisOut      (const char *fileName,
					    Adesk::Boolean asciiFile,
					    const AcDbVoidPtrArray&);
    static  Acad::ErrorStatus acisIn       (const char *fileName,
					    Adesk::Boolean asciiFile,
					    AcDbVoidPtrArray&);

#else   // BINARY_ACIS_FILES

    static  Acad::ErrorStatus acisOut      (const char *fileName,
					    const AcDbVoidPtrArray&);
    static  Acad::ErrorStatus acisIn       (const char *fileName,
					    AcDbVoidPtrArray&);

#endif  // BINARY_ACIS_FILES

    // Subentity acquisition.
    //
    virtual  Acad::ErrorStatus getSubentPathsAtGsMarker(
				AcDb::SubentType type,
				int gsMark,
				const AcGePoint3d& pickPoint,
				const AcGeMatrix3d& viewXform,
				int& numPaths,
				AcDbFullSubentPath*& subentPaths,
				int numInserts = 0,
				AcDbObjectId* entAndInsertStack = NULL) const;

    virtual Acad::ErrorStatus getGsMarkersAtSubentPath(
				const AcDbFullSubentPath& subPath,
				AcDbIntArray& gsMarkers) const;


    virtual AcDbEntity*       subentPtr(const AcDbFullSubentPath& id) const;
 
    virtual Adesk::UInt32 numChanges() const;
};

MAKE_ACDBOPENOBJECT_FUNCTION(AcDbBody);

#endif
