#ifndef AD_DBCURVE_H
#define AD_DBCURVE_H
//
// $Id: dbcurve.h,v 1.22 1995/10/26 16:14:08 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION: API Abstract class for "Curve" entities.

#include "dbmain.h"
#include "gept3dar.h"
#include "gedblar.h"
#include "geplane.h"

class AcDbSpline;
class AcDbVoidPtrArray;

class AcDbCurve: public AcDbEntity
{
public:
    ACDB_DECLARE_MEMBERS(AcDbCurve);

    virtual ~AcDbCurve();

    virtual Adesk::Boolean    isClosed     () const;      
    virtual Adesk::Boolean    isPeriodic   () const;      
    virtual Adesk::Boolean    isPlanar     () const;

    virtual Acad::ErrorStatus getPlane     (AcGePlane&, AcDb::Planarity&) const; 
    virtual Acad::ErrorStatus getStartParam(double&) const;
    virtual Acad::ErrorStatus getEndParam  (double&) const;
    virtual Acad::ErrorStatus getStartPoint(AcGePoint3d&) const;
    virtual Acad::ErrorStatus getEndPoint  (AcGePoint3d&) const;


    virtual Acad::ErrorStatus getPointAtParam(double, AcGePoint3d&) const;
    virtual Acad::ErrorStatus getParamAtPoint(const AcGePoint3d&, double&)const;
    virtual Acad::ErrorStatus getDistAtParam (double param, double& dist) const;
    virtual Acad::ErrorStatus getParamAtDist (double dist, double& param) const;
    virtual Acad::ErrorStatus getDistAtPoint (const AcGePoint3d&, double&)const;
    virtual Acad::ErrorStatus getPointAtDist (double, AcGePoint3d&) const;

    virtual Acad::ErrorStatus getFirstDeriv  (double param,
				              AcGeVector3d& firstDeriv) const;
    virtual Acad::ErrorStatus getFirstDeriv  (const AcGePoint3d&,
					      AcGeVector3d& firstDeriv) const;
    virtual Acad::ErrorStatus getSecondDeriv (double param,
					      AcGeVector3d& secDeriv) const;
    virtual Acad::ErrorStatus getSecondDeriv (const AcGePoint3d&,
					      AcGeVector3d& secDeriv) const;

    virtual Acad::ErrorStatus getClosestPointTo(const AcGePoint3d& givenPnt,
						AcGePoint3d& pointOnCurve,
						Adesk::Boolean extend
						= Adesk::kFalse) const;
    virtual Acad::ErrorStatus getClosestPointTo(const AcGePoint3d& givenPnt,
						const AcGeVector3d& normal,
						AcGePoint3d& pointOnCurve,
						Adesk::Boolean extend
						= Adesk::kFalse) const;

    virtual Acad::ErrorStatus getOrthoProjectedCurve(const AcGePlane&,
					        AcDbCurve*& projCrv) const;
    virtual Acad::ErrorStatus getProjectedCurve(const AcGePlane&,
					        const AcGeVector3d& projDir,
						AcDbCurve*& projCrv) const;

    virtual Acad::ErrorStatus getOffsetCurves(double offsetDist,
					  AcDbVoidPtrArray& offsetCurves) const;
    virtual Acad::ErrorStatus getSpline      (AcDbSpline*& spline) const;
    virtual Acad::ErrorStatus getSplitCurves (const AcGeDoubleArray& params,
			                 AcDbVoidPtrArray& curveSegments) const;
    virtual Acad::ErrorStatus getSplitCurves (const AcGePoint3dArray& points,
					 AcDbVoidPtrArray& curveSegments) const;

    virtual Acad::ErrorStatus extend(double newParam);
    virtual Acad::ErrorStatus extend(Adesk::Boolean extendStart,
                                     const AcGePoint3d& toPoint);

    virtual Acad::ErrorStatus getArea(double&) const;

protected:
    AcDbCurve();

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbCurve);

// Protocols of AcDbCurve available in macro from.

#define DBCURVE_METHODS \
virtual Adesk::Boolean isClosed() const; \
virtual Adesk::Boolean isPeriodic() const; \
virtual Adesk::Boolean isPlanar() const; \
virtual Acad::ErrorStatus getPlane(AcGePlane&, AcDb::Planarity&) const; \
virtual Acad::ErrorStatus getStartParam(double&) const; \
virtual Acad::ErrorStatus getEndParam(double&) const; \
virtual Acad::ErrorStatus getStartPoint(AcGePoint3d&) const; \
virtual Acad::ErrorStatus getEndPoint(AcGePoint3d&) const; \
virtual Acad::ErrorStatus getPointAtParam(double, AcGePoint3d&) const; \
virtual Acad::ErrorStatus getParamAtPoint(const AcGePoint3d&, double&) const; \
virtual Acad::ErrorStatus getDistAtParam(double, double&) const; \
virtual Acad::ErrorStatus getParamAtDist(double, double&) const; \
virtual Acad::ErrorStatus getDistAtPoint(const AcGePoint3d&, double&) const; \
virtual Acad::ErrorStatus getPointAtDist(double, AcGePoint3d&) const; \
virtual Acad::ErrorStatus getFirstDeriv(double, AcGeVector3d&) const; \
virtual Acad::ErrorStatus getFirstDeriv(const AcGePoint3d&,AcGeVector3d&)const;\
virtual Acad::ErrorStatus getSecondDeriv(double, AcGeVector3d&) const; \
virtual Acad::ErrorStatus getSecondDeriv(const AcGePoint3d&, \
                              AcGeVector3d&) const; \
virtual Acad::ErrorStatus getClosestPointTo(const AcGePoint3d&, AcGePoint3d&, \
                              Adesk::Boolean = Adesk::kFalse) const; \
virtual Acad::ErrorStatus getClosestPointTo(const AcGePoint3d&, \
			      const AcGeVector3d&, AcGePoint3d&, \
			      Adesk::Boolean = Adesk::kFalse) const; \
virtual Acad::ErrorStatus getOrthoProjectedCurve(const AcGePlane&, \
                              AcDbCurve*&) const; \
virtual Acad::ErrorStatus getProjectedCurve(const AcGePlane&, \
                              const AcGeVector3d&, AcDbCurve*&) const; \
virtual Acad::ErrorStatus getOffsetCurves(double, AcDbVoidPtrArray&) const; \
virtual Acad::ErrorStatus getSpline(AcDbSpline*&) const; \
virtual Acad::ErrorStatus getSplitCurves(const AcGeDoubleArray&, \
			      AcDbVoidPtrArray& curveSegments) const; \
virtual Acad::ErrorStatus getSplitCurves(const AcGePoint3dArray&, \
			      AcDbVoidPtrArray&) const; \
virtual Acad::ErrorStatus extend(double); \
virtual Acad::ErrorStatus extend(Adesk::Boolean, const AcGePoint3d&); \
virtual Acad::ErrorStatus getArea(double&) const;

#endif
