#ifndef GEOMENT_DBELIPSE_H
#define GEOMENT_DBELIPSE_H
//
// $Id: dbelipse.h,v 1.14 1995/09/30 15:27:19 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION:
//
// The AcDbEllipse class represents both a full ellipse and an elliptical
// arc. The parametrization of the ellipse is the vector equation
//
//         P(O) = A * cos(O) + B * sin(O)
// 
// where A and B are the semi major and minor axes respectively.

#include "gegbl.h"
#include "dbmain.h"
#include "dbcurve.h"

class AcDbEllipse: public  AcDbCurve
{
public:
    AcDbEllipse();
    AcDbEllipse(const AcGePoint3d&  center,
	        const AcGeVector3d& unitNormal,
	        const AcGeVector3d& majorAxis,
	        double              radiusRatio,
	        double              startAngle = 0.0,
	        double              endAngle   = 6.28318530717958647692);
    virtual ~AcDbEllipse();
    ACDB_DECLARE_MEMBERS(AcDbEllipse);

    AcGePoint3d center() const;
    Acad::ErrorStatus setCenter(const AcGePoint3d& center);

    AcGeVector3d normal() const;
    AcGeVector3d majorAxis() const;
    AcGeVector3d minorAxis() const;

    double radiusRatio() const;
    Acad::ErrorStatus setRadiusRatio(double ratio);

    double startAngle() const;
    Acad::ErrorStatus setStartAngle(double startAngle);    

    double endAngle() const;
    Acad::ErrorStatus setEndAngle(double endAngle);

    Acad::ErrorStatus setStartParam(double startParam);
    Acad::ErrorStatus setEndParam(double endParam);

    double paramAtAngle(double angle) const;
    double angleAtParam(double param) const;

    Acad::ErrorStatus get(AcGePoint3d& center, AcGeVector3d& unitNormal,
	AcGeVector3d& majorAxis, double& radiusRatio, double& startAngle,
	double& endAngle) const;

    Acad::ErrorStatus set(const AcGePoint3d& center,
	const AcGeVector3d& unitNormal, const AcGeVector3d& majorAxis,
	double radiusRatio, double startAngle = 0.0,
	double endAngle = 6.28318530717958647692);

    Adesk::Boolean isNull() const;
				   
    DBCURVE_METHODS

    virtual Acad::ErrorStatus getGripPoints(AcGePoint3dArray&    gripPoints,
					    AcDbIntArray& osnapMasks,
					    AcDbIntArray& geomIds)
                                            const;

    virtual  Acad::ErrorStatus moveGripPointsAt(
	const AcDbIntArray& indices, const AcGeVector3d& offset);

    virtual  Acad::ErrorStatus intersectWith(
	const AcDbEntity* ent, AcDb::Intersect intType,
	AcGePoint3dArray& points, int thisGsMarker = 0,
	int otherGsMarker = 0) const;

    virtual  Acad::ErrorStatus intersectWith(
	const AcDbEntity* ent, AcDb::Intersect intType,
	const AcGePlane& projPlane, AcGePoint3dArray& points,
	int thisGsMarker = 0, int otherGsMarker = 0) const;

    virtual Acad::ErrorStatus getTransformedCopy(
	const AcGeMatrix3d& xform, AcDbEntity*& ent) const;

    virtual void list() const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbEllipse);

#endif
