#ifndef _AD_DBID_APPS_H
#define _AD_DBID_APPS_H 1
// 
// $Id: dbidapps.h,v 1.2 1995/11/13 10:20:49 t_lloyn Exp $
//
// (C) Copyright 1995 by Autodesk, Inc. 
// 
// The information contained herein is confidential, proprietary 
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California. 
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written 
// non-disclosure agreement,  expressly  prescribing  the  scope 
// and manner of such use. 
//
// CREATED BY:   Lynn Levy, July, 1995
//
// DESCRIPTION:
//
// This file contains the inlined versions of the comparsion operators
// for the AcDbObjectId class, that are for use by adjacent applications.
// These operators call out to explicit functions, to allow behavioral
// modification of these operators between c4 and c5.  Presumably, the
// AutoCAD/adjacent app division of these operators will be collapsed back
// into dbid.h at a future date.
// 
// See dbid.cc for the body of the function calls.
// See dbid.h for further description of the AcDbObjectId types.

#include "adesk.h"
#include "dbid.h"
#include "dbid_ops.h"


//////////////////// AcDbObjectId inlines ////////////////////

// LHS AcDbObjectId -- RHS AcDbObjectId

inline Adesk::Boolean
AcDbObjectId::operator < (const AcDbObjectId& id) const
{ return c4ObjIdIsLessThan(mId, id.mId); }

inline Adesk::Boolean
AcDbObjectId::operator > (const AcDbObjectId& id) const
{ return c4ObjIdIsGreaterThan(mId, id.mId); }

inline Adesk::Boolean
AcDbObjectId::operator <= (const AcDbObjectId& id) const
{ return c4ObjIdLessThanOrEqual(mId, id.mId); }

inline Adesk::Boolean
AcDbObjectId::operator >= (const AcDbObjectId& id) const
{ return c4ObjIdGreaterThanOrEqual(mId, id.mId); }

inline Adesk::Boolean
AcDbObjectId::operator == (const AcDbObjectId& id) const
{ return c4ObjIdIsEqualTo(mId, id.mId); }

inline Adesk::Boolean
AcDbObjectId::operator != (const AcDbObjectId& id) const
{ return c4ObjIdNotEqualTo(mId, id.mId); }



// LHS AcDbObjectId -- RHS AcDbStub*

inline Adesk::Boolean
AcDbObjectId::operator < (const AcDbStub* pStub) const
{ return c4ObjIdIsLessThan(mId, (unsigned long)(pStub)); }

inline Adesk::Boolean
AcDbObjectId::operator > (const AcDbStub* pStub) const
{ return c4ObjIdIsGreaterThan(mId, (unsigned long)(pStub)); }

inline Adesk::Boolean
AcDbObjectId::operator <= (const AcDbStub* pStub) const
{ return c4ObjIdLessThanOrEqual(mId, (unsigned long)(pStub)); }

inline Adesk::Boolean
AcDbObjectId::operator >= (const AcDbStub* pStub) const
{ return c4ObjIdGreaterThanOrEqual(mId, (unsigned long)(pStub)); }

inline Adesk::Boolean
AcDbObjectId::operator == (const AcDbStub* pStub) const
{ return c4ObjIdIsEqualTo(mId, (unsigned long)(pStub)); }

inline Adesk::Boolean
AcDbObjectId::operator != (const AcDbStub* pStub) const
{ return c4ObjIdNotEqualTo(mId, (unsigned long)(pStub)); }


#if defined(hpux) | defined(ibmrs)
inline Adesk::Boolean
AcDbObjectId::operator == (int n) const
{ return c4ObjIdIsEqualTo(mId, (unsigned long)(n)); }

inline Adesk::Boolean
AcDbObjectId::operator != (int n) const
{ return c4ObjIdNotEqualTo(mId, (unsigned long)(n)); }
#endif //hpux

#endif // _AD_DBID_APPS_H
