#ifndef _DBJIG_H_
#define _DBJIG_H_  1

/*  $Id: dbjig.h,v 1.8 1995/12/20 17:24:07 neb Exp $  */
/*
   (C) Copyright 1994-1995 by Autodesk, Inc.

   The information contained herein is confidential, proprietary
   to Autodesk,  Inc.,  and considered a trade secret as defined
   in section 499C of the penal code of the State of California.
   Use of  this information  by  anyone  other  than  authorized
   employees of Autodesk, Inc.  is granted  only under a written
   non-disclosure agreement,  expressly  prescribing  the  scope
   and manner of such use.

   DBJIG.H

   DESCRIPTION: Definition of class AcEdJig, an object onto which
                you clamp an AcDbEntity while bending it into shape.
                Comes with slots for a few standard dragging attachments
                and a built-in tool or two.

*/

#if defined(__cplusplus)
#ifndef _AD_ACDB_H
#include "acdb.h"
#endif

class	AcEdImpJig;
class	AcDbEntity;

class AcEdJig: public AcRxObject
{
public:
    ACRX_DECLARE_MEMBERS(AcEdJig);

    // SGI - warning(3455): declaration requires a typedef name
    typedef enum UserInputControls {
	kGovernedByOrthoMode		= 0x0001,
	kNullResponseAccepted		= 0x0002,
	kDontEchoCancelForCtrlC		= 0x0004,
	kDontUpdateLastPoint		= 0x0008,
	kNoDwgLimitsChecking		= 0x0010,
	kNoZeroResponseAccepted		= 0x0820,
	kNoNegativeResponseAccepted	= 0x0040,
	kAccept3dCoordinates		= 0x0080,
	kAcceptMouseUpAsPoint		= 0x0100,
	kAnyBlankTerminatesInput	= 0x0200,
	kInitialBlankTerminatesInput	= 0x0400
    } UserInputControls;

    typedef enum DragStatus {
        kNoChange       = -6,
        kCancel         = -4,
        kOther          = -3,
        kNull           = -1,
        kNormal         = 0,
	kKW1		= 1,
	kKW2		= 2,
	kKW3		= 3,
	kKW4		= 4,
	kKW5		= 5,
	kKW6		= 6,
	kKW7		= 7,
	kKW8		= 8,
	kKW9		= 9
    } DragStatus;

    typedef enum CursorType {
	kNoSpecialCursor = -1,           // -1 No Special Cursor Specified
	kCrosshair = 0,                  // 0 Full Screen Cross Hair. 
	kRectCursor,                     // 1 Rectangular cursor. 
	kRubberBand,                     // 2 Rubber band line. 
	kNotRotated,                     // 3 NotRotated Type. 
	kTargetBox,                      // 4 Target Box Type. 
	kRotatedCrosshair,               // 5 Rotated Crosshair w/ rubber band. 
	kCrosshairNoRotate,              // 6 Crosshairs forced non-rotated. 
	kInvisible,                      // 7 Invisible cursor. 
	kEntitySelect,                   // 8 Entity selection target cursor. 
	kArrow,                          // 9 Arrow. 
	kParallelogram,                  // 10 Parallelogram cursor. 
	kEntitySelectNoPersp,            // 11 Pickbox, suppressed in persp. 
	kPkfirstOrGrips                  // 12 Auto-select cursor. 
    } CursorType;
	    AcEdJig();
	    virtual ~AcEdJig();
	    DragStatus	drag();
    virtual DragStatus	sampler(); 
    virtual Adesk::Boolean  update();
	    AcDbObjectId append();

    const char * keywordList();
    void setKeywordList(const char *);

    const char * dispPrompt();
    void setDispPrompt(const char *);

    DragStatus acquireAngle(double &ang);
    DragStatus acquireAngle(double &ang, const AcGePoint3d &pt);
    DragStatus acquireDist(double &dist);
    DragStatus acquireDist(double &dist, const AcGePoint3d &pt);
    DragStatus acquirePoint(AcGePoint3d &bp);
    DragStatus acquirePoint(AcGePoint3d &bp, const AcGePoint3d &pt);

    AcEdJig::CursorType specialCursorType();
    void setSpecialCursorType(CursorType);

    AcEdJig::UserInputControls userInputControls();
    void setUserInputControls(AcEdJig::UserInputControls uic);

    virtual AcDbEntity* entity() const;

private:

    // Pointer to the imp object for this class
    AcEdImpJig		*mpImpJig;

};

#endif /* defined (__cplusplus) */
#endif /* _DBJIG_H_ */
