#ifndef AD_DBLEAD_H
#define AD_DBLEAD_H 1
//
//  $Id: dblead.h,v 1.27 1995/09/30 15:28:16 jamesr Exp $
//
//  (C) Copyright 1994-1995 by Autodesk, Inc.  All rights reserved.
//
//  The information contained herein is confidential, proprietary to 
//  Autodesk, Inc., and considered a trade secret as defined in 
//  section 499C of the penal code of the State of California.  
//  Use of this information by anyone other than authorized employees 
//  of Autodesk, Inc. is granted only under a written non-disclosure 
//  agreement, expressly prescribing the scope and manner of such use.
//
// CREATED BY: Katsuaki Takamizawa  March 28, 1994
// REVISED BY: James Rowell, September 1994.

#include "dbmain.h"
#include "dbcurve.h"
#include "gescl3d.h"
#include "gepnt2d.h"
#include "geplane.h"

class AcDbLeader: public  AcDbCurve
{
public:
    AcDbLeader();
    ~AcDbLeader();
    ACDB_DECLARE_MEMBERS(AcDbLeader);
    DBCURVE_METHODS

    // Leader orientation.
    //
    virtual void              setPlane          (const AcGePlane&);
    virtual AcGeVector3d      normal            () const;

    // Vertex manipulation.
    //
    virtual  int numVertices() const;
    virtual  Adesk::Boolean appendVertex(const AcGePoint3d&);
    virtual void              removeLastVertex  ();
    virtual  AcGePoint3d firstVertex() const;
    virtual  AcGePoint3d lastVertex() const;
    virtual  AcGePoint3d vertexAt(int) const;
    virtual  Adesk::Boolean setVertexAt(int, const AcGePoint3d&);

    // Display characteristics.
    //
    virtual Adesk::Boolean    hasArrowHead      () const;
    virtual void              enableArrowHead   ();
    virtual void              disableArrowHead  ();
    virtual Adesk::Boolean    hasHookLine       () const;
    //
    virtual void              setToSplineLeader ();
    virtual void              setToStraightLeader();
    virtual Adesk::Boolean    isSplined         () const;
    //
    virtual AcDbHardPointerId dimensionStyle    () const;
    virtual void              setDimensionStyle (const AcDbHardPointerId&);
    virtual void              setDimVars        ();
    virtual Acad::ErrorStatus setColorIndex     (Adesk::UInt16);

    // Annotation.
    //
    virtual Acad::ErrorStatus attachAnnotation  (const AcDbObjectId&);
    virtual Acad::ErrorStatus detachAnnotation  ();
    virtual AcDbObjectId      annotationObjId   () const;

    // Trim and extend support.
    //
    virtual  Acad::ErrorStatus intersectWith(const AcDbEntity*,
	AcDb::Intersect, AcGePoint3dArray&, int thisGsMarker = 0,
	int otherGsMarker = 0) const;
    virtual  Acad::ErrorStatus intersectWith(const AcDbEntity*,
	AcDb::Intersect, const AcGePlane& projPlane, AcGePoint3dArray&,
	int thisGsMarker = 0, int otherGsMarker = 0) const;

    // Support for persistant reactor to annotation.
    //
    virtual void              modified          (const AcDbObject*);
    virtual void              erased            (const AcDbObject*,
                                                 Adesk::Boolean = Adesk::kTrue);
    virtual void              goodbye           (const AcDbObject*);
    virtual void              copied            (const AcDbObject*,
                                                 const AcDbObject*);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbLeader);

#endif
