#ifndef AD_DBMLINE_H
#define AD_DBMLINE_H
//
//  $Id: dbmline.h,v 1.36 1995/09/30 15:28:38 jamesr Exp $
//
//  dbmline.h - multi-line entity
//
//  (C) Copyright 1994-1995 by Autodesk, Inc.  All rights reserved.
//
//  The information contained herein is confidential, proprietary to 
//  Autodesk, Inc., and considered a trade secret as defined in 
//  section 499C of the penal code of the State of California.  
//  Use of this information by anyone other than authorized employees 
//  of Autodesk, Inc. is granted only under a written non-disclosure 
//  agreement, expressly prescribing the scope and manner of such use.
//
// DESCRIPTION: Exported AcDbMline Entity class

#include "dbmain.h"

struct Mline
{
    typedef Adesk::Int8 MlineJustification;
    enum
    {
        kTop        = 0,
        kZero       = 1,
        kBottom     = 2
    };
    enum
    {
        kOpen         = 0,
        kClosed       = 1,
        kMerged       = 2
    };
};

// class AcGePlane; When the "To Be Removed.  JPR" lines are removed,
// uncomment this and delete the next file include of geplane.h
#include "geplane.h"

class AcDbMline: public  AcDbEntity
{
public:
    ACDB_DECLARE_MEMBERS(AcDbMline);
    AcDbMline();
    virtual ~AcDbMline();

    Acad::ErrorStatus setStyle(const AcDbObjectId &newStyleId);
    AcDbObjectId      style() const;
    Acad::ErrorStatus setJustification(Mline::MlineJustification newJust);
    Mline::MlineJustification justification() const;
    Acad::ErrorStatus setScale(double newScale);
    double scale() const;
    Acad::ErrorStatus setNormal(const AcGeVector3d & newNormal);
    Acad::ErrorStatus appendSeg(const AcGePoint3d& newVertex);
    Acad::ErrorStatus removeLastSeg(AcGePoint3d& lastVertex);
    Acad::ErrorStatus moveVertexAt(int index, const AcGePoint3d& newPosition);
    Acad::ErrorStatus setClosedMline(Adesk::Boolean closed);
    Adesk::Boolean    closedMline() const;
    void                setSupressStartCaps(Adesk::Boolean supressIt);
    Adesk::Boolean   supressStartCaps() const;
    void                setSupressEndCaps(Adesk::Boolean supressIt);
    Adesk::Boolean   supressEndCaps() const;
    int numVertices() const;
    int               element(const AcGePoint3d & pt) const;
    AcGePoint3d vertexAt(int index) const;
    void              list() const;
    Acad::ErrorStatus getClosestPointTo(const AcGePoint3d& givenPoint,
	AcGePoint3d& pointOnCurve, Adesk::Boolean extend,
	Adesk::Boolean excludeCaps = Adesk::kFalse) const;

    Acad::ErrorStatus getClosestPointTo(const AcGePoint3d& givenPoint,
	const AcGeVector3d& normal, AcGePoint3d& pointOnCurve,
	Adesk::Boolean extend,
	Adesk::Boolean excludeCaps = Adesk::kFalse) const;
    void getPlane(AcGePlane&) const;
    AcGeVector3d      normal            () const;
    Acad::ErrorStatus	getGeomExtents(AcDbExtents&) const;

    // Subentity acquisition.
    //
    Acad::ErrorStatus getSubentPathsAtGsMarker(AcDb::SubentType type,
	int gsMark, const AcGePoint3d& pickPoint,
	const AcGeMatrix3d& viewXform, int& numPaths,
	AcDbFullSubentPath* & subentPaths, int numInserts = 0,
	AcDbObjectId* entAndInsertStack = NULL) const;

    Acad::ErrorStatus getGsMarkersAtSubentPath(
	const AcDbFullSubentPath& subPath, AcDbIntArray& gsMarkers) const;

    AcDbEntity*       subentPtr(const AcDbFullSubentPath& id) const;
 
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbMline);

#endif
