#ifndef AD_DBOLE_H
#define AD_DBOLE_H
//
// $Id: dbole.h,v 1.29 1995/10/26 16:15:06 neb Exp $
//
// (C) Copyright 1994-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION:  Class header for AcDbOleFrame & AcDbOle2Frame, 
//               OLE 2 Container (Client) Feature for Windows/NT
//
//    AcDbEntity
//        AcDbOleFrame
//            AcDbOle2Frame  
//
//    AcDbOleFrame supports 1) non-OLE platform and 2) pre-R13 C4 OLE 1
//    storage and retrieval only.  It doesn't support OLE object
//    display or activation.  It stores
//    binary data in lists that can be scanned later using the DbId.
//    AcDbOle2Frame supports OLE 2 objects.  For non-OLE platforms,
//    the virtual methods are not overridden, so the storage methods
//    in AcDbOleFrame are used instead.  

#include "dbmain.h"
#include "gepnt3d.h"
#include "rect3d.h"


class AcDbOleFrame: public AcDbEntity
{
public:
    ACDB_DECLARE_MEMBERS(AcDbOleFrame);

    AcDbOleFrame();
    virtual ~AcDbOleFrame();

    // --- AcDbObject Protocol

    virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
    virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;
    virtual Adesk::Boolean worldDraw(AcGiWorldDraw*);
    virtual void viewportDraw(AcGiViewportDraw* mode);
    virtual Acad::ErrorStatus getGeomExtents(AcDbExtents& extents) const;

    // OLE Specific Methods
    virtual void* getOleObject(void) const;
    virtual void setOleObject(const void *pItem);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbOleFrame);
 

class AcDbOle2Frame: public AcDbOleFrame
{
public:
    ACDB_DECLARE_MEMBERS(AcDbOle2Frame);
    AcDbOle2Frame();
    virtual ~AcDbOle2Frame();

    // --- AcDbObject Protocol
#ifdef WIN32

    virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
    virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;

    virtual Adesk::Boolean worldDraw(AcGiWorldDraw*);
    virtual void viewportDraw(AcGiViewportDraw* mode);
    virtual Acad::ErrorStatus getGeomExtents(AcDbExtents& extents) const;

    // OLE Specific Methods
    // Get pointer to MFC 2.1 COleClientItem class.
    virtual void* getOleObject(void) const;
    virtual void setOleObject(const void *pItem);

    // Upper left corner of OLE object, world coordinates
    virtual void getLocation(AcGePoint3d& point3d) const;

    // Description, such as "Paintbrush Bitmap".
    // See MFC COleClientItem::GetUserType().
    virtual void getUserType(char* pszName, int nNameSize) const;

    // Linked, embedded, or static.  See MFC COleClientItem::GetType().
    virtual int getType(void) const;
#endif  // WIN32
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbOle2Frame);

#endif
