#ifndef GEOMENT_DBREGION_H
#define GEOMENT_DBREGION_H
//
// $Id: dbregion.h,v 1.25 1995/09/30 15:29:04 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION:
//
// The AcDbRegion class is the interface class for representing
// regions inside AutoCAD.  All the functionality supported by
// this class is implemented in the class AcDbImpRegion and its
// base classes.

#include "dbmain.h"
#include "dbsubeid.h"
#include "gepnt3d.h"
#include "gevec3d.h"

class AcDbRegion: public  AcDbEntity
{
public:
    AcDbRegion();
    virtual ~AcDbRegion();
    ACDB_DECLARE_MEMBERS(AcDbRegion);

    static  Acad::ErrorStatus createFromCurves(
	const AcDbVoidPtrArray& curveSegments, AcDbVoidPtrArray& regions);

    virtual Acad::ErrorStatus getPerimeter         (double&) const;

    virtual  Acad::ErrorStatus getArea(double& regionArea) const;

    virtual void*             body              () const;

    virtual Acad::ErrorStatus getAreaProp       (const AcGePoint3d&  origin,
						 const AcGeVector3d& xAxis,
						 const AcGeVector3d& yAxis,
						 double&      perimeter,
						 double&      area,
						 AcGePoint2d& centroid,
						 double       momInertia[2],
						 double&      prodInertia,
						 double       prinMoments[2],
						 AcGeVector2d prinAxes[2],
						 double       radiiGyration[2],
						 AcGePoint2d& extentsLow,
						 AcGePoint2d& extentsHigh)
						const;
    
    virtual Acad::ErrorStatus getPlane          (AcGePlane& regionPlane) const;

    virtual  Acad::ErrorStatus intersectWith(const AcDbEntity* ent,
	AcDb::Intersect intType, AcGePoint3dArray& points,
	int thisGsMarker, int otherGsMarker) const;

    virtual  Acad::ErrorStatus intersectWith(const AcDbEntity* ent,
	AcDb::Intersect intType, const AcGePlane& projPlane,
	AcGePoint3dArray& points, int thisGsMarker, int otherGsMarker) const;

    virtual Adesk::Boolean    isNull            () const;
    virtual void              list              () const;

    virtual Acad::ErrorStatus getNormal(AcGeVector3d&) const;

    virtual AcDbSubentId  internalSubentId      (void* ent) const;
    virtual void*         internalSubentPtr     (const AcDbSubentId& id) const;

    // Subentity acquisition.
    //
    virtual  Acad::ErrorStatus getSubentPathsAtGsMarker(AcDb::SubentType type,
	int gsMark, const AcGePoint3d& pickPoint,
	const AcGeMatrix3d& viewXform, int& numPaths,
	AcDbFullSubentPath* & subentPaths, int numInserts = 0,
	AcDbObjectId* entAndInsertStack = NULL) const;

    virtual Acad::ErrorStatus getGsMarkersAtSubentPath(
	const AcDbFullSubentPath& subPath, AcDbIntArray& gsMarkers) const;

    virtual AcDbEntity*       subentPtr(const AcDbFullSubentPath& id) const;
 
    virtual Acad::ErrorStatus booleanOper       (AcDb::BoolOperType operation,
						 AcDbRegion* otherRegion);
    virtual Acad::ErrorStatus setBody           (const void* modelerBody);

    virtual Adesk::UInt32 numChanges() const;

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbRegion);

#endif
