#ifndef AD_DBSYMTB_H
#define AD_DBSYMTB_H
//
// $Id: dbsymtb.h,v 1.36 1995/10/26 16:15:14 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION: Exported protocol for AutoCAD's built in symbol
//		tables, thier table  specific record types, and
//		thier iterators.

#include "dbmain.h"
#include "dbdict.h"
#include "dbdate.h"
#include "gevec2d.h"

#ifdef target
# undef target 
#endif

///// AcDbSymbolTable (and Subclasses)
//
//  Note that there is both overloading of functions within each
//  class defined by these macros and overriding of non virtual
//  functions between classes.  The overriding of these functions
//  is necessary to allow some member functions to take arguments
//  that are appropriate for a given table.
//
class AcDbSymbolTableRecord;
class AcDbSymbolTableIterator;
class AcDbSymbolTable: public AcDbObject
{
public:
    ACDB_DECLARE_MEMBERS(AcDbSymbolTable);
    AcDbSymbolTable();
    virtual ~AcDbSymbolTable();

    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbSymbolTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbSymbolTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbSymbolTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbSymbolTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbSymbolTable);

class AcDbBlockTableRecord;
class AcDbBlockTableIterator;
class AcDbBlockTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbBlockTable);
    AcDbBlockTable();
    virtual ~AcDbBlockTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbBlockTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbBlockTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbBlockTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbBlockTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbBlockTable);

class AcDbLayerTableRecord;
class AcDbLayerTableIterator;
class AcDbLayerTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbLayerTable);
    AcDbLayerTable();
    virtual ~AcDbLayerTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbLayerTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbLayerTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbLayerTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbLayerTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbLayerTable);

class AcDbTextStyleTableRecord;
class AcDbTextStyleTableIterator;
class AcDbTextStyleTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbTextStyleTable);
    AcDbTextStyleTable();
    virtual ~AcDbTextStyleTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbTextStyleTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbTextStyleTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbTextStyleTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbTextStyleTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbTextStyleTable);

class AcDbLinetypeTableRecord;
class AcDbLinetypeTableIterator;
class AcDbLinetypeTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbLinetypeTable);
    AcDbLinetypeTable();
    virtual ~AcDbLinetypeTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbLinetypeTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbLinetypeTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbLinetypeTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbLinetypeTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbLinetypeTable);

class AcDbAbstractViewTableRecord;
class AcDbAbstractViewTableIterator;
class AcDbAbstractViewTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbAbstractViewTable);
    AcDbAbstractViewTable();
    virtual ~AcDbAbstractViewTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbAbstractViewTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbAbstractViewTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbAbstractViewTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbAbstractViewTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbAbstractViewTable);

class AcDbViewTableRecord;
class AcDbViewTableIterator;
class AcDbViewTable: public AcDbAbstractViewTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbViewTable);
    AcDbViewTable();
    virtual ~AcDbViewTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbViewTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbViewTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbViewTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbViewTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbViewTable);

class AcDbUCSTableRecord;
class AcDbUCSTableIterator;
class AcDbUCSTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbUCSTable);
    AcDbUCSTable();
    virtual ~AcDbUCSTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbUCSTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbUCSTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbUCSTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbUCSTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbUCSTable);

class AcDbViewportTableRecord;
class AcDbViewportTableIterator;
class AcDbViewportTable: public AcDbAbstractViewTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbViewportTable);
    AcDbViewportTable();
    virtual ~AcDbViewportTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbViewportTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbViewportTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbViewportTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbViewportTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbViewportTable);

class AcDbRegAppTableRecord;
class AcDbRegAppTableIterator;
class AcDbRegAppTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbRegAppTable);
    AcDbRegAppTable();
    virtual ~AcDbRegAppTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbRegAppTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbRegAppTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbRegAppTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbRegAppTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbRegAppTable);

class AcDbDimStyleTableRecord;
class AcDbDimStyleTableIterator;
class AcDbDimStyleTable: public AcDbSymbolTable
{
public:
    ACDB_DECLARE_MEMBERS(AcDbDimStyleTable);
    AcDbDimStyleTable();
    virtual ~AcDbDimStyleTable();
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbDimStyleTableRecord*& pRecord,
                                 AcDb::OpenMode mode,
                                 Adesk::Boolean openErasedRecord
                                 = Adesk::kFalse) const;
    Acad::ErrorStatus   getAt   (const char* entryName,
                                 AcDbObjectId& recordId,
                                Adesk::Boolean getErasedRecord =
                                                Adesk::kFalse) const;
    Adesk::Boolean      has     (const char* name) const;
    Acad::ErrorStatus   newIterator
                                (AcDbDimStyleTableIterator*& pIterator,
                                 Adesk::Boolean atBeginning = Adesk::kTrue,
                                 Adesk::Boolean skipDeleted = Adesk::kTrue)
                                 const;
    Acad::ErrorStatus   add     (AcDbDimStyleTableRecord* record);
    Acad::ErrorStatus   add     (AcDbObjectId&  recordId,
                                 AcDbDimStyleTableRecord* record);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbDimStyleTable);

///// AcDbSymbolTableIterator
//
// This class is responsible for iterating over the records in a symbol
// table.
//
// This is a completely "generic" symbol table iterator in that it can 
// be used to iterate over the contents of any AcDbSymbolTable subclass.
// However, symbol table specific iterators are defined below.
//
// One major item of note.  The creation of one of these iterators
// requires the opening (for read) of the table being iterated over.
// This read stays in effect until the iterator is destroyed.
//
class AcDbSymbolTableIterator
{
public:
    virtual ~AcDbSymbolTableIterator();

    void 		start	(Adesk::Boolean atBeginning = Adesk::kTrue,
				 Adesk::Boolean skipDeleted = Adesk::kTrue);

    Adesk::Boolean	done	() const;


    Acad::ErrorStatus 	getRecordId
				(AcDbObjectId& id) const;

    Acad::ErrorStatus	getRecord(AcDbSymbolTableRecord*& pRecord,
				 AcDb::OpenMode mode,
				 Adesk::Boolean openErasedRecord =
						Adesk::kFalse) const;

    void	 	step    (Adesk::Boolean forward = Adesk::kTrue,
				 Adesk::Boolean skipDeleted = Adesk::kTrue);

    Acad::ErrorStatus   seek	(AcDbObjectId id);

    Acad::ErrorStatus   seek	(const AcDbSymbolTableRecord* record);
protected:
    void* mpImp;
    AcDbSymbolTableIterator();
    friend AcDbSymbolTable;
};

class AcDbBlockTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbBlockTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbBlockTableRecord*);

protected:
    AcDbBlockTableIterator();
    friend class AcDbBlockTable;
};

class AcDbLayerTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbLayerTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbLayerTableRecord*);

protected:
    AcDbLayerTableIterator();
    friend class AcDbLayerTable;
};

class AcDbTextStyleTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbTextStyleTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbTextStyleTableRecord*);

protected:
    AcDbTextStyleTableIterator();
    friend class AcDbTextStyleTable;
};

class AcDbLinetypeTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbLinetypeTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbLinetypeTableRecord*);

protected:
    AcDbLinetypeTableIterator();
    friend class AcDbLinetypeTable;
};

class AcDbAbstractViewTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbAbstractViewTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbAbstractViewTableRecord*);

protected:
    AcDbAbstractViewTableIterator();
    friend class AcDbAbstractViewTable;
};

class AcDbViewTableIterator : public AcDbAbstractViewTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbViewTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbViewTableRecord*);

protected:
    AcDbViewTableIterator();
    friend class AcDbViewTable;
};

class AcDbUCSTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbUCSTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbUCSTableRecord*);

protected:
    AcDbUCSTableIterator();
    friend class AcDbUCSTable;
};

class AcDbViewportTableIterator : public AcDbAbstractViewTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbViewportTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbViewportTableRecord*);

protected:
    AcDbViewportTableIterator();
    friend class AcDbViewportTable;
};

class AcDbRegAppTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbRegAppTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbRegAppTableRecord*);

protected:
    AcDbRegAppTableIterator();
    friend class AcDbRegAppTable;
};

class AcDbDimStyleTableIterator : public AcDbSymbolTableIterator
{
public:
    Acad::ErrorStatus getRecord(AcDbDimStyleTableRecord*&, AcDb::OpenMode,
			Adesk::Boolean openErasedRecord = Adesk::kFalse) const;
    Acad::ErrorStatus seek(AcDbObjectId);
    Acad::ErrorStatus seek(const AcDbDimStyleTableRecord*);

protected:
    AcDbDimStyleTableIterator();
    friend class AcDbDimStyleTable;
};

///// AcDbSymbolTableRecord

class AcDbSymbolTableRecord: public  AcDbObject
{
public:
    AcDbSymbolTableRecord();
    virtual ~AcDbSymbolTableRecord();

    ACDB_DECLARE_MEMBERS(AcDbSymbolTableRecord);

    Acad::ErrorStatus getName(char*& pName) const;
    Acad::ErrorStatus getName(const char*& pName) const;
    Acad::ErrorStatus setName(const char* pName);

    // !!! These are all exported via DXF
    // !!! is there any point in doing so here too ?

    Adesk::Boolean isReferenced() const;

    Adesk::Boolean isDependent() const;

    Adesk::Boolean isResolved() const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbSymbolTableRecord);


class AcDbBlockTableRecordIterator
{
public:
    virtual ~AcDbBlockTableRecordIterator();

    void 		start	(Adesk::Boolean atBeginning = Adesk::kTrue,
				 Adesk::Boolean skipDeleted = Adesk::kTrue);

    Adesk::Boolean	done	() const;


    Acad::ErrorStatus 	getEntityId
				(AcDbObjectId& entityId) const;

    Acad::ErrorStatus	getEntity(AcDbEntity*& pEntity, AcDb::OpenMode mode,
				 Adesk::Boolean openErasedEntity =
						Adesk::kFalse) const;

    AcDbBlockTableRecord* blockTableRecord() const;

    void	 	step
				(Adesk::Boolean forward = Adesk::kTrue,
				 Adesk::Boolean skipDeleted = Adesk::kTrue);

    Acad::ErrorStatus   seek	(AcDbObjectId id);

    Acad::ErrorStatus   seek	(const AcDbEntity* entity);

protected:
    class AcDbImpBlockTableRecordIterator* mpImp;

private:
    AcDbBlockTableRecordIterator();
    friend class AcDbImpBlockTableRecord;
};


class AcDbBlockTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbBlockTableRecord);

    AcDbBlockTableRecord();
    virtual ~AcDbBlockTableRecord();

    Acad::ErrorStatus appendAcDbEntity(AcDbEntity* entity);

    Acad::ErrorStatus appendAcDbEntity(
        AcDbObjectId& pOutputId,       // OUTPUT
        AcDbEntity* entity);           // input

    Acad::ErrorStatus newIterator(
	AcDbBlockTableRecordIterator*& pIterator,	// OUTPUT
	Adesk::Boolean atBeginning = Adesk::kTrue,	// input
	Adesk::Boolean skipDeleted = Adesk::kTrue);	// input

    Acad::ErrorStatus pathName(char*& pPath) const;
    void              setPathName(const char* pathStr);
	 
    AcGePoint3d       origin() const;
    void              setOrigin(const AcGePoint3d& pt);

    Acad::ErrorStatus openBlockBegin(AcDbBlockBegin*& pBlockBegin,
				     AcDb::OpenMode   openMode);
 
    Acad::ErrorStatus openBlockEnd(AcDbBlockEnd*& pBlockEnd,
				   AcDb::OpenMode openMode);

    Adesk::Boolean    hasAttributeDefinitions() const;

    Adesk::Boolean    isAnonymous() const;

    Adesk::Boolean    isFromExternalReference() const;

    Adesk::Boolean    isFromOverlayReference() const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbBlockTableRecord);


// AcDbLayerTableRecord:

class AcDbLayerTableRecord: public  AcDbSymbolTableRecord
{

public:
    ACDB_DECLARE_MEMBERS(AcDbLayerTableRecord);

    AcDbLayerTableRecord();
    virtual ~AcDbLayerTableRecord();

    Adesk::Boolean isFrozen() const;
    void setIsFrozen(Adesk::Boolean);

    Adesk::Boolean isOff() const;
    void setIsOff(Adesk::Boolean);

    Adesk::Boolean VPDFLT() const;
    void setVPDFLT(Adesk::Boolean);

    Adesk::Boolean isLocked() const;
    void setIsLocked(Adesk::Boolean);

    AcCmColor color() const;
    void setColor(const AcCmColor &color);

    AcDbObjectId linetypeObjectId() const;
    void setLinetypeObjectId(AcDbObjectId);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbLayerTableRecord);

// Utility for changing Layer Table Records in the Current Drawing
Acad::ErrorStatus applyCurDwgLayerTableChanges();

// AcDbTextStyleTableRecord:

class AcDbTextStyleTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbTextStyleTableRecord);

    AcDbTextStyleTableRecord();
    virtual ~AcDbTextStyleTableRecord();

    Adesk::Boolean isShapeFile() const;
    void setIsShapeFile(Adesk::Boolean);

    Adesk::Boolean isVertical() const;
    void setIsVertical(Adesk::Boolean);

    double textSize() const;
    void setTextSize(double size);

    double xScale() const;
    void setXScale(double xScale);

    double obliquingAngle() const;
    void setObliquingAngle(double obliquingAngle);

    Adesk::UInt8 flagBits() const;
    void setFlagBits(Adesk::UInt8 flagBits);

    double priorSize() const;
    void setPriorSize(double priorSize);

    // !!! Should we export these "set" functions ?

    Acad::ErrorStatus fileName(char*& pName) const;
    Acad::ErrorStatus fileName(const char*& pName) const;
    Acad::ErrorStatus setFileName(const char* pName);

    Acad::ErrorStatus bigFontFileName(char*& pName) const;
    Acad::ErrorStatus bigFontFileName(const char*& pName) const;
    Acad::ErrorStatus setBigFontFileName(const char* pName);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbTextStyleTableRecord);

// AcDbLinetypeTableRecord:

class AcDbLinetypeTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbLinetypeTableRecord);

    AcDbLinetypeTableRecord();
    virtual ~AcDbLinetypeTableRecord();

    Acad::ErrorStatus asciiDescription(char*& pString) const;
    Acad::ErrorStatus asciiDescription(const char*& pString) const;
    Acad::ErrorStatus setAsciiDescription(const char* pString);

    double patternLength() const;
    void setPatternLength(double patternLength);

    int numDashes() const;
    void setNumDashes(int count);

    double dashLengthAt(int index) const;
    void setDashLengthAt(int index, double value);

    AcDbObjectId shapeStyleAt(int index) const;
    void setShapeStyleAt(int index, AcDbObjectId value);

    int shapeNumberAt(int index) const;
    void setShapeNumberAt(int index, int value);

    AcGeVector2d shapeOffsetAt(int index) const;
    void setShapeOffsetAt(int index, const AcGeVector2d& value);

    double shapeScaleAt(int index) const;
    void setShapeScaleAt(int index, double value);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbLinetypeTableRecord);


///// AcDbAbstractViewTableRecord:

class AcDbAbstractViewTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbAbstractViewTableRecord);

    AcDbAbstractViewTableRecord();
    virtual ~AcDbAbstractViewTableRecord();

    AcGePoint2d centerPoint() const;
    void setCenterPoint(const AcGePoint2d& center);

    double height() const;
    void setHeight(double height);

    double width() const;
    void setWidth(double width);

    AcGePoint3d target() const;
    void setTarget(const AcGePoint3d& target);

    AcGeVector3d viewDirection() const;
    void setViewDirection(const AcGeVector3d& viewDirection);

    double viewTwist() const;
    void setViewTwist(double angle);

    double lensLength() const;
    void setLensLength(double length);

    double frontClipDistance() const;
    void setFrontClipDistance(double distance);

    double backClipDistance() const;
    void setBackClipDistance(double distance);

    Adesk::Boolean perspectiveEnabled() const;
    void setPerspectiveEnabled(Adesk::Boolean);

    Adesk::Boolean frontClipEnabled() const;
    void setFrontClipEnabled(Adesk::Boolean);

    Adesk::Boolean backClipEnabled() const;
    void setBackClipEnabled(Adesk::Boolean);

    Adesk::Boolean frontClipAtEye() const;
    void setFrontClipAtEye(Adesk::Boolean);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbAbstractViewTableRecord);


///// AcDbViewTableRecord:

class AcDbViewTableRecord: public  AcDbAbstractViewTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbViewTableRecord);

    AcDbViewTableRecord();
    virtual ~AcDbViewTableRecord();

    Adesk::Boolean isPaperspaceView() const;
    void setIsPaperspaceView(Adesk::Boolean);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbViewTableRecord);


///// AcDbUCSTableRecord:

class AcDbUCSTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbUCSTableRecord);

    AcDbUCSTableRecord();
    virtual ~AcDbUCSTableRecord();

    AcGePoint3d origin() const;
    void setOrigin(const AcGePoint3d& origin);

    AcGeVector3d xAxis() const;
    void setXAxis(const AcGeVector3d& Xaxis);

    AcGeVector3d yAxis() const;
    void setYAxis(const AcGeVector3d& Yaxis);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbUCSTableRecord);


///// AcDbViewportTableRecord:

class AcDbViewportTableRecord: public  AcDbAbstractViewTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbViewportTableRecord);

    AcDbViewportTableRecord();
    virtual ~AcDbViewportTableRecord();

    AcGePoint2d lowerLeftCorner() const;
    void setLowerLeftCorner(const AcGePoint2d& pt);

    AcGePoint2d upperRightCorner() const;
    void setUpperRightCorner(const AcGePoint2d& pt);

    Adesk::Boolean ucsFollowMode() const;
    void setUcsFollowMode(Adesk::Boolean newVal);

    Adesk::UInt16 circleSides() const;
    void setCircleSides(Adesk::UInt16 circleSizes);

    Adesk::Boolean fastZoomsEnabled() const;
    void setFastZoomsEnabled(Adesk::Boolean);

    Adesk::Boolean iconEnabled() const;
    void setIconEnabled(Adesk::Boolean);

    Adesk::Boolean iconAtOrigin() const;
    void setIconAtOrigin(Adesk::Boolean);

    Adesk::Boolean gridEnabled() const;
    void setGridEnabled(Adesk::Boolean);

    AcGePoint2d gridIncrements() const;
    void setGridIncrements(const AcGePoint2d& base);

    Adesk::Boolean snapEnabled() const;
    void setSnapEnabled(Adesk::Boolean);

    Adesk::Boolean isometricSnapEnabled() const;
    void setIsometricSnapEnabled(Adesk::Boolean);

    Adesk::Int16 snapPair() const;
    void setSnapPair(Adesk::Int16);

    double snapAngle() const;
    void setSnapAngle(double angle);

    AcGePoint2d snapBase() const;
    void setSnapBase(const AcGePoint2d& base);

    AcGePoint2d snapIncrements() const;
    void setSnapIncrements(const AcGePoint2d& base);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbViewportTableRecord);


///// AcDbRegAppTableRecord:

class AcDbRegAppTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbRegAppTableRecord);

    AcDbRegAppTableRecord();
    virtual ~AcDbRegAppTableRecord();

    Adesk::Boolean excludeFromSaveAsR12() const;
    void setExcludeFromSaveAsR12(Adesk::Boolean);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbRegAppTableRecord);


///// AcDbDimStyleTableRecord:

class AcDbDimStyleTableRecord: public  AcDbSymbolTableRecord
{
public:
    ACDB_DECLARE_MEMBERS(AcDbDimStyleTableRecord);

    AcDbDimStyleTableRecord();
    virtual ~AcDbDimStyleTableRecord();

    // Booleans:

    // !!! Is the anonymous bit worth exporting ?

    Adesk::Boolean isAnonymous() const;
    void setIsAnonymous(Adesk::Boolean);

    // Access functions

    Adesk::Boolean dimtol() const;
    void setDimtol(Adesk::Boolean);

    Adesk::Boolean dimlim() const;
    void setDimlim(Adesk::Boolean);

    Adesk::Boolean dimtih() const;
    void setDimtih(Adesk::Boolean);

    Adesk::Boolean dimtoh() const;
    void setDimtoh(Adesk::Boolean);

    Adesk::Boolean dimse1() const;
    void setDimse1(Adesk::Boolean);

    Adesk::Boolean dimse2() const;
    void setDimse2(Adesk::Boolean);

    Adesk::Boolean dimalt() const;
    void setDimalt(Adesk::Boolean);

    Adesk::Boolean dimtofl() const;
    void setDimtofl(Adesk::Boolean);

    Adesk::Boolean dimsah() const;
    void setDimsah(Adesk::Boolean);

    Adesk::Boolean dimtix() const;
    void setDimtix(Adesk::Boolean);

    Adesk::Boolean dimsoxd() const;
    void setDimsoxd(Adesk::Boolean);

    Adesk::Boolean dimsd1() const;
    void setDimsd1(Adesk::Boolean);

    Adesk::Boolean dimsd2() const;
    void setDimsd2(Adesk::Boolean);

    Adesk::Int8  dimtolj() const;
    void setDimtolj(Adesk::Int8);

    Adesk::Boolean dimupt() const;
    void setDimupt(Adesk::Boolean);

    int dimaltd() const;
    void setDimaltd(int);

    int dimzin() const;
    void setDimzin(int);

    int dimjust() const;
    void setDimjust(int);

    int dimfit() const;
    void setDimfit(int);

    int dimtzin() const;
    void setDimtzin(int);

    int dimaltz() const;
    void setDimaltz(int);

    int dimalttz() const;
    void setDimalttz(int);

    int dimtad() const;
    void setDimtad(int);

    int dimunit() const;
    void setDimunit(int);

    int dimaunit() const;
    void setDimaunit(int);

    int dimdec() const;
    void setDimdec(int);

    int dimtdec() const;
    void setDimtdec(int);

    int dimaltu() const;
    void setDimaltu(int);

    int dimalttd() const;
    void setDimalttd(int);

    AcDbObjectId dimtxsty() const;
    void setDimtxsty(AcDbObjectId);

    double dimscale() const;
    void setDimscale(double);

    double dimasz() const;
    void setDimasz(double);

    double dimexo() const;
    void setDimexo(double);

    double dimdli() const;
    void setDimdli(double);

    double dimexe() const;
    void setDimexe(double);

    double dimrnd() const;
    void setDimrnd(double);

    double dimdle() const;
    void setDimdle(double);

    double dimtp() const;
    void setDimtp(double);

    double dimtm() const;
    void setDimtm(double);

    double dimtxt() const;
    void setDimtxt(double);

    double dimcen() const;
    void setDimcen(double);

    double dimtsz() const;
    void setDimtsz(double);

    double dimaltf() const;
    void setDimaltf(double);

    double dimlfac() const;
    void setDimlfac(double);

    double dimtvp() const;
    void setDimtvp(double);

    double dimtfac() const;
    void setDimtfac(double);

    double dimgap() const;
    void setDimgap(double);

    Acad::ErrorStatus getDimpost(char* & pOutput) const;
    Acad::ErrorStatus setDimpost(const char*);

    Acad::ErrorStatus getDimapost(char* & pOutput) const;
    Acad::ErrorStatus setDimapost(const char*);

    Acad::ErrorStatus getDimblk(char* & pOutput) const;
    Acad::ErrorStatus setDimblk(const char*);

    Acad::ErrorStatus getDimblk1(char* & pOutput) const;
    Acad::ErrorStatus setDimblk1(const char*);

    Acad::ErrorStatus getDimblk2(char* & pOutput) const;
    Acad::ErrorStatus setDimblk2(const char*);

    AcCmColor dimclrd() const;
    void setDimclrd(const AcCmColor&);

    AcCmColor dimclre() const;
    void setDimclre(const AcCmColor&);

    AcCmColor dimclrt() const;
    void setDimclrt(const AcCmColor&);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbDimStyleTableRecord);

#endif
