#ifndef AC_GEARC2D_H
#define AC_GEARC2D_H
//
// $Id: gearc2d.h,v 1.2 1995/09/21 11:20:17 jamesr Exp $
//
// (C) Copyright 1993,1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeCircArc2d - A mathematical entity
// used to represent a circular arc in 2-space.

#include "gecurv2d.h"
#include "gepnt2d.h"
#include "gevec2d.h"

class AcGeLine2d;
class AcGeLinearEnt2d;

class AcGeCircArc2d : public AcGeCurve2d
{
public:
    AcGeCircArc2d();
    AcGeCircArc2d(const AcGeCircArc2d&);
    AcGeCircArc2d(const AcGePoint2d& cent, double radius);
    AcGeCircArc2d(const AcGePoint2d& cent, double radius,
                  double startAngle, double endAngle,
                  const AcGeVector2d& refVec = AcGeVector2d::kXAxis,
                  Adesk::Boolean isClockWise = Adesk::kFalse);
    AcGeCircArc2d(const AcGePoint2d&, const AcGePoint2d&, const AcGePoint2d&);
    AcGeCircArc2d(const AcGePoint2d&, const AcGePoint2d&, double bulge);

    // Intersection with other geometric objects.
    //
    Adesk::Boolean intersectWith  (const AcGeLinearEnt2d&, int& intn,
                                   AcGePoint2d& p1, AcGePoint2d& p2,
                                   const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean intersectWith  (const AcGeCircArc2d&, int& intn,
                                   AcGePoint2d& p1, AcGePoint2d& p2,
                                   const AcGeTol& = AcGeContext::gTol) const;

    // Tangent line to the circular arc at given point.
    //
    Adesk::Boolean tangent        (const AcGePoint2d&, AcGeLine2d&,
                                   const AcGeTol& = AcGeContext::gTol) const;

    // Test if point is inside circle.
    //
    Adesk::Boolean isInside       (const AcGePoint2d&,
                                   const AcGeTol& = AcGeContext::gTol) const;

    // Definition of circular arc
    //
    AcGePoint2d    center         () const;
    double         radius         () const;
    double         startAng       () const;
    double         endAng         () const;
    Adesk::Boolean isClockWise    () const;
    AcGeVector2d   refVec         () const;
    AcGePoint2d    startPoint     () const;
    AcGePoint2d    endPoint       () const;

    AcGeCircArc2d& setCenter      (const AcGePoint2d& cent);
    AcGeCircArc2d& setRadius      (double radius);
    AcGeCircArc2d& setAngles      (double startAng, double endAng);
    AcGeCircArc2d& setToComplement();
    AcGeCircArc2d& setRefVec      (const AcGeVector2d&);
    AcGeCircArc2d& set            (const AcGePoint2d& cent, double radius);
    AcGeCircArc2d& set            (const AcGePoint2d& cent, double radius,
                                   double ang1, double ang2,
                                   const AcGeVector2d& refVec = 
                                   AcGeVector2d::kXAxis,
                                   Adesk::Boolean isClockWise = Adesk::kFalse);
    AcGeCircArc2d& set            (const AcGePoint2d&, const AcGePoint2d&,
                                   const AcGePoint2d&);
    AcGeCircArc2d& set            (const AcGePoint2d&, const AcGePoint2d&,
                                   double bulge);

    // Assignment operator.
    //
    AcGeCircArc2d& operator =     (const AcGeCircArc2d&);

protected:
    friend class AcGeImpCircArc2d;

    // Construct object from its corresponding implementation object.
    AcGeCircArc2d (AcGeImpCircArc2d&, int);
    AcGeCircArc2d (AcGeImpCircArc2d*);
};

#endif
