#ifndef AC_GEARC3D_H
#define AC_GEARC3D_H
//
// $Id: gearc3d.h,v 1.5 1995/12/20 15:16:35 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeCircArc3d - A mathematical entity
// used to represent a circular arc in 3-space.

#include "gecurv3d.h"
#include "gevec3d.h"
#include "gepnt3d.h"
#include "geplane.h"

class AcGeLine3d;
class AcGeCircArc2d;
class AcGePlanarEnt;

class AcGeCircArc3d : public AcGeCurve3d
{
public:
    AcGeCircArc3d();
    AcGeCircArc3d(const AcGeCircArc3d&);
    AcGeCircArc3d(const AcGePoint3d& cent, 
                  const AcGeVector3d& nrm, double radius);
    AcGeCircArc3d(const AcGePoint3d& cent, const AcGeVector3d& nrm, 
                  const AcGeVector3d& refVec, double radius, 
                  double ang1, double ang2);
    AcGeCircArc3d(const AcGePoint3d&, const AcGePoint3d&, const AcGePoint3d&);
                                            
    // Return the point on this object that is closest to the other object.
    //
    AcGePoint3d    closestPointToPlane (const AcGePlanarEnt& plane,
                                        AcGePoint3d& pointOnPlane,
                                        const AcGeTol&
                                        = AcGeContext::gTol) const; 

    // Intersection with other geometric objects.
    //
    Adesk::Boolean intersectWith  (const AcGeLinearEnt3d&, int& intn,
                                   AcGePoint3d& p1, AcGePoint3d& p2,
                                   const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean intersectWith  (const AcGeCircArc3d&, int& intn,
                                   AcGePoint3d& p1, AcGePoint3d& p2,
                                   const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean  intersectWith (const AcGePlanarEnt&, int& numOfIntersect,
                                   AcGePoint3d& p1, AcGePoint3d& p2,
                                   const AcGeTol& = AcGeContext::gTol) const;
    
    // Projection-intersection with other geometric objects.
    Adesk::Boolean projIntersectWith (const AcGeLinearEnt3d&,
                                      const AcGeVector3d& projDir, int &numInt,
                                      AcGePoint3d& pntOnArc1,
                                      AcGePoint3d& pntOnArc2,
                                      AcGePoint3d& pntOnLine1,
                                      AcGePoint3d& pntOnLine2,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Tangent to the circular arc.
    //
    Adesk::Boolean tangent        (const AcGePoint3d&, AcGeLine3d&,
                                   const AcGeTol& = AcGeContext::gTol) const;

    // Plane of the arc
    //
    void           getPlane       (AcGePlane&) const;

    // Test if point is inside circle.
    //
    Adesk::Boolean isInside       (const AcGePoint3d&,
                                   const AcGeTol& = AcGeContext::gTol) const;

    // Definition of circular arc 
    //
    AcGePoint3d    center         () const;
    AcGeVector3d   normal         () const;
    AcGeVector3d   refVec         () const;
    double         radius         () const;
    double         startAng       () const;
    double         endAng         () const;
    AcGePoint3d    startPoint     () const;
    AcGePoint3d    endPoint       () const;

    AcGeCircArc3d& setCenter      (const AcGePoint3d&);
    AcGeCircArc3d& setAxes        (const AcGeVector3d& normal,
                                   const AcGeVector3d& refVec);
    AcGeCircArc3d& setRadius      (double);
    AcGeCircArc3d& setAngles      (double startAng, double endAng);

    AcGeCircArc3d& set            (const AcGePoint3d& cent, 
                                   const AcGeVector3d& nrm, double radius);
    AcGeCircArc3d& set            (const AcGePoint3d& cent, 
                                   const AcGeVector3d& nrm,
                                   const AcGeVector3d& refVec, double radius, 
                                   double ang1, double ang2);
    AcGeCircArc3d& set            (const AcGePoint3d&, const AcGePoint3d&, 
                                   const AcGePoint3d&);

    // Assignment operator.
    //
    AcGeCircArc3d& operator =     (const AcGeCircArc3d&);

protected:

    friend class AcGeImpCircArc3d;

    // Construct object from its corresponding implementation object.
    AcGeCircArc3d (AcGeImpCircArc3d&, int);
    AcGeCircArc3d (AcGeImpCircArc3d*);

};

#endif
