#ifndef AC_GECSPL2D_H
#define AC_GECSPL2D_H
     
/* $Id: gecspl2d.h,v 1.2 1995/10/26 16:33:03 neb Exp $
 */
     
// (C) Copyright 1993-1995 by Autodesk, Inc. 
//
// The information contained herein is confidential, proprietary  to 
// Autodesk,  Inc., and considered a trade secret as defined in sec- 
// tion 499C of the penal code of the State of California.   Use  of 
// this information by anyone other than authorized employees of Au- 
// todesk, Inc. is  granted  only  under  a  written  non-disclosure 
// agreement,  expressly  prescribing  the  scope and manner of such 
// use.
//     
// Description:
//
// This file contains the class AcGeCubicSplineCurve2d - A mathematic entity used to 
// represent a 2d interpolation cubic-spline curve.
//
//
class AcGePointOnCurve2d;
class AcGeCurve2dIntersection;
class AcGeInterval;
class AcGePlane;
    
#include "gesent2d.h"
#include "gept2dar.h"
#include "gevc2dar.h"
#include "gevec2d.h"
#include "gekvec.h"

class AcGeCubicSplineCurve2d : public AcGeSplineEnt2d
{
public:
     
    AcGeCubicSplineCurve2d      ();
    AcGeCubicSplineCurve2d      ( const AcGeCubicSplineCurve2d& );

    // Construct a periodic cubic spline curve
    // Contract: the first fit point must be equal to the last fit point
    //
    AcGeCubicSplineCurve2d      ( const AcGePoint2dArray& fitPnts, 
			      	              const AcGeTol& = 
			      	                    AcGeContext::gTol );

    // Construct a cubic spline curve with clamped end condition
    //
	AcGeCubicSplineCurve2d      ( const AcGePoint2dArray& fitPnts,
			      	              const AcGeVector2d& startDeriv,
			      	              const AcGeVector2d& endDeriv,
                                  const AcGeTol& = 
                                        AcGeContext::gTol );
    
    // Construct a cubic spline approximating the curve
    //
    AcGeCubicSplineCurve2d      (const AcGeCurve2d& curve,
                                 double epsilon = AcGeContext::gTol.equalPoint());

    // Construct a cubic spline curve with given fit points and 1st derivatives
    //
	AcGeCubicSplineCurve2d      ( const AcGeKnotVector& knots,
	                              const AcGePoint2dArray& fitPnts,
			      	              const AcGeVector2dArray& firstDerivs,
	                              Adesk::Boolean isPeriodic = Adesk::kFalse );
    
    // Definition of spline
    //

    int numFitPoints() const;

    AcGePoint2d fitPointAt(int idx) const;
    AcGeCubicSplineCurve2d& setFitPointAt(int idx, const AcGePoint2d&);
    AcGeVector2d firstDerivAt(int idx) const;
    AcGeCubicSplineCurve2d& setFirstDerivAt(int idx, const AcGeVector2d&);

    // Assignment operator.
    //
    AcGeCubicSplineCurve2d&  operator = (const AcGeCubicSplineCurve2d&);

protected:

    friend class AcGeImpCubicSplineCurve2d;

    // Construct object from its corresponding implementation object.
    AcGeCubicSplineCurve2d (AcGeImpCubicSplineCurve2d&, int);
    AcGeCubicSplineCurve2d (AcGeImpCubicSplineCurve2d*);
};
     
#endif
