#ifndef AC_GECSPL3D_H
#define AC_GECSPL3D_H
     
/* $Id: gecspl3d.h,v 1.3 1995/10/26 16:33:10 neb Exp $
 */
     
// (C) Copyright 1993-1995 by Autodesk, Inc. 
//
// The information contained herein is confidential, proprietary  to 
// Autodesk,  Inc., and considered a trade secret as defined in sec- 
// tion 499C of the penal code of the State of California.   Use  of 
// this information by anyone other than authorized employees of Au- 
// todesk, Inc. is  granted  only  under  a  written  non-disclosure 
// agreement,  expressly  prescribing  the  scope and manner of such 
// use.
//     
// Description:
//
// This file contains the class AcGeCubicSplineCurve3d - A mathematic entity used to 
// represent a 3d interpolation cubic-spline curve.
//
//
class AcGePointOnCurve3d;
class AcGeCurve3dIntersection;
class AcGeInterval;
class AcGePlane;
    
#include "gesent3d.h"
#include "gept3dar.h"
#include "gevc3dar.h"
#include "gevec3d.h"
#include "gekvec.h"

class AcGeCubicSplineCurve3d : public AcGeSplineEnt3d
{
public:
     
    AcGeCubicSplineCurve3d      ();
    AcGeCubicSplineCurve3d      ( const AcGeCubicSplineCurve3d& );

    // Construct a periodic cubic spline curve
    // Contract: the first fit point must be equal to the last fit point
    //
    AcGeCubicSplineCurve3d      ( const AcGePoint3dArray& fitPnts, 
			      	              const AcGeTol& = 
			      	                    AcGeContext::gTol );

    // Construct a cubic spline curve with clamped end condition
    //
	AcGeCubicSplineCurve3d      ( const AcGePoint3dArray& fitPnts,
			      	              const AcGeVector3d& startDeriv,
			      	              const AcGeVector3d& endDeriv,
                                  const AcGeTol& = 
                                        AcGeContext::gTol );
    
    // Construct a cubic spline approximating the curve
    //
    AcGeCubicSplineCurve3d      (const AcGeCurve3d& curve,
                                 double epsilon = AcGeContext::gTol.equalPoint());

    // Construct a cubic spline curve with given fit points and 1st derivatives
    //
	AcGeCubicSplineCurve3d      ( const AcGeKnotVector& knots,
	                              const AcGePoint3dArray& fitPnts,
			      	              const AcGeVector3dArray& firstDerivs,
	                              Adesk::Boolean isPeriodic = Adesk::kFalse );
    // Definition of spline
    //

    int numFitPoints() const;

    AcGePoint3d fitPointAt(int idx) const;
    AcGeCubicSplineCurve3d& setFitPointAt(int idx, const AcGePoint3d&);
    AcGeVector3d firstDerivAt(int idx) const;
    AcGeCubicSplineCurve3d& setFirstDerivAt(int idx, const AcGeVector3d&);

    // Assignment operator.
    //
    AcGeCubicSplineCurve3d&  operator = (const AcGeCubicSplineCurve3d&);

protected:

    friend class AcGeImpCubicSplineCurve3d;

    // Construct object from its corresponding implementation object.
    AcGeCubicSplineCurve3d (AcGeImpCubicSplineCurve3d&, int);
    AcGeCubicSplineCurve3d (AcGeImpCubicSplineCurve3d*);
};
     
#endif
