#ifndef AC_GECURV3D_H
#define AC_GECURV3D_H
//
// $Id: gecurv3d.h,v 1.8 1995/12/20 15:16:48 neb Exp $
//
// (C) Copyright 1993,1995 by Autodesk, Inc. 
//
// The information contained herein is confidential, proprietary  to 
// Autodesk,  Inc., and considered a trade secret as defined in sec- 
// tion 499C of the penal code of the State of California.   Use  of 
// this information by anyone other than authorized employees of Au- 
// todesk, Inc. is  granted  only  under  a  written  non-disclosure 
// agreement,  expressly  prescribing  the  scope and manner of such 
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeCurve3d - An abstract base class 
// from which all 3d curves are derived.
     
#include "adesk.h"
#include "geent3d.h"
#include "geponc3d.h"
     
class AcGeCurve2d;
class AcGeSurface;
class AcGePoint3d;
class AcGePlane;
class AcGeVector3d;
class AcGeLinearEnt3d;
class AcGeLine3d;
class AcGePointOnCurve3d;
class AcGePointOnSurface;
class AcGeInterval;
class AcGeMatrix3d;
class AcGePointOnCurve3dData;
class AcGeVector3dArray;
     
class AcGeCurve3d : public AcGeEntity3d
{   
public:
     
    // Parametrization.
    //
    void           getInterval    (AcGeInterval&) const;
    void           getInterval    (AcGeInterval&, AcGePoint3d& start,
                                   AcGePoint3d& end) const;
    AcGeCurve3d&   reverseParam   ();
     
    // Distance to other geometric objects. 
    //
    double         distanceTo     (const AcGePoint3d&,
                                   const AcGeTol& = AcGeContext::gTol) const;
    double         distanceTo     (const AcGeCurve3d&,
                                   const AcGeTol& = AcGeContext::gTol) const;

    // Return the point on this object that is closest to the other object.
    // These methods return point on this curve as a simple 3d point.
    //
    AcGePoint3d closestPointTo(const AcGePoint3d& pnt, 
                                  const AcGeTol& = AcGeContext::gTol) const;
    AcGePoint3d closestPointTo(const AcGeCurve3d& curve3d, 
                                        AcGePoint3d& pntOnOtherCrv, 
                                  const AcGeTol& = AcGeContext::gTol) const;

    // Alternate signatures for above functions.  These methods return point
    // on this curve as an AcGePointOnCurve3d.
    //
    void getClosestPointTo(const AcGePoint3d& pnt, AcGePointOnCurve3d&, 
                           const AcGeTol& = AcGeContext::gTol) const;
    void getClosestPointTo(const AcGeCurve3d& curve3d, 
                                 AcGePointOnCurve3d& pntOnThisCrv, 
                                 AcGePointOnCurve3d& pntOnOtherCrv, 
                           const AcGeTol& = AcGeContext::gTol) const;

    // Return closest points when projected in a given direction.
    // These methods return point on this curve as a simple 3d point.
    //
    AcGePoint3d projClosestPointTo(const AcGePoint3d& pnt, 
                                      const AcGeVector3d& projectDirection, 
                                      const AcGeTol& = AcGeContext::gTol) const;
    AcGePoint3d projClosestPointTo(const AcGeCurve3d& curve3d, 
                                      const AcGeVector3d& projectDirection, 
                                            AcGePoint3d& pntOnOtherCrv, 
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Alternate signatures for above functions.  These methods return point
    // on this curve as an AcGePointOnCurve3d.
    //
    void getProjClosestPointTo(const AcGePoint3d& pnt, 
                               const AcGeVector3d& projectDirection, 
                                     AcGePointOnCurve3d&, 
                               const AcGeTol& = AcGeContext::gTol) const;
    void getProjClosestPointTo(const AcGeCurve3d& curve3d, 
                               const AcGeVector3d& projectDirection, 
                                     AcGePointOnCurve3d& pntOnThisCrv, 
                                     AcGePointOnCurve3d& pntOnOtherCrv, 
                               const AcGeTol& = AcGeContext::gTol) const;
    // Project methods.
    //
    AcGeEntity3d*  project       (const AcGePlane& projectionPlane,
                                  const AcGeVector3d& projectDirection,
                                  const AcGeTol& = AcGeContext::gTol) const;
    AcGeEntity3d*  orthoProject  (const AcGePlane& projectionPlane,
                                  const AcGeTol& = AcGeContext::gTol) const;

    // Tests if point is on curve.
    //
    Adesk::Boolean isOn          (const AcGePoint3d&,
                                  const AcGeTol& = AcGeContext::gTol) const; 
    Adesk::Boolean isOn          (const AcGePoint3d& pt, double& param,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isOn          (double param,
                                  const AcGeTol& = AcGeContext::gTol) const;
     
    // Parameter of the point on curve.  Contract: point IS on curve
    //
    double         paramOf       (const AcGePoint3d& pt,
                                  const AcGeTol& = AcGeContext::gTol) const; 
     
    // Geometric inquiry methods.
    //
    Adesk::Boolean isClosed      (const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPlanar      (AcGePlane&,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isLinear      (AcGeLine3d&,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isCoplanarWith(const AcGeCurve3d& curve3d,
                                  AcGePlane& plane,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPeriodic    (double& period) const;

    // Length based methods.
    //
    double         length        (double fromParam, double toParam,
                                  double = AcGeContext::gTol.equalPoint())
                                 const;
    double         paramAtLength (double datumParam, double length,
                                  Adesk::Boolean posParamDir = Adesk::kTrue,
                                  double = AcGeContext::gTol.equalPoint())
                                 const;     
    // Degeneracy.
    //
    Adesk::Boolean isDegenerate  (AcGe::EntityId& degenerateType,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isDegenerate  (AcGeEntity3d*& pConvertedEntity,
                                  const AcGeTol& = AcGeContext::gTol) const;

    // Modify methods.
    //
    void           getSplitCurves(double param, AcGeCurve3d* & piece1, 
                                  AcGeCurve3d* & piece2) const;
     
    // Local closest points
    //
    void getLocalClosestPoints(const AcGePoint3d& point, 
                                     AcGePointOnCurve3d& approxPnt, 
                               const AcGeInterval* nbhd = 0,
                               const AcGeTol& = AcGeContext::gTol) const;
     
    void getLocalClosestPoints(const AcGeCurve3d& otherCurve, 
                                     AcGePointOnCurve3d& approxPntOnThisCrv, 
                                     AcGePointOnCurve3d& approxPntOnOtherCrv, 
                               const AcGeInterval* nbhd1 = 0,
                               const AcGeInterval* nbhd2 = 0,
                               const AcGeTol& = AcGeContext::gTol) const;
    
    // Return start and end points.
    //
    Adesk::Boolean hasStartPoint  (AcGePoint3d& startPoint) const;
    Adesk::Boolean hasEndPoint    (AcGePoint3d& endPoint) const;
    
    // Evaluate methods.
    //
    AcGePoint3d    evalPoint   (double param) const;
    AcGePoint3d    evalPoint   (double param, int numDeriv,
				AcGeVector3dArray& derivArray) const;

    // Assignment operator.
    //
    AcGeCurve3d&   operator =  (const AcGeCurve3d&);

protected:
     
    friend class AcGeImpCurve3d;
	friend class AcGeExternalCurve3d;

    // Private constructors so that no object of this class can be instantiated.
    AcGeCurve3d ();
    AcGeCurve3d (const AcGeCurve3d&);

    // Construct object from its corresponding implementation object.
    AcGeCurve3d (AcGeImpCurve3d&, int);
    AcGeCurve3d (AcGeImpCurve3d*);
};
     
#endif
