#ifndef AC_GEDWGIO_H
#define AC_GEDWGIO_H
//
// $Id: gedwgio.h,v 1.4 1995/12/20 15:16:54 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeDwgIO - an utility class that
// performs DWG file IO.

#include "dbfiler.h"

class AcGePoint2d;
class AcGeVector2d;
class AcGeMatrix2d;
class AcGeScale2d;
class AcGePoint2dArray;
class AcGeVector2dArray;
class AcGePoint3d;
class AcGeVector3d;
class AcGeMatrix3d;
class AcGeScale3d;
class AcGePoint3dArray;
class AcGeVector3dArray;
class AcGeTol;
class AcGeInterval;
class AcGeKnotVector;
class AcGeDoubleArray;
class AcGeCurveBoundary;
class AcGePosition2d;
class AcGePointOnCurve2d;
class AcGeLine2d;
class AcGeLineSeg2d;
class AcGeRay2d;
class AcGeCircArc2d;
class AcGeEllipArc2d;
class AcGeExternalCurve2d;
class AcGeCubicSplineCurve2d;
class AcGeNurbCurve2d;
class AcGePolyline2d;
class AcGePosition3d;
class AcGePointOnCurve3d;
class AcGePointOnSurface;
class AcGeLine3d;
class AcGeRay3d;
class AcGeLineSeg3d;
class AcGePlane;
class AcGeBoundedPlane;
class AcGeCircArc3d;
class AcGeEllipArc3d;
class AcGeCubicSplineCurve3d;
class AcGeNurbCurve3d;
class AcGePolyline3d;
class AcGeAugPolyline3d;
class AcGeExternalCurve3d;
class AcGeCone;
class AcGeCylinder;
class AcGeTorus;
class AcGeExternalSurface;
class AcGeOffsetSurface;
class AcGeNurbSurface;
class AcGeExternalBoundedSurface;
class AcGeSphere;


struct AcGeDwgIO
{
	// Write out to file
	//
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePoint2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeVector2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeMatrix2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeScale2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePoint2dArray&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeVector2dArray&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePoint3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeVector3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeMatrix3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeScale3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePoint3dArray&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeVector3dArray&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeTol&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeInterval&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeKnotVector&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeDoubleArray&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCurveBoundary&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePosition2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePointOnCurve2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeLine2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeLineSeg2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeRay2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCircArc2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeEllipArc2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeExternalCurve2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCubicSplineCurve2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeNurbCurve2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePolyline2d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePosition3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePointOnCurve3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePointOnSurface&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeLine3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeRay3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeLineSeg3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePlane&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeBoundedPlane&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCircArc3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeEllipArc3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCubicSplineCurve3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeNurbCurve3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGePolyline3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeAugPolyline3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeExternalCurve3d&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCone&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeCylinder&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeTorus&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeExternalSurface&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeOffsetSurface&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeNurbSurface&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeExternalBoundedSurface&) ;
	static 
	Acad::ErrorStatus outFields(AcDbDwgFiler*, const AcGeSphere&) ;
	
	// Read in from file
	//
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePoint2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeVector2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeMatrix2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeScale2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePoint2dArray&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeVector2dArray&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePoint3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeVector3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeMatrix3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeScale3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePoint3dArray&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeVector3dArray&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeTol&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeInterval&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeKnotVector&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeDoubleArray&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCurveBoundary&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePosition2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePointOnCurve2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeLine2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeLineSeg2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeRay2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCircArc2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeEllipArc2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeExternalCurve2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCubicSplineCurve2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeNurbCurve2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePolyline2d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePosition3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePointOnCurve3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePointOnSurface&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeLine3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeRay3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeLineSeg3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePlane&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeBoundedPlane&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCircArc3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeEllipArc3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCubicSplineCurve3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeNurbCurve3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGePolyline3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeAugPolyline3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeExternalCurve3d&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCone&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeCylinder&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeTorus&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeExternalSurface&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeOffsetSurface&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeNurbSurface&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeExternalBoundedSurface&) ;
	static 
	Acad::ErrorStatus inFields(AcDbDwgFiler*, AcGeSphere&) ;
};

#endif
