#ifndef AC_GEELL2D_H
#define AC_GEELL2D_H
//
// $Id: geell2d.h,v 1.2 1995/10/26 16:33:34 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeEllipArc2d - A mathematical entity
// used to represent an ellipse in 2-space.


#include "gecurv2d.h"
#include "gevec2d.h"
#include "gepnt2d.h"
#include "geponc2d.h"
#include "geintrvl.h"

class AcGeCircArc2d;
class AcGePlanarEnt;
class AcGeEllipArc2d;
class AcGeLinearEnt2d;


class AcGeEllipArc2d : public AcGeCurve2d
{
public:

    AcGeEllipArc2d();
    AcGeEllipArc2d(const AcGeEllipArc2d&);
    AcGeEllipArc2d(const AcGeCircArc2d&);
    AcGeEllipArc2d(const AcGePoint2d& cent, const AcGeVector2d& majorAxis,
                   const AcGeVector2d& minorAxis, double majorRadius,
                   double minorRadius);
    AcGeEllipArc2d(const AcGePoint2d& cent, const AcGeVector2d& majorAxis,
                   const AcGeVector2d& minorAxis, double majorRadius,
                   double minorRadius, double ang1, double ang2);

    // Intersection with other geometric objects.
    //    
    Adesk::Boolean intersectWith (const AcGeLinearEnt2d&, int& intn,
                                  AcGePoint2d& p1, AcGePoint2d& p2,
                                  const AcGeTol& = AcGeContext::gTol) const;
    // Inquiry Methods
    //
    Adesk::Boolean isCircular    (const AcGeTol& = AcGeContext::gTol) const;

    // Test if point is inside full ellipse
    //
    Adesk::Boolean isInside      (const AcGePoint2d&,
                                  const AcGeTol& = AcGeContext::gTol) const;


    // Definition of ellipse
    //
    AcGePoint2d    center        () const;
    double         minorRadius   () const;
    double         majorRadius   () const;
    AcGeVector2d   minorAxis     () const;
    AcGeVector2d   majorAxis     () const;
    double         startAng      () const;
    double         endAng        () const;
    AcGePoint2d    startPoint    () const;
    AcGePoint2d    endPoint      () const;
    Adesk::Boolean isClockWise   () const;

    AcGeEllipArc2d& setCenter     (const AcGePoint2d&);
    AcGeEllipArc2d& setMinorRadius(double);
    AcGeEllipArc2d& setMajorRadius(double);
    AcGeEllipArc2d& setAxes       (const AcGeVector2d&, const AcGeVector2d&);
    AcGeEllipArc2d& setAngles     (double startAngle, double endAngle);
    AcGeEllipArc2d& set           (const AcGePoint2d& cent,
                                   const AcGeVector2d& majorAxis,
                                   const AcGeVector2d& minorAxis,
                                   double majorRadius, double minorRadius);
    AcGeEllipArc2d& set           (const AcGePoint2d& cent,
                                   const AcGeVector2d& majorAxis,
                                   const AcGeVector2d& minorAxis,
                                   double majorRadius, double minorRadius,
                                   double ang1, double ang2);
    AcGeEllipArc2d& set           (const AcGeCircArc2d&);

    // Assignment operator.
    //
    AcGeEllipArc2d& operator =    (const AcGeEllipArc2d&);

protected:

    friend class AcGeImpEllipArc2d;

    // Construct object from its corresponding implementation object.
    AcGeEllipArc2d (AcGeImpEllipArc2d&, int);
    AcGeEllipArc2d (AcGeImpEllipArc2d*);
};

#endif

