#ifndef AC_GEENT2D_H
#define AC_GEENT2D_H
//
// $Id: geent2d.h,v 1.3 1995/10/26 16:33:49 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class GeEntity2d - An abstract base class
// for all Geometry Library 2d entities.


#include "gegbl.h"
#include "gepnt2d.h"
#include "geintrvl.h"


class AcGeEntity2d
{
public:

    // Destructor.
    //
    ~AcGeEntity2d();

    // Run time type information.
    //
    Adesk::Boolean   isKindOf    (AcGe::EntityId) const;
    AcGe::EntityId   type        () const;

    // Make a copy of the entity.
    //
    AcGeEntity2d*    copy        () const;
    AcGeEntity2d&    operator =  (const AcGeEntity2d&);

    // Equivalence
    //
    Adesk::Boolean   operator == (const AcGeEntity2d&) const;
    Adesk::Boolean   operator != (const AcGeEntity2d&) const;
    Adesk::Boolean   isEqualTo   (const AcGeEntity2d&,
                                  const AcGeTol& = AcGeContext::gTol) const;
                                              
    // Matrix multiplication
    //
    AcGeEntity2d&    transformBy (const AcGeMatrix2d&);
    AcGeEntity2d&    translateBy (const AcGeVector2d&);
    AcGeEntity2d&    rotateBy    (double angle, const AcGePoint2d& wrtPoint
                                  = AcGePoint2d::kOrigin);        
    AcGeEntity2d&    mirror      (const AcGeLine2d&);
    AcGeEntity2d&    scaleBy     (double scaleFactor, 
                                  const AcGePoint2d& wrtPoint
                                  = AcGePoint2d::kOrigin);
    // Point containment
    //
    Adesk::Boolean   isOn        (const AcGePoint2d&,
                                  const AcGeTol& = AcGeContext::gTol) const; 
                                         
protected:

    friend class AcGeEntity3d;
    friend class AcGeImpEntity2d;

    AcGeImpEntity2d  *mpImpEnt;
    int              mDelEnt;

    // Private constructors so that no object of this class can be instantiated.
    AcGeEntity2d ();
    AcGeEntity2d (const AcGeEntity2d&);

    // Construct object from its corresponding implementation object.
    AcGeEntity2d (AcGeImpEntity2d&, int);
    AcGeEntity2d (AcGeImpEntity2d*);

    // Member functions that are used internally.
    AcGeEntity2d*    newEntity2d (AcGeImpEntity2d*) const;
};


inline AcGeEntity2d*     
AcGeEntity2d::newEntity2d (AcGeImpEntity2d *impEnt ) const
{
    return new AcGeEntity2d ( impEnt );
}

#endif
