#ifndef AC_GENURBSF_H
#define AC_GENURBSF_H
//
// $Id: genurbsf.h,v 1.2 1995/10/26 16:35:29 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc. 
//
// The information contained herein is confidential, proprietary  to 
// Autodesk,  Inc., and considered a trade secret as defined in sec- 
// tion 499C of the penal code of the State of California.   Use  of 
// this information by anyone other than authorized employees of Au- 
// todesk, Inc. is  granted  only  under  a  written  non-disclosure 
// agreement,  expressly  prescribing  the  scope and manner of such 
// use.
//
// Description:
//
// This file implements the class AcGeNurbSurface, a representation 
// for a non-uniform, rational BSpline surface.
// 
     
#include "gegbl.h"
#include "gesurf.h"
     
     
class AcGePoint3dArray;
class AcGeDoubleArray;
     
class AcGeNurbSurface : public AcGeSurface 
{
public:
   AcGeNurbSurface();
   AcGeNurbSurface(int degreeU, int degreeV, int propsInU, int propsInV,
                   int numControlPointsInU, int numControlPointsInV,
                   const AcGePoint3d controlPoints[],
                   const double weights[],
                   const AcGeKnotVector& uKnots,
                   const AcGeKnotVector& vKnots,
                   const AcGeTol& = AcGeContext::gTol);
   AcGeNurbSurface(const AcGeNurbSurface&);
     
   // Assignment.
   //
   AcGeNurbSurface& operator = (const AcGeNurbSurface&);
     
   // Geometric properties.
   //
   Adesk::Boolean   isRationalInU      () const; 
   Adesk::Boolean   isClosedInU        () const; 
   Adesk::Boolean   isPeriodicInU      (double&) const; 
   Adesk::Boolean   isRationalInV      () const; 
   Adesk::Boolean   isClosedInV        () const; 
   Adesk::Boolean   isPeriodicInV      (double&) const;
     
   int singularityInU () const; 
   int singularityInV () const;
     
   // Definition data.
   //
   int            degreeInU            () const; 
   int            numControlPointsInU  () const; 
   int            degreeInV            () const; 
   int            numControlPointsInV  () const;
   void           getControlPoints     (AcGePoint3dArray&) const; 
   Adesk::Boolean getWeights           (AcGeDoubleArray&) const;
     
   int       numKnotsInU         () const;
   void      getUKnots           (AcGeKnotVector&) const;
     
   int       numKnotsInV         () const;
   void      getVKnots           (AcGeKnotVector&) const;
     
   void      getDefinition      (int& degreeU, int& degreeV,
                                 int& propsInU,	int& propsInV,
                                 int& numControlPointsInU,
                                 int& numControlPointsInV,
                                 AcGePoint3dArray& controlPoints,
                                 AcGeDoubleArray& weights,
                                 AcGeKnotVector& uKnots,
                                 AcGeKnotVector& vKnots) const;
        
   // Reset surface
   //
   AcGeNurbSurface& set     (int degreeU, int degreeV,
                             int propsInU, int propsInV,
                             int numControlPointsInU,
                             int numControlPointsInV,
                             const AcGePoint3d controlPoints[],
                             const double weights[],
                             const AcGeKnotVector& uKnots,
                             const AcGeKnotVector& vKnots,
                             const AcGeTol& = AcGeContext::gTol);           
                
     
private:
     
    friend class AcGeImpNurbSurface;
     
    // Construct object from its corresponding implementation object. 
    AcGeNurbSurface (AcGeImpNurbSurface&, int);
    AcGeNurbSurface (AcGeImpNurbSurface*);
};
     
#endif
