#ifndef AC_GEPLANE_H
#define AC_GEPLANE_H
//
// $Id: geplane.h,v 1.7 1995/12/20 15:17:40 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGePlane - A mathematical entity
// used to represent a plane in 3-space.

#include "geplanar.h"

class AcGeBoundedPlane;
class AcGeLine3d;

class AcGePlane : public AcGePlanarEnt
{
public:
    // Global plane objects.
    //
    static const AcGePlane     kXYPlane,
                               kYZPlane,
                               kZXPlane;

    AcGePlane();
    AcGePlane(const AcGePlane&);
    AcGePlane(const AcGePoint3d&, const AcGeVector3d&);
    AcGePlane(const AcGePoint3d&, const AcGePoint3d&, const AcGePoint3d&);
    AcGePlane(const AcGePoint3d&, const AcGeVector3d& uAxis,
              const AcGeVector3d& vAxis);
    AcGePlane(double, double, double, double);

    // Signed distance from a point to a plane.
    //
    double         signedDistanceTo (const AcGePoint3d&) const;

    // Intersection
    //
    Adesk::Boolean intersectWith    (const AcGeLinearEnt3d&, AcGePoint3d&,
                                     const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean intersectWith    (const AcGePlane&, AcGeLine3d&,
                                     const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean intersectWith    (const AcGeBoundedPlane&, AcGeLineSeg3d&,
                                     const AcGeTol& = AcGeContext::gTol) const;
    
    // Geometry redefinition.
    //
    AcGePlane&     set              (const AcGePoint3d&, const AcGeVector3d&);
    AcGePlane&     set              (const AcGePoint3d&, const AcGePoint3d&,
                                     const AcGePoint3d&);
    AcGePlane&     set              (double, double, double, double);
    AcGePlane&     set              (const AcGePoint3d&, 
                                     const AcGeVector3d& uAxis,
                                     const AcGeVector3d& vAxis);
    // Assignment operator.
    //
    AcGePlane&     operator =       (const AcGePlane&);

protected:

    friend class AcGeImpPlane;

    // Construct object from its corresponding implementation object.
    AcGePlane (AcGeImpPlane&, int);
    AcGePlane (AcGeImpPlane*);
};




#endif
