#ifndef AC_GEPONC2D_H
#define AC_GEPONC2D_H
//
// $Id: geponc2d.h,v 1.1 1995/07/28 16:53:53 tomba Exp $
//
// (C) Copyright 1993,1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains:
// class      AcGeEvaluatorDataCurve2d - An abstract base  class  from which 
// all 2d curves  evaluator data is derived. EvalData is data stored between
// calls to the evaluator to speed up evaluation of points  and  derivatives.
//
// class AcGePointOnCurve2d class to keep information of a point belonging to
// a 2d curve

#include "gepent2d.h"

class AcGeCurve2d;

class  AcGePointOnCurve2d : public AcGePointEnt2d
{
public:
    // Constructors.
    //
    AcGePointOnCurve2d  ();
    AcGePointOnCurve2d  (const AcGeCurve2d& crv);
    AcGePointOnCurve2d  (const AcGeCurve2d& crv, double param);
    AcGePointOnCurve2d  (const AcGePointOnCurve2d& src);

    // Assignment operator.
    //
    AcGePointOnCurve2d& operator =     (const AcGePointOnCurve2d&);

    // Query functions.
    //
    const AcGeCurve2d   *curve         () const;
    double              parameter      () const;

    // Functions to evaluate a point.
    //
    AcGePoint2d         point          () const;
    AcGePoint2d         point          (double param);
    AcGePoint2d         point          (const AcGeCurve2d& crv, double param);

    // Functions to evaluate the derivatives.
    //
    AcGeVector2d        deriv          (int order) const;
    AcGeVector2d        deriv          (int order, double param);
    AcGeVector2d        deriv          (int order, const AcGeCurve2d& crv, 
                                         double param);

    // Singularity
    //
    Adesk::Boolean      isSingular     ( const AcGeTol& = 
                                         AcGeContext::gTol) const;
    Adesk::Boolean      curvature      ( double& res );
    Adesk::Boolean      curvature      ( double param, double& res );
    // Set functions.  
    //
    AcGePointOnCurve2d& setCurve       (const AcGeCurve2d& crv);
    AcGePointOnCurve2d& setParameter   (double param);

protected:

    friend class AcGeImpPointOnCurve2d;

    // Construct object from its corresponding implementation object.
    AcGePointOnCurve2d (AcGeImpPointOnCurve2d&, int);
    AcGePointOnCurve2d (AcGeImpPointOnCurve2d*);
};

#endif

