#ifndef AC_GEPONC3D_H
#define AC_GEPONC3D_H
//
// $Id: geponc3d.h,v 1.1 1995/07/28 16:53:55 tomba Exp $
//
// (C) Copyright 1993,1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains:
// class      AcGeEvaluatorDataCurve3d - An abstract base  class  from which 
// all 3d curves  evaluator data is derived. EvalData is data stored between
// calls to the evaluator to speed up evaluation of points  and  derivatives.
//
// class AcGePointOnCurve3d class to keep information of a point belonging to
// a 3d curve

#include "gepent3d.h"

class AcGeCurve3d;

class  AcGePointOnCurve3d : public AcGePointEnt3d
{
public:
    // Constructors.
    //
    AcGePointOnCurve3d  ();
    AcGePointOnCurve3d  (const AcGeCurve3d& crv);
    AcGePointOnCurve3d  (const AcGeCurve3d& crv, double param);
    AcGePointOnCurve3d  (const AcGePointOnCurve3d& src);

    // Assignment operator.
    //
    AcGePointOnCurve3d& operator =     (const AcGePointOnCurve3d&);

    // Query functions.
    //
    const AcGeCurve3d   *curve         () const;
    double              parameter      () const;

    // Functions to evaluate a point.
    //
    AcGePoint3d         point          () const;
    AcGePoint3d         point          (double param);
    AcGePoint3d         point          (const AcGeCurve3d& crv, double param);

    // Functions to evaluate the derivatives.
    //
    AcGeVector3d        deriv          (int order) const;
    AcGeVector3d        deriv          (int order, double param);
    AcGeVector3d        deriv          (int order, const AcGeCurve3d& crv, 
                                        double param);
    // Singularity
	//
	Adesk::Boolean 		isSingular     ( const AcGeTol& = 
	                                     AcGeContext::gTol) const;
    Adesk::Boolean     	curvature      ( double& res );
    Adesk::Boolean     	curvature      ( double param, double& res );

    // Set functions.  
    //
    AcGePointOnCurve3d& setCurve       (const AcGeCurve3d& crv);
    AcGePointOnCurve3d& setParameter   (double param);

protected:

    friend class AcGeImpPointOnCurve3d;

    // Construct object from its corresponding implementation object.
    AcGePointOnCurve3d (AcGeImpPointOnCurve3d&, int);
    AcGePointOnCurve3d (AcGeImpPointOnCurve3d*);
};

#endif

