#ifndef AC_GESURF_H
#define AC_GESURF_H
//
// $Id: gesurf.h,v 1.5 1995/10/26 16:36:38 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// AcGeSurface class is an abstract base class for all other parametric
// surfaces.

#include "geent3d.h"

class AcGeVector3dArray;
class AcGePoint2d;
class AcGeCurve3d;
class AcGePointOnCurve3d;
class AcGePointOnSurface;
class AcGePointOnSurfaceData;
class AcGeInterval;


class AcGeSurface : public AcGeEntity3d
{
public:

    // Parameter related.
    //
    AcGePoint2d     paramOf        (const AcGePoint3d&, 
                                    const AcGeTol& = AcGeContext::gTol) const;
   
    // Point containment
    //
    Adesk::Boolean  isOn           (const AcGePoint3d&,
                                    const AcGeTol& = AcGeContext::gTol) const; 
    Adesk::Boolean  isOn           (const AcGePoint3d&, AcGePoint2d& paramPoint,
                                    const AcGeTol& = AcGeContext::gTol ) const;

    // Operations.
    //
    AcGePoint3d closestPointTo(const AcGePoint3d&, 
                                  const AcGeTol& = AcGeContext::gTol) const;

    void getClosestPointTo(const AcGePoint3d&, AcGePointOnSurface&, 
                           const AcGeTol& = AcGeContext::gTol) const;
	
    double          distanceTo     (const AcGePoint3d&,
                                    const AcGeTol& = AcGeContext::gTol) const;

    Adesk::Boolean  isNormalReversed () const;
    AcGeSurface&    reverseNormal    ();
    
    // Assignment operator.
    //
    AcGeSurface&    operator =     (const AcGeSurface&);

    // Bounds in parameter space.
    //
    void   getEnvelope  (AcGeInterval& u, AcGeInterval& v) const;
	
	// Evaluators.
	// Derivative arrays are indexed partialU, partialV followed by
	// the mixed partial.
	//
	AcGePoint3d   evalPoint   (const AcGePoint2d& param) const;
	AcGePoint3d   evalPoint   (const AcGePoint2d& param, int NumDeriv,
	                           AcGeVector3dArray& derivatives) const;
	AcGePoint3d   evalPoint   (const AcGePoint2d& param, int NumDeriv,
	                           AcGeVector3dArray& derivatives,
	                           AcGeVector3d& normal) const;
protected:
     
    friend class AcGeImpSurface;
	friend class AcGeExternalSurface;
	friend class AcGeOffsetSurface;

    // Private constructors so that no object of this class can be instantiated.
    AcGeSurface ();
    AcGeSurface (const AcGeSurface&);

    // Construct object from its corresponding implementation object.
    AcGeSurface (AcGeImpSurface&, int);
    AcGeSurface (AcGeImpSurface*);
};

#endif
