/* Next available MSG number is   1 */
/* $Id: rxdlinkr.h,v 1.13 1995/10/26 17:07:55 neb Exp $ */
/*
      (C) Copyright 1982-1992,1994,1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

#ifndef _rxdlinkr_h
#define _rxdlinkr_h 1

class AcRxDLinkerReactor;
class AcRxService;

#include "rxobject.h"

class AcRxDynamicLinker: public AcRxObject
{
public:
    ACRX_DECLARE_MEMBERS(AcRxDynamicLinker);

    /* getSymbolAddress()

       Given an service name and a text string representing
       a symbol in the module, return the corresponding address
       as a (void *).
    */
    virtual void * getSymbolAddress(const char * serviceName,
				    const char * symbol) const = 0;

    /* initListedModules()

       Opens file indicated by input fileName, which
       is assumed to be a text file which is assumed
       to have the following format for each line:

       <Module filename>  <init function name> <analyze system function name>

       Each line is processed, the module is opened and a call to 
       its <init functin> is made. 

       After the set of filenames is loaded, each module is called
       with its <analyze system> function, so it can find out what
       is in the system, cache values, etc.

       returns kFalse if any module listed in the file is not
       successfully loaded, Adesk::kTrue if all were loaded successfully.
    */
    virtual Adesk::Boolean initListedModules(const char * fileName) = 0;

    /* loadModule()

       This member function does the following:

       - Dynamically loads the named module.

       - Gets the address of the moduleInitFcn

       - Calls the module init function

       - register the module name, internal handle and analyze system
	 function.
	 
       - the <Analyze system> call for each loaded module is called.

       Returns Adesk::kTrue if module was successfully found, loaded, and had
       the named initialization function, which was invoked.
    */
    virtual Adesk::Boolean loadModule(char * fileName, int printit) = 0;

    /* registerService()

       Service names and objects are registered through this class, which
       associates them with the module that it is currently making an 
       initialization call to, or the main program if it is not currently
       loading the module.
    */
    virtual AcRxObject * registerService(char * serviceName,
					  AcRxService * serviceObj) = 0;

    /* unloadModule()

       *** Implemented stub which returns Adesk::kFalse for now. ***

    */
    virtual Adesk::Boolean unloadModule(const char * fileName) = 0;

    /* addReactor()
       delReactor()
    
       Allow system objects to monitor the loading and unloading
       of Arx applications.

    */


    virtual void addReactor(AcRxDLinkerReactor* newReactor) = 0;

    virtual void removeReactor(AcRxDLinkerReactor* oldReactor) = 0;

    virtual int isApplicationLocked  (const char* modulename) const = 0;
    virtual int lockApplication  (void* appId) const = 0;
    virtual int unlockApplication(void* appId) const = 0;

};


/*  acrxDynamicLinker

    The kernel creates the one and only dynamic linker, at

    sysRegistry->at(ACRX_DYNAMIC_LINKER)
*/

#define acrxDynamicLinker						\
AcRxDynamicLinker::cast(acrxSysRegistry()->at(ACRX_DYNAMIC_LINKER))


/* AcRxDLinkerReactor

   Class defining notification events from the dynamic linker,
   specifically the loading and unloading of Arx applications.

    Recipients of these calls can and should also
    check the registered services, class and objects to find out
    what happened to logical elements of the system, rather
    than be bound to file name.
*/

class AcRxDLinkerReactor: public AcRxObject
{
  public:
    ACRX_DECLARE_MEMBERS(AcRxDLinkerReactor);

    virtual void rxAppLoaded(const char * moduleName);

    virtual void rxAppUnloaded(const char * moduleName);

};


#endif
