/* Next available MSG number is   1 */
/* $Id: acdbtest.cc,v 1.46 1995/12/20 17:04:27 neb Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/* ========= acdbtest.cc:  ARx AcDbLibrary Test ================ 

    Test App for ARx Usage of AcDb libary to manipulate the database.

    CREATED BY:  Bill Adkison, March 1994

*/

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <string.h>

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
#include "rxditer.h"
#include "aced.h"

#include "adslib.h"
#include "dbents.h"
#include "dbdict.h"
#include "dbsymtb.h"
#include "geassign.h"

#include "acdbabb.h"

// Test functions.
//
void createExtDict();
void listExtDict();
void createPersReac();
void testPersReac();
void delPersReac();

#define APP_NAME_X "ECHS"
#define APP_NAME_Y "WHY"
#define APP_NAME_Z "ZEE"

extern "C" {
// locally defined entry point invoked by ARx.
AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt);
}

// Status checking

Adesk::Boolean statOk(Acad::ErrorStatus stat)
{
    if (stat == Acad::eOk)
	return Adesk::kTrue;
    ads_printf(/*MSG0*/"\nStatus Check Failed:  %d\n", stat);
    return Adesk::kFalse;
}

// Utility routine to get a yes or no anser.

static Adesk::Boolean getYorN(const char* pStr)
{
    char yorn_str[132];

    // specific prompt.
    ads_printf("\n%s", pStr);

    ads_initget(0, "No Yes");

    yorn_str[0] = 'Y';
    yorn_str[1] = '\0';

    switch (ads_getstring(Adesk::kFalse, " -- No/<Yes>:  ",
                          yorn_str)) {
    case RTKWORD:
        ads_getinput(yorn_str);
        /* Deliberate fallthrough */
    default:
        break;
    }      
 
    return (!((yorn_str[0] == 'N') || (yorn_str[0] == 'n')));
}

// Copy an open object, and append it to the database.

Adesk::Boolean cloneAndAdd(AcDbEntity* entObj)
{
    Acad::ErrorStatus    as, bs, cs, ds, es;
    AcDbBlockTable*	 pBlockTable;
    AcDbBlockTableRecord*pBlockTableRecord;
    AcDbEntity*          tEnt;
    AcDbObjectId         newEntId;

    // Copy the Entity;.
    tEnt = AcDbEntity::cast(entObj->clone());

    // Open the current drawings block table.
    as = acdbCurDwg()->getBlockTable(pBlockTable, AcDb::kForRead);
    if (!statOk(as))
	return Adesk::kFalse;

    // get the model space block.
    bs = pBlockTable->getAt(ACDB_MODEL_SPACE, pBlockTableRecord, kForWrite);
    if (!statOk(bs)) {
	pBlockTable->close();
	return Adesk::kFalse;
    }

    // Append the copy
    cs = pBlockTableRecord->appendAcDbEntity(newEntId, tEnt);

    // Close the block, and the block table.
    //
    ds = pBlockTableRecord->close();
    as = pBlockTable->close();

    // Close the new Object.  Must be done after being appended.
    // doesn't matter if before or after closing block.
    //
    es = tEnt->close();

    if (!(
	statOk(as) &&
	statOk(bs) &&
	statOk(cs) &&
	statOk(ds) &&
	statOk(es)
	))
    {
	return Adesk::kFalse;
    }

    return Adesk::kTrue;
}


AcDbObject* selectObject(AcDb::OpenMode openMode)
{
    char handleStr[17];
    ads_name     objName;
    AcDbObjectId objId;
    AcDbObject*  pObj = NULL;

    // Use Handent on input string.  Useful for subentity testing.

    switch (ads_getstring(Adesk::kFalse, "Enter Valid Object Handle: ",
		      handleStr)) {
	case RTNORM:
	    if (ads_handent(handleStr, objName) == RTNORM) {
	        if (acdbGetObjectId(objId, objName) == Acad::eOk) {
		    if (!statOk(acdbOpenObject(pObj, objId, openMode)))
			pObj = NULL;  // Not guaranteed on failure.
	        } else
		   ads_printf("\nacdbGetObjectId failed with valid input.\n");
	    } else
		ads_printf("\nInvalid Handle Specified.\n");
	    break;
	default:
	    ads_printf("\nNo Handle Specified.\n");
	    break;
    }
    return pObj;
}

AcDbEntity* selectEntity(AcDbObjectId& eId, AcDb::OpenMode openMode)
{
    ads_name en;
    ads_point pt;

    Acad::ErrorStatus retStat;
    int ss;

    if ((ss = ads_entsel("\nSelect an Entity:  ", en, pt)) != RTNORM) {
	ads_printf("Nothing Selected, Return Code==%d\n", ss);
        return NULL;
    }

    // Now, exchange the old-fangled ads_name for the new-fangled
    // object id...
    retStat = acdbGetObjectId(eId, en);
    if (retStat != Acad::eOk) {
	ads_printf("\nacdbGetObjectId failed");
	ads_printf("\nen==(%lx,%lx), retStat==%d\n",
			   en[0], en[1], eId);
	return NULL;
    }

    AcDbEntity* entObj;

    if ((retStat = acdbOpenObject(entObj, eId, openMode))
	!= Acad::eOk) {
	ads_printf(
	    "\nacdbOpenObject failed: ename=(%lx,%lx), mode==%d retStat==%d\n",
	    en[0], en[1], openMode, retStat);
        return NULL;
    }
    return entObj;
}


void printObjectInfo(AcDbObjectId id)
{
    AcDbObject* obj;
    Acad::ErrorStatus es;

    if ((es = acdbOpenObject(obj, id, AcDb::kForRead)) !=
	Acad::eOk)

	ads_printf("  Object Open Failed, Stat==%d\n", es);
    else {
	ads_printf(/*MSG0*/"Object Id==%lx, ", id);

	AcDbHandle objHandle;
	obj->getAcDbHandle(objHandle);
	char handleStr[20];
	objHandle.getIntoAsciiBuffer(handleStr);
	ads_printf(/*MSG0*/"Handle==%s, ", handleStr);

	ads_printf("Class==%s\n", obj->isA()->name());

	obj->close();
    }
}

/* =================== Commands defined in this Module ================ */


/* "DUMP" -- Print the entity contents of the current drawing */

void dump()
{
    AcDbDatabase* pDb = acdbCurDwg();

    ads_printf("\n\nDUMP Current Drawing ...");

    AcDbBlockTable *pTable;
    if (!statOk(pDb->getBlockTable (pTable, AcDb::kForRead)))
	return;

    AcDbBlockTableIterator* pBTIter;
    if (!statOk(pTable->newIterator(pBTIter))) {
	pTable->close();
	return;
    }

    AcDbBlockTableRecord *pRec;
    for (; !pBTIter->done(); pBTIter->step()) {

        if (!statOk(pBTIter->getRecord(pRec, AcDb::kForRead))) {
	    delete pBTIter;
	    pTable->close();
	    return;
	}

        char *pName;
        if (!statOk(pRec->getName(pName))) {
	    pRec->close();
	    delete pBTIter;
	    pTable->close();
	    return;
	}
        ads_printf("\nBTR %s: %ld", pName, pRec->objectId().asOldId());

        AcDbBlockTableRecordIterator *pIter;
        if (!statOk(pRec->newIterator(pIter))) {
	    pRec->close();
	    delete pBTIter;
	    pTable->close();
	    return;
	}

        AcDbEntity *pEnt;
        for (; !pIter->done(); pIter->step()) {

            if (!statOk(pIter->getEntity(pEnt, AcDb::kForRead))) {
		delete pIter;
		pRec->close();
		delete pBTIter;
		pTable->close();
		return;
	    }

            ads_printf("\n  %s: %ld, owner %ld", pEnt->isA()->name(),
                                      pEnt->objectId().asOldId(),
                                      pEnt->ownerId().asOldId());

	    AcDbObjectIterator *pSubIter = NULL;
	    AcDb2dPolyline *p2d = AcDb2dPolyline::cast(pEnt);
	    AcDb3dPolyline *p3d = AcDb3dPolyline::cast(pEnt);
            AcDbBlockReference *pBR = AcDbBlockReference::cast(pEnt);
            if (pBR) {
                ads_printf(" -> BTR %ld", pBR->blockTableRecord().asOldId());
		pSubIter = pBR->attributeIterator();
	    } else if (p2d)
		pSubIter = p2d->vertexIterator();
	    else if (p3d)
		pSubIter = p3d->vertexIterator();
		
	    if (pSubIter) {
		AcDbEntity *pSubEnt;
		AcDbObjectId subId;
		for (; !pSubIter->done(); pSubIter->step()) {

		    if (!statOk(acdbOpenAcDbEntity(pSubEnt,
						   pSubIter->objectId(),
						   AcDb::kForRead))) 
		    {
			delete pSubIter;
			pEnt->close();
			delete pIter;
			pRec->close();
			delete pBTIter;
			pTable->close();
			return;
		    }

		    ads_printf("\n    %s: %ld, owner %ld",
					      pSubEnt->isA()->name(),
					      pSubEnt->objectId().asOldId(),
					      pSubEnt->ownerId().asOldId());

		    if (!statOk(pSubEnt->close())) {
			delete pSubIter;
			pEnt->close();
			delete pIter;
			pRec->close();
			delete pBTIter;
			pTable->close();
			return;
		    }
		}
		delete pSubIter;
	    }

            if (!statOk(pEnt->close())) {
		delete pIter;
		pRec->close();
		delete pBTIter;
		pTable->close();
		return;
	    }
        }
        delete pIter;
        if (!statOk(pRec->close())) {
	    delete pBTIter;
	    pTable->close();
	    return;
	}
    }
    delete pBTIter;
    statOk(pTable->close());
}

/* "SWAP" -- Swap the Ids of two entities */

void swap()
{
    AcDbEntity *pEnt1, *pEnt2;
    AcDbObjectId id1, id2;

    pEnt2 = selectEntity(id2, AcDb::kForRead);
    if (pEnt2 == NULL)
	return;

    // Must do the Write 2nd because cannot call selectEntity
    // when anything is currently open for Write.
    //
    pEnt1 = selectEntity(id1, AcDb::kForWrite);
    if (pEnt1 == NULL) {
	pEnt2->close();
	return;
    }
    if (pEnt1->isA() != pEnt2->isA()) {
	ads_printf("\nBoth entities must be the same kind.");
	pEnt1->close();
	pEnt2->close();
	return;
    }
    pEnt2->close();
    statOk(pEnt1->swapIdWith(id2));
    pEnt1->close();
    ads_printf("\n%ld & %ld have been swapped.", id1.asOldId(), id2.asOldId());
}

/* "ADDLINE" -- Add an AcDbLine to the database.  */

void addLine()
{
    AcGePoint3d fromPt;
    AcGePoint3d toPt;
    Acad::ErrorStatus es;
    struct resbuf* rb;

    ads_initget(0, "Guess");
    switch (ads_getpoint(NULL, "Enter 1st Point/(or \"Guess\"): ",
                         &fromPt.x)) {
    case RTNORM:
        break;
    case RTKWORD:
        ads_printf("\nMy guess is... 8.0,2.0\n");
        fromPt.set(8.0, 2.0, 0.0);
        break;
    default:
        ads_alert("Try it again, and enter a point!");
        return;
    }  

    switch (ads_getpoint(asDblArray(fromPt),
                          "Enter 2nd Point: ",
                         asDblArray(toPt))) {
    case RTNORM:
        break;
    default:
        ads_alert("We need two points!  Try again, please!");
        return;
    }  

    // Give it the current UCS normal.
    AcGeVector3d normal(0.0, 0.0, 1.0);
    struct resbuf to, from;

    from.restype = RTSHORT;
    from.resval.rint = 1; // UCS (Pass in AcGeVector3d::kIdentity below)
    to.restype = RTSHORT;
    to.resval.rint = 0;   // WCS
    ads_trans(asDblArray(normal), &from, &to, 
	      Adesk::kTrue, asDblArray(normal));

    AcDbLine*  newLine = new AcDbLine;
    newLine->setStartPoint(fromPt);
    newLine->setEndPoint(toPt);
    newLine->setNormal(normal);

// #define TRY_PRE_APPEND 1
#ifdef TRY_PRE_APPEND
    // Try adding Xdata before appending the object to the database.
    //
    ads_regapp(APP_NAME_X);
    rb = ads_buildlist(-3,
		1001, APP_NAME_X,
		1070, 1070,
		0);
    if ((es = newLine->setXData(rb)) == Acad::eOk)
	ads_printf("Pre-append addition of Xdata ok.");
    else
	ads_printf("Pre-append addition of Xdata failed %d.", es);
    ads_relrb(rb);
#endif //TRY_PRE_APPEND

    // Now, to add the entity to the current space.
    // This protocol will change, but will still involve getting
    // the correct block definition.  WCA 3/10/94

    Acad::ErrorStatus es1, es2, es3, es4, es5, es6;
    AcDbBlockTable* pBlockTable;
    AcDbBlockTableRecord* pBlockTableRecord;
    AcDbObjectId lineObjId;

    // To add an entity to a block (paper and model space are now
    // blocks) you first need to get the block (from the block
    // table, which you get from the database).
    //
    // BTW, you can't see the protocol for handling BLKDEF and
    // BLKEND entities here yet - which is good, because the
    // protocol for those beasts is still not implemented in
    // its final form.
    //
    es1 = acdbCurDwg()->getBlockTable(pBlockTable, AcDb::kForRead);

    if (es1 == Acad::eOk) {

	es2 = pBlockTable->getAt(ACDB_MODEL_SPACE, pBlockTableRecord,kForWrite);

        if (es2 == Acad::eOk) {
	    es3 = pBlockTableRecord->appendAcDbEntity(lineObjId, newLine);
	    newLine->setColorIndex(1); // aka Red
	    newLine->setLayer("0");    // Hammer it, for no real reason...

// #define TRY_POST_APPEND 1
#ifdef TRY_POST_APPEND
	    // Try adding Xdata after appending the object to the database.
	    //
	    ads_regapp(APP_NAME_Y);
	    rb = ads_buildlist(-3,
			1001, APP_NAME_Y,
			1070, 1071,
			0);
	    if ((es = newLine->setXData(rb)) == Acad::eOk)
		ads_printf("Post-append addition of Xdata ok.");
	    else
		ads_printf("Post-append addition of Xdata failed %d.", es);
	    ads_relrb(rb);
#endif

	    es4 = pBlockTableRecord->close();
	}

	es5 = pBlockTable->close();
    }

    if ((es1 == Acad::eOk &&
    	 es2 == Acad::eOk &&
	 es3 == Acad::eOk &&
	 es4 == Acad::eOk &&
	 es5 == Acad::eOk &&
	 (es6 = newLine->close()) == Acad::eOk))
    {
	ads_printf("\nSuccess: New Line Added. Id ==%lx\n", lineObjId);

#define TRY_POST_CLOSE 1
#ifdef TRY_POST_CLOSE
		
	    AcDbEntity* pEnt;

	    es = acdbOpenAcDbEntity(pEnt, lineObjId, AcDb::kForWrite);

	    if (es == Acad::eOk) {

	        // Try adding Xdata after the append and close.
	        //
	        ads_regapp(APP_NAME_Z);
	        rb = ads_buildlist(-3,
			1001, APP_NAME_Z,
			1070, 1072,
			0);
	        if ((es = pEnt->setXData(rb)) == Acad::eOk)
		    ads_printf("Post-close addition of Xdata ok.");
	        else
		    ads_printf("Post-close addition of Xdata failed %d.", es);
	        ads_relrb(rb);

		pEnt->close();

	    } else {

		ads_printf("Open of new line failed: %d", es);

	    }
#endif

    } else {
	ads_printf("\nSomething went wrong in addLine():");
	ads_printf("\n    1:%d 2:%d 3:%d 4:%d 5:%d 6:%d",
	    es1, es2, es3, es4, es5, es6);
    }

    // How to get an entname from an Object Id.
    //
    ads_name lineEntName;

    if ((es = acdbGetAdsName(lineEntName, lineObjId)) != Acad::eOk) {
	ads_printf("\nacdbGetAdsName failed, es==%d\n", es);
	return;
    }
}

void addPline()
{
    AcDbBlockTable* pBlockTable;
    AcDbBlockTableRecord* pBlockTableRecord;
    AcDbObjectId plineObjId;

    if (statOk(acdbCurDwg()->getBlockTable(pBlockTable, AcDb::kForRead))) {

	if (statOk(pBlockTable->getAt(ACDB_MODEL_SPACE, 
	    pBlockTableRecord, AcDb::kForWrite)))
	{
	    AcDb2dPolyline* newPline = new AcDb2dPolyline;

	    newPline->setColorIndex(3);
	    newPline->makeClosed();
	    statOk(pBlockTableRecord->appendAcDbEntity(plineObjId, newPline));
	    statOk(newPline->setLayer("0"));

	    int i;
	    for (i = 0; i < 4; i++) {
		AcGePoint3d tPt(0.0,0.0,2.0);
		tPt.x = i / 2;
		tPt.y = i % 2;
		AcDb2dVertex* newVtx = new AcDb2dVertex();
		newVtx->setPosition(tPt);
		statOk(newPline->appendVertex(newVtx));
		statOk(newVtx->close());
	    }
	    statOk(newPline->close());
	    statOk(pBlockTableRecord->close());
	}
	statOk(pBlockTable->close());
    }
}

void addDim()
{
    AcDbBlockTable* pBlockTable;
    AcDbBlockTableRecord* pBlockTableRecord;
    AcDbObjectId dimObjId;

    if (statOk(acdbCurDwg()->getBlockTable(pBlockTable, AcDb::kForRead))) {

        if (statOk(pBlockTable->getAt(ACDB_MODEL_SPACE, 
	    pBlockTableRecord, kForWrite)))
	{
            if (!statOk(pBlockTable->close())) {
	         statOk(pBlockTableRecord->close());
                 return;
            }
            AcDbRotatedDimension* newDim = new AcDbRotatedDimension;

            newDim->setColorIndex(5);
            newDim->setHorizontalRotation(10.0);
            statOk(pBlockTableRecord->appendAcDbEntity(dimObjId, newDim));

            AcGePoint3d tPt(0.0,0.0,2.0);
            tPt.x = 4.0;
            tPt.y = 4.0;
            newDim->setXLine1Point(tPt);
            tPt.x = 6.0;
            tPt.y = 3.5;
            newDim->setXLine1Point(tPt);
            tPt.x = 4.0;
            tPt.y = 5.0;
            newDim->setDimLinePoint(tPt);
            newDim->setXLine1Point(tPt);
            
            statOk(newDim->close());
        } else {
            statOk(pBlockTable->close());
            return;
        }
        statOk(pBlockTableRecord->close());
    }
}

// OBELISK
//
// This works best when from a 3d view,
// vpoint 1,3,2 works fine.
//
// Builds a pyramid within the currently specified limits, in the current
// UCS, by cloning an extruded line, and modifying the selected line on 
// every step.
//
// It isn't a particulaly focused test, it exercises
// the following features:
//
// 1. Entity Selection (ads_nentsel)
// 2. Exchange ads_name for an AcDbObjectId
// 3. AcDbObject::clone()
// 4. Appending new Entities to current space
// 5. Repeated open/modification/close of an entity.
// 6. Runtime class testing.
//

void obelisk()
{
    ads_name en;
    ads_point pt;

    AcDbObjectId eId;
    Acad::ErrorStatus retStat;


    switch (ads_entsel("\nPick an Entity, any Entity:  ", en, pt)) {

    case RTNORM:   break;
    case RTNONE:   ads_alert("C'mon, I'm not gonna\nselect it for ya!");
		   return;
    case RTCAN:    ads_alert("Coward!");
		   return;
    case RTERROR:  ads_alert("RTFM!");
	           return;
    default:       ads_alert("Eh?");
	           return;
    }
    // Now, exchange the old-fangled ads_name for the new-fangled
    // object id...
    retStat = acdbGetObjectId(eId, en);
    if (retStat != Acad::eOk) {
	ads_printf("\nEither acdbGetObjectId or ads_entsel failed");
	ads_printf("\nen==(%lx,%lx), retStat==%d\n",
			   en[0], en[1], eId);
	return;
    }

    AcDbEntity* entObj;

    if ((retStat = acdbOpenObject(entObj, eId, AcDb::kForRead))
	!= Acad::eOk) {
	ads_printf("\nBad Clams for openAcDbObject: retStat==%d\n",
		   retStat);
        return;
    }

    AcDbLine* lineObj = AcDbLine::cast(entObj);

    if (lineObj == NULL) {
	// It is something else.  This should print its class and address.
	ads_printf("\nThis is an %s.\n", entObj->isA()->name());
	ads_printf("\nI only work with lines.  Pick one.\n");
	entObj->close();
    } else {
	ads_printf(/*MSG0*/"\nJust a few seconds...\n");

	struct resbuf lim_min_rb, lim_max_rb;
	if ((ads_getvar("LIMMIN", &lim_min_rb) != RTNORM) ||
	    (ads_getvar("LIMMAX", &lim_max_rb) != RTNORM)) {
            ads_printf("\nBad getvar calls\n");
	    // Always close your objects...
	    lineObj->close();
	    return;
	}

	#define LMIN 0
	#define LMID 1
	#define LMAX 2
	AcGePoint3d lim[3];

	lim[LMIN].set(lim_min_rb.resval.rpoint[X],
	              lim_min_rb.resval.rpoint[Y],
	              lim_min_rb.resval.rpoint[Z]);
	lim[LMAX].set(lim_max_rb.resval.rpoint[X],
	              lim_max_rb.resval.rpoint[Y],
	              lim_max_rb.resval.rpoint[Z]);

	lim[LMID].x = (lim[LMIN].x + lim[LMAX].x) / 2.0;
	lim[LMID].y = (lim[LMIN].y + lim[LMAX].y) / 2.0;
	lim[LMID].z = (lim[LMIN].z + lim[LMAX].z) / 2.0;

	double xinc = (lim[LMAX].x - lim[LMIN].x) / 100.0;
	double yinc = (lim[LMAX].y - lim[LMIN].y) / 100.0;

	// Save entity state.
	//
	AcGePoint3d   saveStart,
		      saveEnd,
		      hosePoint;

	saveStart = lineObj->startPoint();
	saveEnd = lineObj->endPoint();

	Adesk::UInt16 saveColor = lineObj->colorIndex();
	double        saveThickness = lineObj->thickness();

	// You can go from read enabled to write enabled with upgradeOpen
        //
	
        if (!statOk(lineObj->upgradeOpen()))
	    return;

	// Now set the thickness and color to something suitable
	// for pyramid building.
	lineObj->setThickness(xinc * 2.0);
	lineObj->setColorIndex(1);

	// Close the object, since it will be opened for write in the loop.
        if (!statOk(lineObj->close()))
	    return;

	// Now, for the death spiral....

	int i, j, k, m, n;
	Acad::ErrorStatus es;
	Adesk::Boolean firstTimeIn = Adesk::kTrue;

	for (i = 0; i < 50; i++) {
	    k = 0;
	    // 6 sides for a 4-sided pyramid?  Debugging Exercise.
	    for (j = 0; j < 3; j++) {
		if ((m = (k+1)) > LMAX)
		    m = LMID;
		if ((n = (m+(m-k))) > LMAX)
		    n = LMID;

		es = acdbOpenObject(lineObj, eId, AcDb::kForWrite);
		if (!statOk(es))
		    return;
		hosePoint = lineObj->startPoint();
		hosePoint[X] = lim[k][X];
		hosePoint[Y] = lim[m][Y];

		if (firstTimeIn)
		    firstTimeIn = Adesk::kFalse;
		else {
		    // Clone and append the clone
		    cloneAndAdd(lineObj);
                }

		lineObj->setStartPoint(hosePoint);
		lineObj->setColorIndex(lineObj->colorIndex() + 1);
		lineObj->close();
		    
		es = acdbOpenObject(lineObj, eId, AcDb::kForWrite);
		if (!statOk(es))
		    return;

		hosePoint = lineObj->endPoint();
		hosePoint[X] = lim[m][X];
		hosePoint[Y] = lim[n][Y];
		lineObj->setEndPoint(hosePoint);
		lineObj->setColorIndex(lineObj->colorIndex() + 1);

		// Clone and append
		cloneAndAdd(lineObj);
		lineObj->close();

	        k += 2;
		if (k > 3)
		   k = 2;
	    }

	    // Move up a step...
	    es = acdbOpenObject(lineObj, eId, AcDb::kForWrite);
	    if (!statOk(es))
		return;
	    hosePoint = lineObj->startPoint();
	    hosePoint.z += xinc;
	    lineObj->setStartPoint(hosePoint);
	    hosePoint = lineObj->endPoint();
	    hosePoint.z += xinc;
	    lineObj->setEndPoint(hosePoint);
	    es = lineObj->close();
	    if (!statOk(es))
		return;

	    // Tighten the perimeter...
	    lim[LMIN].x += xinc;
	    lim[LMAX].x -= xinc;
	    lim[LMIN].y += yinc;
	    lim[LMAX].y -= yinc;
	}

	// OK, now put the original line back...
	//
	es = acdbOpenObject(lineObj, eId, AcDb::kForWrite);
	if (!statOk(es))
	    return;
	lineObj->setStartPoint(saveStart);
	lineObj->setEndPoint(saveEnd);
	lineObj->setColorIndex(saveColor);
	lineObj->setThickness(saveThickness);
	lineObj->close();
    }
}


// PRINTEMALL
//
// Demonstrates iteration over elements in a block definition.
//
// ACDB_MODEL_SPACE == Model space, ACDB_PAPER_SPACE == Paper space.
//
void printemall()
{

    char blkName[50];
    int has_name;
    Acad::ErrorStatus es;

    has_name = ads_getstring(Adesk::kTrue,
                             "Enter Block Name <CR for current space>: ",
                             blkName);

    if (has_name)
	has_name = blkName[0] != '\0';

    AcDbBlockTable* pBlockTable;
    AcDbBlockTableRecord* pBlockTableRecord;
    AcDbBlockTableRecordIterator* pBlockIterator;

    es = acdbCurDwg()->getBlockTable(pBlockTable, AcDb::kForRead);
    if (es != Acad::eOk) {
        ads_printf("\nOpen of block table failed, es==%d\n", es);
        return;
    }

    es = pBlockTable->getAt(blkName, pBlockTableRecord, AcDb::kForRead);
    if (es != Acad::eOk) {
        ads_printf("\nOpen of block %s failed [es==%d]\n", blkName, es);
 	pBlockTable->close();
        return;
    }
    pBlockTable->close();

    // newIterator() takes a default argument that specifies which
    // end the iterator starts at.  The default is to start at the
    // beginning.
    //
    // BTW, the interface is still a little incomplete here because
    // the newIterator() also needs a decond default argument to
    // specify whether or not deleted entities are to be skipped.
    // At the moment it is possible for newIterator to return an
    // iterator that is positioned on a deleted entity !
    //
    es = pBlockTableRecord->newIterator(pBlockIterator);
    if (es != Acad::eOk) {
        ads_printf("\nNew of block iterator failed [es==%d]\n", es);
 	pBlockTableRecord->close();
        return;
    }

    // The iterator counts as one reader of the block table record,
    // so we close the record now that we have an iterator.
    //
    es = pBlockTableRecord->close();
    if (es != Acad::eOk) {
	ads_printf("\nClose of block table Record failed [es==%d", es);
	delete pBlockIterator;
    }

    for (; !pBlockIterator->done(); pBlockIterator->step()) {

        AcDbEntity* pEntity;

        es = pBlockIterator->getEntity(pEntity, AcDb::kForRead);

	if (es == Acad::eOk) {
	    ads_printf("\n%s.", pEntity->isA()->name());
	    pEntity->close();
	} else {
	    ads_printf("\nEntity open failed [es==%d]", es);
	}
    }

    delete pBlockIterator;

    ads_printf("\n");
}

/* ================= AcDbDictionary Test Suite ====================== */

// First, a custom object definition

// -- ObjectPointer class definition/implementation

class ObjectPointer: public AcDbObject
{
public:
    ACRX_DECLARE_MEMBERS(ObjectPointer);
    ObjectPointer();

    void         setId(AcDbObjectId id);
    AcDbObjectId setId() const;

    //  AcRxObject protocol.

    virtual void debugPrint() const;
    virtual Acad::ErrorStatus   dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus   dwgOutFields(AcDbDwgFiler* filer) const;

    virtual Acad::ErrorStatus   dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus   dxfOutFields(AcDbDxfFiler* filer) const;

private:
    AcDbSoftPointerId       mId;
    Adesk::Int32            mTestInt;
    double                  mTestDouble;
};

ACRX_DXF_DEFINE_MEMBERS(ObjectPointer,AcDbObject,0,OBJECT_PTR,/*MSG0*/"acdbtest");

             ObjectPointer::ObjectPointer()          { mId = 0;
                                                       mTestInt = 42; 
                                                       mTestDouble = 42.0; }

void         ObjectPointer::setId(AcDbObjectId pId)  { mId = pId; }

AcDbObjectId ObjectPointer::setId() const            { return mId; }

void         ObjectPointer::debugPrint() const
{
    assertReadEnabled();
    if (mId != NULL)
	printObjectInfo(mId);
    else
	ads_printf("  ObjectId is NULL!");
    ads_printf("  %d %f\n", mTestInt, mTestDouble);
}


Acad::ErrorStatus   
ObjectPointer::dwgInFields(AcDbDwgFiler* filer)
{
    assertWriteEnabled();
    statOk(AcDbObject::dwgInFields(filer));
    statOk(filer->readItem(&mId));
    statOk(filer->readItem(&mTestInt));
    statOk(filer->readItem(&mTestDouble));
    return filer->filerStatus();
}


Acad::ErrorStatus   
ObjectPointer::dwgOutFields(AcDbDwgFiler* filer) const
{
    assertReadEnabled();
    statOk(AcDbObject::dwgOutFields(filer));
    statOk(filer->writeItem(mId));
    statOk(filer->writeItem(mTestInt));
    statOk(filer->writeItem(mTestDouble));
    return filer->filerStatus();
}


Acad::ErrorStatus   
ObjectPointer::dxfInFields(AcDbDxfFiler* filer)
{
    assertWriteEnabled();

    resbuf             rb;

    statOk(AcDbObject::dxfInFields(filer));
    if (!filer->atSubclassData(/*MSG0*/"ObjectPointer"))
        return filer->filerStatus();

    while (filer->readResBuf(&rb) == Acad::eOk) {

	switch (rb.restype) {
	    
	case AcDb::kDxfSoftPointerId:

	    mId.setFromOldId(rb.resval.rlong);
	    break;
	    
	case AcDb::kDxfInt32:
	    
	    mTestInt   = rb.resval.rlong;
	    break;
	    
	case AcDb::kDxfReal:
	    
	    mTestDouble    = rb.resval.rreal;
	    break;
	    
	default:
	    // An unrecognized group. Push it back so that the subclass
	    // can read it again.
	    
	    filer->pushBackItem();
	    return Acad::eEndOfFile;
	} /*switch*/
    } /*while*/
 
	    
    // At this point the es variable must contain eEndOfFile - either 
    // from readResBuf() or from pushback. If not, it indicates that an 
    // error happened and we should return immediately.

    if (filer->filerStatus() != Acad::eEndOfFile)
	return Acad::eInvalidResBuf;

    return filer->filerStatus();
}


Acad::ErrorStatus   
ObjectPointer::dxfOutFields(AcDbDxfFiler* filer) const
{
    assertReadEnabled();
    statOk(AcDbObject::dxfOutFields(filer));
    statOk(filer->writeItem(AcDb::kDxfSubclass, /*MSG0*/"ObjectPointer"));
    statOk(filer->writeItem(AcDb::kDxfSoftPointerId, mId));
    statOk(filer->writeItem(AcDb::kDxfInt32, mTestInt));
    statOk(filer->writeItem(AcDb::kDxfReal, mTestDouble));
    return filer->filerStatus();
}

// --- Our "DWG header variable" dictionary name.

#define ACDB_TEST_DICT /*MSG0*/"AcDbTestDictionary"


// getDict
//
// Fetch our dictionary out of the current DWG, using our
// chosen object name.  If it isn't there, we must not have created
// it.
//

Adesk::Boolean getDict(AcDbObjectId& dictId)
{
    // Look for dictionary in the current drawing's "NamedObjects"
    // dictionary.  If not present, create a dictionary, add it to
    // the database, then give it a name, by which it can be accessed
    // forever after.
    Acad::ErrorStatus es;

    AcDbDictionary *nameList;
    if (!statOk(acdbCurDwg()->getNamedObjectsDictionary(nameList, 
                                                        AcDb::kForWrite)))
	return Adesk::kFalse;

    es = nameList->getAt(ACDB_TEST_DICT, dictId);

    if (es == Acad::eKeyNotFound) {
	AcDbDictionary* dict;
	dict = new AcDbDictionary;
	es = nameList->setAt(ACDB_TEST_DICT, dict, dictId);
	if (statOk(es))
	    statOk(dict->close());
    } 
    statOk(nameList->close());
    return (es == Acad::eOk);
}


//********* AcEditorReactor --  WBLOCK Handler for  AcDbTestDictionary

class TestWblockReactor : public AcEditorReactor {

  public:

    virtual void otherWblock(AcDbDatabase* pTo,
			     AcDbIdMapping& pIdMap,
			     AcDbDatabase* pFrom);

};

void TestWblockReactor::otherWblock(AcDbDatabase* pTo,
				    AcDbIdMapping&, // Need to use this...
			            AcDbDatabase* pFrom)
{
    if (pFrom != acdbCurDwg()) {
	ads_printf("Source Dwg != Cur Dwg. Must Upgrade test case.\n");
        return;
    }
	
    // Fetch source ACDB_TEST_DICTIONARY
    AcDbObjectId srcDictId;
    getDict(srcDictId);
    AcDbDictionary* srcDict;
    if (!statOk(acdbOpenObject(srcDict, srcDictId, AcDb::kForRead)))
	return;

    // Get the destination Named Object Dictionary
    AcDbDictionary *destNamedObjDict;
    if (!statOk(pTo->getNamedObjectsDictionary(destNamedObjDict, 
                                               AcDb::kForWrite)))
	return;
    
    // Create empty destination ACDB_TEST_DICTIONARY
    AcDbObjectId destDictId;
    AcDbDictionary* destDict;
    if (destNamedObjDict->getAt(ACDB_TEST_DICT,
	((AcDbObject*&)destDict), kForWrite) != Acad::eOk)
    {
        destDict = new AcDbDictionary;
        if (!statOk(destNamedObjDict->setAt(ACDB_TEST_DICT, destDict,
	    destDictId)))
	{
	    statOk(destNamedObjDict->close());
	    return;
	}
    }

    AcDbDictionaryIterator* i;
    for (i = srcDict->newIterator(); !i->done(); i->next()) {
	// Fetch the entry name and object
	const char* entryName = i->name();
	AcDbObject* entryObj;
        if (!statOk(i->getObject(entryObj, kForRead)))
	    break;

	ads_printf("  Wblock Cloning ObjectPointer '%'  \n", entryName);
	// Now, to clone the object and add it to the dest dictionary
	AcDbObject*  destEntry = (AcDbObject*) entryObj->clone();
	AcDbObjectId destEntryId;
	statOk(destDict->setAt(entryName, destEntry, destEntryId));

	// Clean up and close.
	statOk(entryObj->close());
	statOk(destEntry->close());
    }
    delete i;
    // Close 'em up.
    statOk(srcDict->close());
    statOk(destDict->close());
    statOk(destNamedObjDict->close());
}
 

static TestWblockReactor* wblockTest = NULL;


// MAKEDICT
//
// Look for this test suite's "DWG dictionary" in the current
// drawing header.  If it is not there, create it.  If it is
// there, a real program would return without doing anything.
//
// Exercises:
//
// AcDbDatabase::addAcDbObject()
// AcDbDatabase::addObjectName()
// AcDbDatabase::lookupAcDbObjectId()
// Custom non-entity class derived directly from AcDbObject.
// AcDbDictionary protocol
// acdbOpenObject() // all 3 modes
// AcDbObject::cancel()
// AcDbObject::close()
//
// This program opens and closes/cancels it in every combination,
// without modifying it.  Returns kFalse if no dictionary existed
// or could be created.
//

void makeDict()
{
    AcDbObjectId ourAppDictId;

    getDict(ourAppDictId);
}

void openAndShut()
{
    AcDbObjectId ourAppDictId;
    AcDbDictionary* dict;

    if (!getDict(ourAppDictId))
	return;
    statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForNotify));
    statOk(dict->close());
    statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForRead));
    statOk(dict->close());
    statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForWrite));
    statOk(dict->close());
    statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForNotify));
    statOk(dict->cancel());
    statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForRead));
    statOk(dict->cancel());
    statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForWrite));
    statOk(dict->cancel());
}

void add2Dict()
{
    AcDbObjectId ourAppDictId;

    getDict(ourAppDictId);

    AcDbDictionary* dict;
    if (!statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForWrite)))
	return;

    char nameBuffer[132];
    char* objName = &nameBuffer[0];
    int has_name;

    has_name = ads_getstring(Adesk::kTrue,
                             "Enter Dictionary Entry Key: ",
                             objName);

    if (!has_name || (*objName == '\0')) {
	ads_printf(/*MSG0*/"\nInvalid Entry Name.\n");
	dict->close();
	return;
    }

    //
    // Now, create a new test entry ObjectPointer, and add it to the dictionary
    // demonstrating a instantiation and addition of a
    // custom object class into the database.
    // 


    AcDbEntity* pEnt;
    AcDbObjectId pEntId;
    if ((pEnt = selectEntity(pEntId, AcDb::kForRead)) == NULL) {
	dict->close();
	return;
    }
    pEnt->close();  // Didn't really want do anything, just get the Id.

    ObjectPointer* newObjPtr = new ObjectPointer;

    newObjPtr->setId(pEntId);
    newObjPtr->debugPrint();

    AcDbObjectId retObjId;
    if (!statOk(dict->setAt(objName, newObjPtr, retObjId)))
        return;

    statOk(newObjPtr->close());
    statOk(dict->close());
}

void delFromDict()
{
    AcDbObjectId ourAppDictId;

    getDict(ourAppDictId);

    AcDbDictionary* dict;
    if (!statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForWrite)))
	return;

    char nameBuffer[132];
    char* objName = &nameBuffer[0];
    int has_name;

    has_name = ads_getstring(Adesk::kTrue,
                             "Enter Dictionary Entry to Remove: ",
                             objName);

    if (!has_name || (*objName == '\0')) {
	ads_printf(/*MSG0*/"\nInvalid Entry Name.\n");
	return;
    }

    if (getYorN(/*MSG0*/"By Name"))
        // Tell the dictionary to delete its entry.
	statOk(dict->remove(objName));
    else {
        // Tell the entry to delete itself, and relay such to the dictionary,
        // via notification reactors.
        AcDbObject* tObj;
	// By Object, to test back reactor mechanism. and Erase.
        if (statOk(dict->getAt(objName, tObj, kForWrite))) {
            statOk(tObj->erase(Adesk::kTrue));
            statOk(tObj->close());
        }
    }
    statOk(dict->close());
}

void listDict()
{
    AcDbObjectId ourAppDictId;

    getDict(ourAppDictId);

    AcDbDictionary* dict;
    if (!statOk(acdbOpenObject(dict, ourAppDictId, AcDb::kForRead)))
	return;

    ads_printf(/*MSG0*/"\nDictionary Contents\n===================\n");
    AcDbDictionaryIterator* i;
    for (i = dict->newIterator(); !i->done(); i->next()) {
	const char* entryName = i->name();
        ads_printf(/*MSG0*/"Name: %s;  ", entryName);
	printObjectInfo(i->objectId());
    }
    delete i;
    ads_printf
    (/*MSG0*/"==========================\nEnd of Dictionary Contents\n");

    statOk(dict->close());
}

void makeABlock()
{
     // Create and name a new block table record.
     //
     AcDbBlockTableRecord* blockTableRec = new AcDbBlockTableRecord();
     blockTableRec->setName("ACDBNOATTR");
  
     // Get block symbol table.
     //
     AcDbDatabase* db = acdbCurDwg();
     AcDbBlockTable* blockTable = NULL;
     if (!statOk(db->getBlockTable(blockTable, AcDb::kForWrite)))
	 return;
  
     // Add the new block table record to the block symbol table.
     //
     AcDbObjectId blockTableRecordId;
  
     if (!statOk(blockTable->add(blockTableRecordId, blockTableRec)))
         return;

     blockTable->close();
              
     // Add an entity to the component's block record.
     // Create a line.
     //
     AcDbLine* line = new AcDbLine();
     AcDbObjectId lineId;
  
     line->setStartPoint(AcGePoint3d(3,3,0));
     line->setEndPoint(AcGePoint3d(6,6,0));
     line->setColorIndex(3);
  
     statOk(blockTableRec->appendAcDbEntity(lineId, line));
     statOk(line->close());
     statOk(blockTableRec->close());
}


//
// Custom objects and entities for persistent reactors and extension
// dictionary testing.
//

// Class to allow creation of an AcDbObject.
//
class AcDbMyObj: public AcDbObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcDbMyObj);
    AcDbMyObj() {};
};

ACRX_DXF_DEFINE_MEMBERS(AcDbMyObj,AcDbObject,0,MYOBJ,/*MSG0*/"extdict");


// Class to allow creation of an AcDbEntity.
//
class AcDbMyEnt: public AcDbEntity 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcDbMyEnt);
    AcDbMyEnt() {};

    virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;
};

ACRX_DXF_DEFINE_MEMBERS(AcDbMyEnt,AcDbEntity,0,MYENT,/*MSG0*/"persreac");

Acad::ErrorStatus AcDbMyEnt::dwgInFields(AcDbDwgFiler* filer)
{
  AcDbEntity::dwgInFields(filer);
  ads_printf("Inside dwgInFields, id = %d\n", objectId().asOldId() );
  return Acad::eOk;
}

Acad::ErrorStatus AcDbMyEnt::dwgOutFields(AcDbDwgFiler* filer) const
{
  AcDbEntity::dwgOutFields(filer);
  ads_printf("Inside dwgOutFields, id = %d\n", objectId().asOldId() );
  return Acad::eOk;
}

// Class to create an AcDbObject customized with persistent reactors.
//
class AcDbCustObj: public AcDbObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcDbCustObj);
    AcDbCustObj() {};

    virtual void              cancelled(const AcDbObject* dbObj)
			          { ads_printf("\nInside cancelled.\n"); }

    virtual void              copied(const AcDbObject* dbObj,
				     const AcDbObject* newObj)
			          { ads_printf("\nInside copied.\n"); }

    virtual void              erased(const AcDbObject* dbObj,
				     Adesk::Boolean pErasing = Adesk::kTrue)
			          { ads_printf("\nInside erased.\n"); }

    virtual void              goodbye(const AcDbObject* dbObj)
			          { ads_printf("\nInside goodbye.\n"); }

    virtual void              openedForModify(const AcDbObject* dbObj)
			          { ads_printf("\nInside openedForModify.\n"); }

    virtual void              modified(const AcDbObject* dbObj)
			          { ads_printf("\nInside modified.\n"); }

    virtual void              modifyUndone(const AcDbObject* dbObj)
			          { ads_printf("\nInside modifyUndone.\n"); }

    virtual void              modifiedXData(const AcDbObject* dbObj)
			          { ads_printf("\nInside modifiedXData.\n"); }
};

ACRX_DXF_DEFINE_MEMBERS(AcDbCustObj,AcDbObject,0,CUSTOBJ,/*MSG0*/"persreac");


// Class to create an AcDbEntity customized with persistent reactors.
//
class AcDbCustEnt: public AcDbEntity 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcDbCustEnt);
    AcDbCustEnt() {};
    virtual void modifiedGraphics(const AcDbEntity*)
		 { ads_printf("\nInside modifiedGraphics.\n"); }
};

ACRX_DXF_DEFINE_MEMBERS(AcDbCustEnt,AcDbEntity,0,CUSTENT,/*MSG0*/"persreac");


//
// ------- Functions to test extension dictionary.
//
void createExtDict()
{
  // Get the entity whose entity is to be extended.
  //
  ads_name exEname;
  ads_point pt;
  ads_entsel("Pick object to extend", exEname, pt);

  // Convert extension entity's name to AcDbObjectId.
  //
  AcDbObjectId exEntId;
  Acad::ErrorStatus status = acdbGetObjectId(exEntId, exEname);

  if (status == Acad::eOk)
  {
     // Now open the extension entity.
     //
     AcDbEntity* pExtEnt;
     status = acdbOpenObject( pExtEnt, exEntId, AcDb::kForWrite );

     if (status == Acad::eOk)
     {
        // Iterate through selection set adding each entity into the
        // extension dictionary.
        //
        status = pExtEnt->createExtensionDictionary();

        if (status == Acad::eOk)
        {
	   AcDbObjectId extDictId = pExtEnt->extensionDictionary();

	   AcDbDictionary* pExtDict = NULL;
           status = acdbOpenObject( pExtDict, extDictId, AcDb::kForWrite );

	   if (status == Acad::eOk)
           {
	       // Add four arbitrary objects to the extension object's
	       // extension dictionary.
	       //
	       AcDbObject* pObj;
	       AcDbObjectId dummyId;
	       char nextName[10];

	       for (int i=0; i<4; i++)
	       {
		   sprintf(nextName, "TEST%d", i);
		   pObj = new AcDbMyObj();
		   status = pExtDict->setAt(nextName, (AcDbObject*)pObj, 
                                            dummyId );
		   if (status != Acad::eOk)
		   {
		       ads_printf("\nError updating dictionary, stat = %d, i = %d\n",
				  status, i);
		   }
		   else
		     pObj->close();
	       }

	       pExtDict->close();
	   }
	   else
	     ads_printf("\nError opening extension dictionary, stat = %d\n",
		        status);
	}
	else
	  ads_printf("\nError creating extension dictionary, stat = %d\n",
                       status);

        pExtEnt->close();
     }
     else
       ads_printf("\nError opening extension object, stat = %d\n", status);
  }
  else
     ads_printf("\nError getting extension object id, stat = %d\n", status);
}

void listExtDict()
{
  // Get the entity whose entity is to be extended.
  //
  ads_name exEname;
  ads_point pt;
  ads_entsel("Pick object to list", exEname, pt);

  // Convert extension entity's name to AcDbObjectId.
  //
  AcDbObjectId exEntId;
  Acad::ErrorStatus status = acdbGetObjectId(exEntId, exEname);

  if (status == Acad::eOk)
  {
     // Now open the extension entity.
     //
     AcDbEntity* pExtEnt;
     status = acdbOpenObject( pExtEnt, exEntId, AcDb::kForWrite );

     if (status == Acad::eOk)
     {
	   AcDbObjectId extDictId = pExtEnt->extensionDictionary();

	   AcDbDictionary* pExtDict = NULL;
           status = acdbOpenObject( pExtDict, extDictId, AcDb::kForWrite );

	   if (status == Acad::eOk)
           {
	       // Iterate through the extension dictionary and display each 
               // entity's id.
	       //
	       AcDbDictionaryIterator* pIter;
	       AcDbObjectId id;

	       for (pIter = pExtDict->newIterator(); !pIter->done(); 
		    pIter->next())
	       {
	         id = pIter->objectId();
	         ads_printf("\nid: %d\n", id.asOldId() );
	       }
	       delete pIter;

	       pExtDict->close();
	   }
	   else
	     ads_printf("\nError opening extension dictionary, stat = %d\n",
		        status);

           pExtEnt->close();
     }
     else
       ads_printf("\nError opening extension object, stat = %d\n", status);
  }
  else
     ads_printf("\nError getting extension object id, stat = %d\n", status);
}

//
// ------- Functions to test persistent reactors.
//

void testNotify(AcDbObjectId testId)
{
    AcDbEntity* tEnt;
    AcDbObject* tObj;
    AcDbObjectId tId;
    Acad::ErrorStatus status = Acad::eOk;

    // Open the passed entity.
    //
    status = acdbOpenAcDbEntity(tEnt, testId, AcDb::kForRead);
    if (status != Acad::eOk) {
       ads_alert("Problem opening entity.");
       return;
    }

    // Iterate through all the reactors looking for persistent reactors.
    // If found, call its protocol.
    //
    AcDbObjectReactor* tmpReac;
    AcDbVoidPtrArray *pReactors;
    if ((pReactors = tEnt->reactors()) != NULL) {
        for (int i = 0; i < pReactors->length(); i++) {
	   tmpReac = (AcDbObjectReactor*)(pReactors->at(i));

	   if (acdbIsPersistentReactor(tmpReac)) {

	       // Get the id stored with the persistent reactor and open it
	       // to get the AcDbObject pointer.
	       //
	       tId = acdbPersistentReactorObjectId(tmpReac);
	       status = acdbOpenAcDbObject(tObj, tId, AcDb::kForNotify);
	       if (status != Acad::eOk) {
		  ads_alert("Problem opening entity.");
		  return;
	       }


	       // Call all reactor protocol, just to make sure we get there.
	       //
	       tObj->cancelled(tObj);
	       tObj->copied(tObj, tObj);
	       tObj->erased(tObj);
	       tObj->goodbye(tObj);
	       tObj->openedForModify(tObj);
	       tObj->modified(tObj);
	       tObj->modifyUndone(tObj);
	       tObj->modifiedXData(tObj);

	       // Close the object.
	       //
	       tObj->close();
	    }
	}
    }
    tEnt->close();
}


void createPersReac()
{
    Acad::ErrorStatus status;

    // Create a customized object.
    //
    AcDbCustObj* pCustObj = new AcDbCustObj();
    AcDbObjectId custObjId;
    status = acdbCurDwg()->addAcDbObject( custObjId, pCustObj );
    if (status != Acad::eOk)
    {
        ads_printf("\nError adding custom object, stat = %d\n", status);
	return;
    }
    pCustObj->close();

    // Create an object whose activity will be monitored.  Notification
    // will be sent to pCustObj.
    //
    AcDbMyObj* pMyObj = new AcDbMyObj();
    AcDbObjectId myObjId;
    status = acdbCurDwg()->addAcDbObject( myObjId, pMyObj );
    if (status != Acad::eOk)
    {
        ads_printf("\nError adding my object, stat = %d\n", status);
	return;
    }
    ads_printf("\nmyObjId: %d\n", myObjId.asOldId() );
    ads_printf("\ncustObjId: %d\n", custObjId.asOldId() );
    pMyObj->addPersistentReactor( custObjId );
    pMyObj->close();

    // Now create a persistent entity reactor.
    //
    ads_name ename1;
    ads_point pt;
    ads_entsel("Pick entity to monitor", ename1, pt);

    ads_name ename2;
    ads_entsel("Pick entity to notify", ename2, pt);

  AcDbObjectId entId1;
  status = acdbGetObjectId(entId1, ename1);

  AcDbObjectId entId2;
  status = acdbGetObjectId(entId2, ename2);

  if (status == Acad::eOk)
  {
     // Now open the entity and add the reactor.
     //
     AcDbEntity* pEnt1;
     status = acdbOpenObject( pEnt1, entId1, AcDb::kForWrite );

     if (status == Acad::eOk)
     {
        ads_printf("\nentId2: %d\n", entId2.asOldId() );
	pEnt1->addPersistentReactor( entId2 );
        pEnt1->close();
     }
     else
       ads_printf("\nError opening extension object1, stat = %d\n", status);
  }
  else
     ads_printf("\nError getting object id, stat = %d\n", status);
  testNotify(entId1);
    
}


void testPersReac()
{
    Acad::ErrorStatus status;

    // Get object id's
    //
    AcDbObjectId myObjId, myEntId;
    double r;
    ads_getreal("Enter myObjId:", &r);
    myObjId.setFromOldId((long)r);

    ads_name ename1;
    ads_point pt;
    ads_entsel("Pick entity to monitor", ename1, pt);

    acdbGetObjectId(myEntId, ename1);

ads_printf("\nTesting object\n");
    // Test the "modified" callbacks.
    //
    AcDbObject* pObj;
    status = acdbOpenObject( pObj, myObjId, AcDb::kForWrite );
    if (status == Acad::eOk) {
        ( (AcDbMyObj*)pObj )->createExtensionDictionary();
        pObj->close();
    }
    else {
      ads_printf("\nError opening my object, stat = %d\n", status);
    }

ads_printf("\nTesting entity\n");

    status = acdbOpenObject( pObj, myEntId, AcDb::kForWrite );
    if (status == Acad::eOk) {
        ( (AcDbEntity*)pObj )->setColorIndex(1);
        pObj->close();
    }
    else {
      ads_printf("\nError opening my entity, stat = %d\n", status);
    }
}

void delPersReac()
{
    Acad::ErrorStatus status;

    // Get object id's
    //
    AcDbObjectId myObjId, myEntId, custObjId, entId2;
    double r;
    ads_getreal("Enter myObjId:", &r);
    myObjId.setFromOldId((long)r);

    ads_getreal("Enter custObjId:", &r);
    custObjId.setFromOldId((long)r);

    ads_getreal("Enter entId2:", &r);
    entId2.setFromOldId((long)r);

    ads_name ename1;
    ads_point pt;
    ads_entsel("Pick entity to monitor", ename1, pt);

    acdbGetObjectId(myEntId, ename1);

ads_printf("\nTesting object\n");
    // Remove the reactor from the object.
    //
    AcDbObject* pObj;
    status = acdbOpenObject( pObj, myObjId, AcDb::kForWrite );
    if (status == Acad::eOk) {
        AcDbMyObj* pMyObj = (AcDbMyObj*)pObj;
        pMyObj->removePersistentReactor( custObjId );
        pMyObj->close();
    }
    else {
      ads_printf("\nError opening my object, stat = %d\n", status);
    }

ads_printf("\nTesting entity\n");

    status = acdbOpenObject( pObj, myEntId, AcDb::kForWrite );
    if (status == Acad::eOk) {
        ( (AcDbEntity*)pObj )->removePersistentReactor( entId2 );
        pObj->close();
    }
    else {
      ads_printf("\nError opening my entity, stat = %d\n", status);
    }
}


void moveBy (AcGePoint3d &p, AcGePoint3d p0)
//
//  Translate point p by point p0
//
{
   p.x += p0.x ;
   p.y += p0.y ;
   p.z += p0.z ;

}

int defineDABlock (AcDbObjectId *blockId,
		   AcGePoint3d   p0,
		   ads_real      height,
		   ads_real      angle)
//
//  Creates DA entity 
//
{
   int                     retCode = 0 ;
   AcDbDatabase            *pDatabase = acdbCurDwg() ;
   AcDbBlockTable          *pBlockTable = NULL ;
   AcDbBlockTableRecord    *pBlockRecord = NULL ;
   AcDbObjectId            entityId ;

   do {

       if (pDatabase == NULL) {
           break ;
       }

       // create new block table record
       if ((pBlockRecord = new AcDbBlockTableRecord) == NULL ) {
           break ;
       } 
       
       // set block name
       pBlockRecord->setName("ACDBATTR");
       pBlockRecord->setOrigin (p0) ;


       // open block table for write
       if (pDatabase->getBlockTable(pBlockTable, AcDb::kForWrite) != Acad::eOk) {
           break ;
       }

       // Add BTR to table.
       if (pBlockTable->add(*blockId, pBlockRecord) != Acad::eOk) {
           break ;
       }

       // Add a circle.
       AcDbCircle* pCircle = new AcDbCircle;
       pCircle->setCenter(p0);
       pCircle->setRadius(height * 4.0);
       pCircle->setColorIndex(3);

       statOk(pBlockRecord->appendAcDbEntity(entityId, pCircle));
       statOk(pCircle->close());

       AcDbAttributeDefinition         *pAttdef ;
       if ((pAttdef = new AcDbAttributeDefinition) == NULL)
           break ;
       pAttdef->setPosition (p0) ;
       pAttdef->setHeight(height) ;
       pAttdef->setRotation(angle) ;
       pAttdef->setHorizontalMode(AcDb::kTextLeft) ;
       pAttdef->setVerticalMode(AcDb::kTextBase) ;

       pAttdef->setPrompt("Prompt") ;
       pAttdef->setTextString("DEFAULT") ;
       pAttdef->setTag("Tag") ;
       pAttdef->setInvisible(Adesk::kFalse) ;
       pAttdef->setVerifiable(Adesk::kFalse) ;
       pAttdef->setPreset(Adesk::kFalse) ;
       pAttdef->setConstant(Adesk::kFalse) ;
       pAttdef->setFieldLength(25) ;

       if (pBlockRecord->appendAcDbEntity(entityId, pAttdef) != Acad::eOk)
           break;

       // Now, get going on a second Attdef Entity.
       AcDbAttributeDefinition         *pAttdef2;
       if (!(pAttdef2 = AcDbAttributeDefinition::cast(pAttdef->clone())))
           break ;

       // Then close the first.
       if (pAttdef->close() != Acad::eOk)
           break;

       p0.y -= pAttdef2->height();
       pAttdef2->setPosition (p0) ;
       pAttdef2->setColorIndex(1); // aka Red
       pAttdef2->setConstant(Adesk::kTrue);

       if (pBlockRecord->appendAcDbEntity(entityId, pAttdef2) != Acad::eOk)
           break;

       if (pAttdef2->close() != Acad::eOk)
           break;

       // add block end
       retCode = 1 ;

   } while (0) ;

   if (pBlockRecord != NULL) {
       pBlockRecord->close() ;
   }
   if (pBlockTable != NULL) {
       pBlockTable->close() ;
   }

   return retCode ;
}

void addDA()
{
   AcDbDatabase            *pDatabase = acdbCurDwg() ;
   AcDbBlockTable          *pBlockTable = NULL ;
   AcDbBlockTableRecord    *pBlockTableRecord = NULL ;
   AcDbObjectId            blockId ;
   AcDbObjectId            newEntId = 0;
   AcDbBlockReference      *pBlkRef = NULL ;

   AcGeVector3d normal(0.0, 0.0, 1.0);
   AcGeVector3d xaxis(1.0, 0.0, 0.0);

   do {

       ads_point pp0 ;
       if (ads_getpoint (NULL, "\nEnter left point: ",  pp0) != RTNORM) {
           break ;
       }

       ads_real angle ;
       if (ads_getangle (pp0, "\nEnter rotation angle: ", &angle)
                                                        != RTNORM) {
           break ;
       }

       ads_real height ;
       if (ads_getdist (pp0, "\nEnter height: ", &height) != RTNORM) {
           break ;
       }

       // create block definition

       // Need one of them classy points.
       AcGePoint3d p0;

       p0.x = pp0[X];
       p0.y = pp0[Y];
       p0.z = pp0[Z];

       if (defineDABlock(&blockId, p0, height, angle) == 0) {
           break ;
       }

       if ((pBlkRef = new AcDbBlockReference) == NULL) {
           break ;
       } 

       if (pBlkRef->setBlockTableRecord(blockId) != Acad::eOk) {
           break ;
       }

       // Give it the current UCS normal.
	struct resbuf to, from;

	from.restype = RTSHORT;
	from.resval.rint = 1; // UCS (Pass in AcGeVector3d::kIdentity below)
	to.restype = RTSHORT;
	to.resval.rint = 0;   // WCS
	ads_trans(asDblArray(normal), &from, &to, 
		  Adesk::kTrue, asDblArray(normal));

	// Also need the position in WCS
	ads_trans(asDblArray(p0), &from, &to, 
		  Adesk::kTrue, asDblArray(p0));

       // The LCS 0.0 angle:  not necessarily the UCS 0.0 angle.
       pBlkRef->setRotation(0.0) ;
       pBlkRef->setNormal(normal);
       // points must always be set after the normal.
       pBlkRef->setPosition(p0) ;

       // open current database
       if (pDatabase->getBlockTable(pBlockTable, AcDb::kForRead) != Acad::eOk) {
           break ;
       }
   
       if (pBlockTable->getAt(ACDB_MODEL_SPACE, pBlockTableRecord, kForWrite)
	   != Acad::eOk)
       {
           break ;
       }
   
       if (pBlockTableRecord->appendAcDbEntity(newEntId, pBlkRef) != Acad::eOk){
           break ;
       }

       AcDbObject              *pObj;
       if (acdbOpenAcDbObject(pObj, blockId, AcDb::kForRead) != Acad::eOk) {
           break;  
       }
       AcDbBlockTableRecord    *pBTR = AcDbBlockTableRecord::cast(pObj) ;

       // Temporary code
       AcDbBlockTableRecordIterator      *pAttdefIterator ;
       if (pBTR->newIterator(pAttdefIterator) != Acad::eOk) {
           break;  // Error condition
       }

       for (pAttdefIterator->start(Adesk::kTrue) ;
            pAttdefIterator->done() != Adesk::kTrue ; 
            pAttdefIterator->step()) {

           AcDbEntity* ent;
           if (pAttdefIterator->getEntity(ent, kForRead) == Acad::eOk) {

               AcDbAttributeDefinition *pAttdef;
               if ((pAttdef = AcDbAttributeDefinition::cast(ent)) != NULL &&
		   !pAttdef->isConstant()) {

                   AcDbAttribute* att = new AcDbAttribute();
		   att->setPropertiesFrom(pAttdef);
                   att->setInvisible(pAttdef->isInvisible());

		   // Translate attribute by block reference.
		   // To be really correct, entire block reference
		   // transform should be applied here.  WCA.
		   p0 = pAttdef->position();
		   moveBy(p0, pBlkRef->position());
		   att->setPosition (p0) ;

		   att->setHeight(pAttdef->height()) ;
		   att->setRotation(pAttdef->rotation()) ;

		   att->setHorizontalMode(AcDb::kTextLeft) ;
		   att->setVerticalMode(AcDb::kTextBase) ;

		   att->setTag("Tag") ;
		   att->setFieldLength(25) ;


                   char *tStr = pAttdef->tag(); 
                   att->setTag(tStr);
                   free (tStr) ;

                   att->setFieldLength(pAttdef->fieldLength());

                   // Database Column value should be displayed
		   // INSERT would prompt for this...
		   att->setTextString("Assigned Attribute Value");

                   AcDbObjectId attId ;    

                   if (pBlkRef->appendAttribute(attId, att) != Acad::eOk) {
                       break ;
                   }
                   att->close() ;
               }
           }
       }
       delete pAttdefIterator;

   } while (0) ;

   if (pBlockTableRecord != NULL) {
       pBlockTableRecord->close();
   }
   if (pBlockTable != NULL) {
       pBlockTable->close();
   }
   if (pBlkRef != NULL) {
       pBlkRef->close();
   }
}  

void bumpColorGuts(AcDbEntity* pEnt)
{
    statOk(pEnt->setColorIndex((pEnt->colorIndex() % 256) + 1));
    statOk(pEnt->close());
}

void bumpColor()
{
    AcDbEntity* pEnt;
    AcDbObjectId pEntId;

    if ((pEnt = selectEntity(pEntId, AcDb::kForWrite)) == NULL)
	return;

    bumpColorGuts(pEnt);
}

void bumpColorHand()
{
    AcDbEntity*  pEnt;

    if ((pEnt = AcDbEntity::cast(selectObject(AcDb::kForWrite))) != NULL)
	bumpColorGuts(pEnt);
}

void selectASet()
{
    ads_name ss;
    ads_point pt1, pt2;

    int rc;

    if ((ads_getpoint(NULL, "\nS-s-select an Entity:  ", pt1)) != RTNORM)
	return;
    if ((rc = ads_ssget(NULL, pt1, pt2, NULL, ss)) != RTNORM)
	ads_printf("Nothing Selected, Return Code==%d\n", rc);
    else {
	// Let it go, we were just having fun...
	if ((rc = ads_ssfree(ss)) != RTNORM)
	    ads_printf("Failed ads_ssfree, Return Code==%d\n", rc);
    }
}


static AcDbObject* openedObject;

static void closeOpenedObject() {
    if (openedObject == NULL)
	return;

    statOk(openedObject->close());
    openedObject = NULL;
}

void kcsCurse()
{
    Adesk::Boolean reopenObject = Adesk::kFalse;
    int openCloseCode;
    AcDb::OpenMode openMode;

    if (openedObject != NULL) {
	if (getYorN("An object is open under KC's curse.  Close it?")) {
	    // Close till yer closed.
	    while (!openedObject->isReallyClosing())
		closeOpenedObject();
	    closeOpenedObject();
	} else if (getYorN("Cancel it?")) {
	    while (!openedObject->isReallyClosing())
		statOk(openedObject->cancel());
	    statOk(openedObject->cancel());
	    openedObject = NULL;
	} else {
	   ads_printf("\nIn that case, we'll open it some more...\n");
	    reopenObject = Adesk::kTrue;
        }

    }

    if (ads_getint(
	"Pick an Open Mode code: 0 = Read; 1 = Write; 2 = Notify",
        &openCloseCode) != RTNORM) 
        return;
    switch(openCloseCode) {
	case 0:  openMode = AcDb::kForRead;
		 break;
	case 1:  openMode = AcDb::kForWrite;
		 break;
	case 2:  openMode = AcDb::kForNotify;
		 break;
	default: ads_printf("\nNo open performed.");
		 return;
    }

    if (reopenObject)
	statOk(acdbOpenAcDbObject(openedObject,
				  openedObject->objectId(),
				  openMode));
    else
        if ((openedObject = selectObject(openMode)) != NULL)
	    // And we now have a time bomb in the system...
	    ads_printf("\nOpen Successful.\n");
}

/* =================== ARx Module Interface Functions ================ */
 
AcRxDictionary * acdbCmds = NULL;

// Initialization Function

void initModule()
{
    openedObject = NULL;

    // Add some commands to test the AcDb API Library

    acedRegCmds->addCommand("ACDB_TEST",
                            "DUMP",
                            "DUMP",
                            ACRX_CMD_MODAL,
                            &dump);

    acedRegCmds->addCommand("ACDB_TEST",
                            "SWAP",
                            "SWAP",
                            ACRX_CMD_MODAL,
                            &swap);

    acedRegCmds->addCommand("ACDB_TEST",
                            "BLOCKWITHATTR",
                            "BLOCKWITHATTR",
                            ACRX_CMD_MODAL,
                            &addDA);

    acedRegCmds->addCommand("ACDB_TEST",
                            "MAKEABLOCK",
                            "MAKEABLOCK",
                            ACRX_CMD_MODAL,
                            &makeABlock);

    acedRegCmds->addCommand("ACDB_TEST",
                            "MAKEDICT",
                            "MAKEDICT",
                            ACRX_CMD_MODAL,
                            &makeDict);

    acedRegCmds->addCommand("ACDB_TEST",
                            "ADD2DICT",
                            "ADD2DICT",
                            ACRX_CMD_MODAL,
                            &add2Dict);

    acedRegCmds->addCommand("ACDB_TEST",
                            "LISTDICT",
                            "LISTDICT",
                            ACRX_CMD_MODAL,
                            &listDict);

    acedRegCmds->addCommand("ACDB_TEST",
                            "DELFROMDICT",
                            "DELFROMDICT",
                            ACRX_CMD_MODAL,
                            &delFromDict);

    acedRegCmds->addCommand("ACDB_TEST",
                            "ADDLINE",
                            "ADDLINE",
                            ACRX_CMD_MODAL,
                            &addLine);

    acedRegCmds->addCommand("ACDB_TEST",
                            "OBELISK",
                            "OBELISK",
                            ACRX_CMD_MODAL,
                            &obelisk);

    acedRegCmds->addCommand("ACDB_TEST",
                            "PRINTEMALL",
                            "PRINTEMALL",
                            ACRX_CMD_TRANSPARENT,
                            &printemall);

    acedRegCmds->addCommand("ACDB_TEST", 
                            "CREATEEXTDICT", 
                            "CREATEEXTDICT",
			    ACRX_CMD_MODAL, 
                            &createExtDict);

    acedRegCmds->addCommand("ACDB_TEST",
                            "LISTEXTDICT",
                            "LISTEXTDICT",
			    ACRX_CMD_MODAL,
                            &listExtDict); 

    acedRegCmds->addCommand("ACDB_TEST", 
                            "CREATEPERSREAC", 
                            "CREATEPERSREAC",
			    ACRX_CMD_MODAL, 
                            &createPersReac);

    acedRegCmds->addCommand("ACDB_TEST", 
                            "TESTPERSREAC",
                            "TESTPERSREAC",
			    ACRX_CMD_MODAL,
                            &testPersReac);

    acedRegCmds->addCommand("ACDB_TEST",
                            "DELPERSREAC", 
                            "DELPERSREAC",
			    ACRX_CMD_MODAL,
                            &delPersReac);

    acedRegCmds->addCommand("ACDB_TEST",
                            "BUMPCOLOR",
                            "UNSETTLECOLOUR",
                            ACRX_CMD_MODAL,
                            &bumpColor);


    acedRegCmds->addCommand("ACDB_TEST",
                            "BUMPCOLORH",
                            "UNSETTLECOLOURH",
                            ACRX_CMD_MODAL,
                            &bumpColorHand);


    acedRegCmds->addCommand("ACDB_TEST",
                            "KCS_CURSE",
                            "KCS_CURSE",
                            ACRX_CMD_MODAL,
                            &kcsCurse);


    acedRegCmds->addCommand("ACDB_TEST",
                            "ADD_PLINE",
                            "ACQ_PLINE",
                            ACRX_CMD_MODAL,
                            &addPline);


    acedRegCmds->addCommand("ACDB_TEST",
                            "ADD_DIM",
                            "ACQ_DIM",
                            ACRX_CMD_MODAL,
                            &addDim);

    acedRegCmds->addCommand("ACDB_TEST",
                            "SELECT_A_SET",
                            "SELECT_A_SET",
                            ACRX_CMD_TRANSPARENT,
                            &selectASet);



    ObjectPointer::rxInit();
    AcDbMyObj::rxInit();
    AcDbMyEnt::rxInit();
    AcDbCustObj::rxInit();
    AcDbCustEnt::rxInit();
    acrxBuildClassHierarchy();

    wblockTest = new TestWblockReactor;
}



AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void*) {
    switch(msg) {
	case AcRx::kInitAppMsg:
	    initModule();
	    break;
	case AcRx::kUnloadADSMsg:
	    closeOpenedObject();
	    break;
	case AcRx::kUnloadAppMsg:
	    acedEditor->removeReactor(wblockTest);
	    delete wblockTest;
            acedRegCmds->removeGroup("ACDB_TEST");
	    delete acdbCmds;
            acrxClassDictionary->remove("AcDbMyObj");
            acrxClassDictionary->remove("AcDbMyEnt");
            acrxClassDictionary->remove("AcDbCustObj");
            acrxClassDictionary->remove("AcDbCustEnt");
	    acrxClassDictionary->remove("ObjectPointer");
        default:
	    break;
    }
    return AcRx::kRetOK;
}
