//
// $Id: command.cc,v 1.19 1995/12/20 17:04:44 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// COMMAND.CC 
//
// CREATED BY: Keshav Sahoo		     May 1994 
//
// DESCRIPTION:
//
// This file contains the command interface code for Arx sample app -
// POLY.


#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
#include "aced.h"

#include "adslib.h"
#include "dbents.h"
#include "dbregion.h"
#include "dbsol3d.h"
#include "dbdict.h"
#include "dbsymtb.h"

#include "gepnt3d.h"
#include "gevec3d.h"
#include "gemat3d.h"
#include "geassign.h"

#include "poly.h"
#include "util.h"

#include "dbsubeid.h"
#include "actrans.h"


 
 
static int dbExtractEntityInfo(struct resbuf *rb,
 			      ads_name  ename)
{
    if ( rb == NULL || rb->restype != RTLB )
    {
      assert(0);
      return 0;
    }
    
    // Get the selection method.
    //
    rb = rb->rbnext;  // Bump up to the selection method, always after RTLB.
    int sel_method = rb->resval.rint;
 
    // Get the first ename (could be either the actual entity name or
    // subentity name).
    //
    rb = rb->rbnext;  // Bump up to the first name, always after sel method.
    ename[0] = rb->resval.rlname[0];
    ename[1] = rb->resval.rlname[1];
    //subname[0] = rb->resval.rlname[0];
    //subname[1] = rb->resval.rlname[1];
 
    // Get marker info.
    //
    //rb = rb->rbnext;
    //marker = rb->resval.rint;
 
    return 1;
}
 
 
 
void 
testOnScandr()
{
   // start a transaction
   actrTransactionManager->startTransaction();
 
   // pick AcDb3dSolid
   ads_name  solid, sset;
   ads_printf("\nSelect Solid:");
   ads_ssget(NULL, NULL, NULL, NULL, sset);
   struct resbuf *rb;
 	if (ads_ssnamex(&rb, sset, 0) != RTNORM) {
 		ads_ssfree(sset);
     return;
   }
   dbExtractEntityInfo(rb, solid);
   AcDbObjectId solidId;
 	acdbGetObjectId(solidId, solid);
 
   // pick pline
   ads_name  pline, sset2;
   ads_printf("\nSelect pline:");
   ads_ssget(NULL, NULL, NULL, NULL, sset2);
   struct resbuf *rb2;
 	if (ads_ssnamex(&rb2, sset2, 0) != RTNORM) {
 		ads_ssfree(sset2);
     return;
   }
   dbExtractEntityInfo(rb2, pline);
   AcDbObjectId plineId;
 	acdbGetObjectId(plineId, pline);
   
   // open solid for write
   AcDbEntity* pSolidObj;
 	acdbOpenAcDbEntity(pSolidObj, solidId, AcDb::kForWrite);
 
   // entget pline
   struct resbuf *rb3;
   rb3 = ads_entget(pline);
 
   // entmod the pline
   ads_entmod(rb3);  //<--- crash happens here

   pSolidObj->close();
 
   // end transaction
   actrTransactionManager->endTransaction();
}
 
void
addXDict()
{
    ads_name ename;
    ads_point dummy;

    // have user select an entity.  Then get it's data
    //
    ads_entsel("\nselect entity: ", ename, dummy);

    AcDbObjectId objId;
    AOK(acdbGetObjectId(objId, ename));

    AcDbObject* pObj;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForWrite));

    if (pObj->extensionDictionary() == NULL) {
        pObj->createExtensionDictionary();
    }
    pObj->close();
}


// Allows the user to select an entity.  It then checks to see if the
// selected entity has an extension dictionary.  If not,  it creates
// one and adds it to the entity.  It then creates an xrecord,  fills it
// in with some data,  and adds it to the extension dictionary
// associated with the key "TEST_REC".
//
// This function assumes that if an extension dictionary already exists
// it does not have an entry with the key "TEST_REC".
//
void
createXrecord()
{
    struct resbuf *pXrec, *pEnt, *pDict, *pTemp, *pTemp2;
    ads_point dummy,
              testpt = {1.0, 2.0, 0.0};
    ads_name xrecname, ename, extDict = {0L, 0L};

    int err;
    struct resbuf errno;

    // have user select an entity.  Then get it's data
    //
    ads_entsel("\nselect entity: ", ename, dummy);
    pEnt = ads_entget(ename);

    // Now check to see if the entity already has an extension
    // dictionary
    //
    for (pTemp = pEnt;pTemp->rbnext != NULL;pTemp = pTemp->rbnext) {
        if (pTemp->restype == 102) {
            if (!strcmp("{ACAD_XDICTIONARY", pTemp->resval.rstring)) {
                ads_name_set(pTemp->rbnext->resval.rlname, extDict);
                break;
            }
        }
    }
    // If no extension dictionary exists add one
    //
    if (extDict[0] == 0L) {
        pDict = ads_buildlist(RTDXF0, "DICTIONARY", 100,
            "AcDbDictionary", 0);
        if ((err = ads_entmakex(pDict, extDict)) != RTNORM) {
            ads_getvar("ERRNO", &errno);
            ads_printf("error is : %d", errno.resval.rint);
        }
        ads_relrb(pDict);
        pDict = ads_buildlist(102, "{ACAD_XDICTIONARY", 360, extDict,
            102, "}", 0);
        
	// set pTemp to point to the resbuf just before where we want to
	// insert the 102/360/102 list
	//
        for (pTemp = pEnt;pTemp->rbnext->restype != 100;pTemp = pTemp->rbnext)
            { ; }

	// set pTemp2 to point to the last resbuf in the 102/360/102 list
	//
        for (pTemp2 = pDict;pTemp2->rbnext != NULL;pTemp2 = pTemp2->rbnext)
            { ; }
	
	// set the end of the 102/360/102 list to point to the resbuf that
	// will come after it in the entity data chain
	//
        pTemp2->rbnext = pTemp->rbnext;

	// hook the 102/360/102 list head into the entity data list
        pTemp->rbnext = pDict;

        err = ads_entmod(pEnt);
        if (err != RTNORM) {
            ads_getvar("ERRNO", &errno);
            ads_printf("error is : %d", errno.resval.rint);
        }
        ads_relrb(pEnt);
    }
     
    
    // At this point the entity has an extension dictionary.  So,
    // create a resbuf list of the xrecord's entity info and data
    //
    pXrec = ads_buildlist(RTDXF0, "XRECORD",
        100, "AcDbXrecord",
        1, "This is a test Xrecord list",   //AcDb::kDxfText
        10, testpt,                         //AcDb::kDxfXCoord
        40, 3.14159,                        //AcDb::kDxfReal
        50, 3.14159,                        //AcDb::kDxfAngle
        60, 1,                              //AcDb::kDxfColor
        70, 180,                            //AcDb::kDxfInt16
        0);

    // create the Xrecord with no owner set.  The Xrecord's new
    // entity name will be placed into the xrecname argument
    //
    err = ads_entmakex (pXrec, xrecname);
    if (err != RTNORM) {
        ads_getvar("ERRNO", &errno);
        ads_printf("error is : %d", errno.resval.rint);
    }

    // release the resbuf list now that we no longer need it
    //
    ads_relrb(pXrec);

    // set the Xrecord's owner to be the extension dictionary
    //
    err = ads_dictadd(extDict, "XRECADS", xrecname);
    if (err != RTNORM) {
        ads_getvar("ERRNO", &errno);
        ads_printf("error is : %d", errno.resval.rint);
    }
}


void
verifyXDict()
{
    ads_point dummy;
    ads_name ename;

    // have user select an entity.  Then get its data
    //
    ads_entsel("\nselect entity: ", ename, dummy);

    AcDbObjectId objId;
    AOK(acdbGetObjectId(objId, ename));

    AcDbObject *pObj;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForRead));

    AcDbObjectId xDictId = pObj->extensionDictionary();
    if (xDictId == NULL) {
        ads_printf("\nExtension dictionary is not there.\n");
    } else {
        AcDbDictionary *pDict;
        AOK(acdbOpenObject(pDict, xDictId, AcDb::kForRead));
        AcDbDictionaryIterator* pIter = pDict->newIterator();

        int count = 0;
        while (!pIter->done()) {
            count++;
            pIter->next();
        }
        delete pIter;
        ads_printf("\nExtension dictionary has %d objects.\n", count);
    }
}

void
wblockTest()
{
    ads_point dummy;
    ads_name ename;

    // have user select an entity.  Then get it's data
    //
    ads_entsel("\nselect entity: ", ename, dummy);

    AcDbObjectIdArray idArray;
    AcDbObjectId objId;
    AOK(acdbGetObjectId(objId, ename));
    idArray.append(objId);

    ads_entsel("\nselect entity: ", ename, dummy);

    AOK(acdbGetObjectId(objId, ename));
    idArray.append(objId);

    AcDbDatabase *wbDb;
    AcGePoint3d pt(0,0,0);
    acdbCurDwg()->wblock(wbDb, idArray, pt);
    wbDb->saveAs("mynew.dwg");
    
    delete wbDb;
}


void 
saveCurDwgToANewDwg()
{
    acdbCurDwg()->saveAs("mynew.dwg");
}

void btrIterator()
{
    AcDbBlockTableRecord* pBTR;
    Acad::ErrorStatus es;
    AcDbBlockTable*        pBlockTable;

    if ((es = acdbCurDwg()->getBlockTable(pBlockTable, AcDb::kForRead))
           != Acad::eOk) {
        return;
    }

    if ((es = pBlockTable->getAt(ACDB_MODEL_SPACE, pBTR,
	AcDb::kForWrite)) != Acad::eOk)
    {
        return;
    }
    pBlockTable->close();

    AcDbBlockTableRecordIterator* pIter;

    pBTR->newIterator(pIter);

    pBTR->close();

    ads_name ename;
    ads_point dummy;

    // have user select an entity.  Then get it's data
    //
    ads_entsel("\nselect entity: ", ename, dummy);

    AcDbObjectId objId;
    AOK(acdbGetObjectId(objId, ename));

    es  = pIter->seek(objId);
    assert(es == Acad::eOk);

    delete pIter;
}


void highlightBlockInsert()
{
    ads_point dummy;
    ads_name ename;

    // have user select an entity.  Then get it's data
    //
    ads_entsel("\nselect entity: ", ename, dummy);

    AcDbObjectId objId;
    AOK(acdbGetObjectId(objId, ename));

    AcDbBlockReference* pInsert = NULL;
    AOK(acdbOpenObject(pInsert, objId, AcDb::kForRead));
    pInsert->highlight();
    pInsert->close();

}


// Invoked by the command - POLY


void polyCommand()
{
    int nSides = 0;

    while (nSides < 3) {

        ads_initget(INP_NNEG, "");
        switch (ads_getint("\nEnter number of sides: ", &nSides)) {

	case RTNORM:
	    if (nSides < 3)
               ads_printf("\nNeed at least 3 sides.");
	    break;
	default:
	    return;
	}
    }

    ads_point center, startPt, normal;

    if (ads_getpoint(NULL, "\nLocate center of polygon: ", center) 
						       != RTNORM)
	return;

    startPt[0] = center[0]; startPt[1] = center[1]; startPt[2] = center[2];
    while (asPnt3d(startPt) == asPnt3d(center)) {
        switch (ads_getpoint(center, "\nLocate start point of polygon: ", 
							    startPt)) {
	    case RTNORM:
		if (asPnt3d(center) == asPnt3d(startPt))
                    ads_printf("\nPick a point different from the center.");
		break;
	    default:
		return;
	}
    }

    char nameBuf[133];
    switch (ads_getstring(Adesk::kTrue, "\nEnter polygon name: ", nameBuf)) {
    case RTNORM:
	break;
    default:
	return;
    }

    AcDbObjectId tsId = 0;
    char styleBuf[133];
    switch (ads_getstring(Adesk::kTrue, "\nEnter text style: ", styleBuf)) {
    case RTNORM:
	rx_getTextStyleId(styleBuf, acdbCurDwg(), tsId);
	break;
    default:
	return;
    }


    // Set the normal to the plane of the polygon to be the same as the
    // z direction of the current ucs, i.e. (0, 0, 1) since we also got the
    // center and start point in the current UCS. (ads_getpoint() returns in
    // the current UCS.)

    normal[X] = 0.0; normal[Y] = 0.0; normal[Z] = 1.0;

    rx_uc2wc(center, center, Adesk::kFalse);
    rx_uc2wc(startPt, startPt, Adesk::kFalse);
    rx_uc2wc(normal, normal, Adesk::kTrue);

    AcGePoint3d cen = asPnt3d(center), start = asPnt3d(startPt);
    AcGeVector3d norm = asVec3d(normal);

    AcDbPoly* poly = new AcDbPoly(cen, start, nSides, norm, nameBuf, tsId);

    poly->setDatabaseDefaults(acdbCurDwg());
    postToDb(poly);
}


struct PolyEdit
{
    enum EditMode { kGrow     =  0,
		    kShrink   =  1,
		    kMore     =  2,
		    kLess     =  3,
		    kThicken  =  4,
		    kFlatten  =  5,
		    kName     =  6
		  };
};

// static functions used in this file.

static Acad::ErrorStatus    getEditOption(char* option);
static Acad::ErrorStatus
getYOrN(char* promptStr, Adesk::Boolean defaultY, Adesk::Boolean& yes);

static Acad::ErrorStatus    growPoly	(AcDbPoly* poly);
static Acad::ErrorStatus    shrinkPoly	(AcDbPoly* poly);
static Acad::ErrorStatus    morePoly	(AcDbPoly* poly);
static Acad::ErrorStatus    lessPoly	(AcDbPoly* poly);
static Acad::ErrorStatus    thickenPoly (AcDbPoly* poly);
static Acad::ErrorStatus    flattenPoly (AcDb3dSolid* solid);
static Acad::ErrorStatus    namePoly	(AcDbPoly* poly);
static Acad::ErrorStatus    stylePoly	(AcDbPoly* poly);
static Acad::ErrorStatus    setPolyArea (AcDbPoly* poly, double area);
static Acad::ErrorStatus    updatePoly	(AcDbEntity* poly);
static Acad::ErrorStatus    replaceEntity(AcDbEntity*& thisOne, 
					  AcDbEntity*  with);

static int extractEntityInfo(struct resbuf *rb,
			      int&      sel_method,
			      ads_name  ename,
			      ads_name  subname,
			      short&	marker,
			      AcGePoint3d&    pickpnt,
			      AcGeVector3d&   pickvec,
// AcGeMatrix3d& xform,
			      struct resbuf*& containers);


static PolyEdit::EditMode   eMode = PolyEdit::kGrow;
static Adesk::Boolean	    done = Adesk::kFalse, interrupted = Adesk::kFalse; 

static AcGePoint3d   savedCenter;
static AcGePoint3d   savedStartPoint;
static int	     savedNumSides;
static AcGeVector3d  savedNormal;
static char	     savedName[133];

static AcDbEntity   *ent = NULL;


void polyeditCommand()
{
    interrupted = Adesk::kFalse;
    Acad::ErrorStatus es = Acad::eOk;

    AcDbFullSubentPath nullSub;

    // Select a AcDbPoly entity.

    AcDbPoly	 *poly = NULL;
    AcDb3dSolid  *solid = NULL;
    AcDbObjectId  objId;

    ads_name	  ename, sset;

    for (;;) {

        switch (ads_ssget(NULL, NULL, NULL, NULL, sset)) {

	case RTNORM:

	    // Get the poly from the selection set and do some subentity
	    // highlighting testing.
	    //
	    // This code assumes that the user selected only one item and
	    // that's a poly. This is just some test code, so minimal (zero)
	    // error checking is being done.

	{
            struct resbuf *rb;
	    if (ads_ssnamex(&rb, sset, 0) != RTNORM) {
		ads_ssfree(sset);
		return;
	    }
	    int            sel_method;
	    ads_name       subname;
	    short          marker;
	    AcGePoint3d    pickpnt;
	    AcGeVector3d   pickvec;
	    AcGeMatrix3d   xform;
	    struct resbuf* containers;

            if (!extractEntityInfo(rb,
			      sel_method,
			      ename,
			      subname,
			      marker,
			      pickpnt,
			      pickvec,
			      containers)) {
		ads_ssfree(sset);
		return;
	    }

	    AOK(acdbGetObjectId(objId, ename));

	    AOK(acdbOpenAcDbEntity(ent, objId, AcDb::kForRead));
	    assert(ent != NULL);

	    poly = AcDbPoly::cast(ent);
	    if (poly == NULL) {
                ads_printf("\nNot a polygon.");
		AOK(ent->close());
		continue;
	    }

	    // Get the subentity and highlight it.

            int                  numPaths;
            AcDbFullSubentPath*  subentPaths;
            es = poly->getSubentPathsAtGsMarker( AcDb::kEdgeSubentType,
                                            marker,
                                            pickpnt,
                                            xform,
                                            numPaths,
                                            subentPaths);
	    assert(numPaths == 1);

	    ads_printf("Bear with me for a second. Testing highlighting...");

	    // Highlight and unhighlight the subentity.
	    //
	    es = poly->highlight(subentPaths[0]);

	    Adesk::Boolean yes;
	    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
	        es = poly->unhighlight(subentPaths[0]);
	    else if (interrupted)
		goto Done;

	    // Highlight and unhighlight the whole entity.
	    //

	    es = poly->highlight();
	    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
	        es = poly->unhighlight();
	    else if (interrupted)
	        goto Done;

	    delete []subentPaths;
	}
	    break;

	case RTNONE:
	case RTCAN:
	    return;
	default:
	    continue;
	}
	break;
    }

    // Now we have a polygon. Start editing it.

    char option[256];
    eMode = PolyEdit::kGrow;

    done = Adesk::kFalse;
    interrupted = Adesk::kFalse;

    while (!done && !interrupted) {

	AOK(getEditOption(option));

        if (strcmp(option, "Grow") == 0) {

	    if (!ent->isKindOf(AcDbPoly::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kGrow;
	    AOK(growPoly(poly));
	    AOK(updatePoly(poly));

        } else if (strcmp(option, "Shrink") == 0) {

	    if (!ent->isKindOf(AcDbPoly::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kShrink;
	    AOK(shrinkPoly(poly));
	    AOK(updatePoly(poly));

        } else if (strcmp(option, "More") == 0) {

	    if (!ent->isKindOf(AcDbPoly::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kMore;
	    AOK(morePoly(poly));
	    AOK(updatePoly(poly));

        } else if (strcmp(option, "Less") == 0) {

	    if (!ent->isKindOf(AcDbPoly::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kLess;
	    AOK(lessPoly(poly));
	    AOK(updatePoly(poly));

        } else if (strcmp(option, "Thicken") == 0) {

	    if (!ent->isKindOf(AcDbPoly::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kThicken;
	    AOK(thickenPoly(poly));

	    solid = AcDb3dSolid::cast(ent);
	    assert(solid != NULL);
	    poly = NULL;

	    AOK(updatePoly(solid));

	    Adesk::Boolean yes;

#if 0
	    if (getYOrN("\nHit Yes so that the darn thing gets drawn! [Y]", 
			   Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
		;
	    else if (interrupted)
		goto Done;
#endif
	    // But now we can force it with the wonderful demand "draw"
	    // method of AcDbEntity.
	    //
	    solid->draw();

	    // Let's do some highlight testing on the solid.
	    //
	    // Highlight the first edge of the solid, then unhighlight it.
	    //
	    ads_printf("Bear with me for a second. Testing highlighting...");

	    // Highlight and unhighlight the first edge of the solid.
	    //
	    AcDbFullSubentPath subentPath(AcDb::kEdgeSubentType, 1);
	    es = solid->highlight(subentPath);

	    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
	        es = solid->unhighlight(subentPath);
	    else if (interrupted)
		goto Done;

	    // Highlight and unhighlight the faces corresponding to the
	    // first edge of the solid.
	    //
            int                  numPaths;
            AcDbFullSubentPath*  subentPaths;
	    AcGePoint3d    pickpnt;
	    AcGeMatrix3d   xform;
	    es = solid->getSubentPathsAtGsMarker( AcDb::kFaceSubentType,
                                            1, // just the first edge
                                            pickpnt,
                                            xform,
                                            numPaths,
                                            subentPaths);
	    assert(numPaths == 2);

	    es = solid->highlight(subentPaths[0]);

	    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
	        es = solid->unhighlight(subentPaths[0]);
	    else if (interrupted) {
		delete []subentPaths;
		goto Done;
	    }

	    es = solid->highlight(subentPaths[1]);

	    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
	        es = solid->unhighlight(subentPaths[1]);
	    else if (interrupted) {
		delete []subentPaths;
		goto Done;
	    }

	    // Let's get the subentity from these subentIds and post them
	    // to the database. Ask the user if he wants to do this mess
	    // in the first plance.

	    ads_printf("One more test, promise! Hit Enter if you don't want it!");
	    if (getYOrN("\nPost the face subentities? [N]", Adesk::kFalse, 
			yes) == Acad::eOk && !interrupted
		                          && yes == Adesk::kTrue) {
		AcDbEntity* ent = solid->subentPtr(subentPaths[0]);
		postToDb(ent);
		ent = solid->subentPtr(subentPaths[1]);
		postToDb(ent);

	    } else if (interrupted) {
		delete []subentPaths;
		goto Done;
	    }

	    delete []subentPaths;

	    // Highlight the entire solid, then unhighlight it.
	    //
	    es = solid->highlight();
	    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
		&& !interrupted
		&& yes == Adesk::kTrue)
	        es = solid->unhighlight();
	    else if (interrupted)
	        goto Done;

        } else if (strcmp(option, "Flatten") == 0) {

	    if (!ent->isKindOf(AcDb3dSolid::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kFlatten;
	    AOK(flattenPoly(solid));

	    poly = AcDbPoly::cast(ent);
	    assert(poly != NULL);
	    solid = NULL;

	    AOK(updatePoly(poly));

        } else if (strcmp(option, "Name") == 0) {

	    if (!ent->isKindOf(AcDbPoly::desc())) {
                ads_printf("\nNot Applicable");
		continue;
	    }
	    eMode = PolyEdit::kName;
	    AOK(namePoly(poly));
	    AOK(stylePoly(poly));
	    AOK(updatePoly(poly));

        } else if (strcmp(option, "EXit") == 0) {

	    done = Adesk::kTrue;
	} else {
	    done = Adesk::kTrue;
	}

    }

 Done:

    // Close the entity corresponding to the open for read right
    // after selection.

    AOK(ent->close());
}


void
insertHighlightCommand()
{
    // Create a nested insert and try highlighting it's various
    // sub-components.
    //
    // There are six entities in total -- three polys and three
    // boxes (solids). Let's call them: poly1, poly2, poly3 and
    // box1, box2, box3. We also have three inserts: ins1, ins2,
    // ins3.
    //
    // ins3 is insert of a block that contains (poly3, box3).
    // ins2 is insert of a block that contains (poly2, box2, ins3).
    // ins1 is insert of a block that contians (poly1, box1, ins2).
    //
    //
    // Let's create these things first.
    //

    // Polys
    //
    AcDbPoly *poly1, *poly2, *poly3;
    AcGeVector3d norm(0, 0, 1);
    poly1 = new AcDbPoly(AcGePoint3d(2, 8, 0), AcGePoint3d(4, 8, 0), 
					       6, norm, "POLY1");
    poly2 = new AcDbPoly(AcGePoint3d(7, 8, 0), AcGePoint3d(9, 8, 0), 
					       6, norm, "POLY2");
    poly3 = new AcDbPoly(AcGePoint3d(12, 8, 0), AcGePoint3d(14, 8, 0), 
					       6, norm, "POLY3");
    postToDb(poly1);
    postToDb(poly2);
    postToDb(poly3);

    // Boxes
    //
    AcDb3dSolid *box1, *box2, *box3;
    box1 = new AcDb3dSolid(); 
    box2 = new AcDb3dSolid();
    box3 = new AcDb3dSolid();

    box1->createBox(2, 2, 2);
    box2->createBox(2, 2, 2);
    box3->createBox(2, 2, 2);

    AcGeMatrix3d mat;

    mat(0, 3) = 2; mat(1, 3) = 2;
    box1->transformBy(mat);
    mat(0, 3) = 7; mat(1, 3) = 2;
    box2->transformBy(mat);
    mat(0, 3) = 12; mat(1, 3) = 2;
    box3->transformBy(mat);

    postToDb(box1);
    postToDb(box2);
    postToDb(box3);

    // Inserts
    //
    // Arguments to BLOCK are   blockname
    //                          insert point 
    //                          select objects
    //                          empty string for selection complete
    // Arguments to INSERT are  blockname
    //                          insertion point
    //                          xscale
    //                          yscale
    //                          rotation angle

    ads_command(RTSTR, "_globcheck", RTSHORT, 0, RTNONE);

    ads_command(RTSTR, "BLOCK", RTSTR, "blk3", RTSTR, "0,0",
                RTSTR, "14,8", RTSTR, "11,1", RTSTR, "", RTNONE);

    ads_command(RTSTR, "INSERT", RTSTR, "blk3", RTSTR, "0,0", RTSHORT, 1,
                RTSHORT, 1, RTSHORT, 0, RTNONE);

    ads_command(RTSTR, "BLOCK", RTSTR, "blk2", RTSTR, "0,0",
                RTSTR, "9,8", RTSTR, "6,1", RTSTR, "11,1", RTSTR, "", RTNONE);

    ads_command(RTSTR, "INSERT", RTSTR, "blk2", RTSTR, "0,0", RTSHORT, 1,
                RTSHORT, 1, RTSHORT, 0, RTNONE);

    ads_command(RTSTR, "BLOCK", RTSTR, "blk1", RTSTR, "0,0", 
                RTSTR, "4,8", RTSTR, "1,1", RTSTR, "6,1", RTSTR, "", RTNONE);

    ads_command(RTSTR, "INSERT", RTSTR, "blk1", RTSTR, "0,0", RTSHORT, 1,
                RTSHORT, 1, RTSHORT, 0, RTNONE);

    // By now, we have the nested insert we need.
    //
    // Grab the top most insert and open it for read and hold onto it.
    //
    ads_name ename;
    ads_entlast(ename);
    AcDbObjectId topInsertId;
    acdbGetObjectId(topInsertId, ename);

    // We will test highlighting of all the polys (one side and the
    // whole poly) inside the inserts at all the levels. 
    //
    // Then, we will do the same for all the boxes and we will highlight
    // faces of the boxes as well as the whole box.
    //
    // Finally, we will highlight the internal inserts. 
    //

    // Let's just do one right now. Do an ssget and select poly. Then
    // do an ads_nentselp() so that we can get the stack.
    // This is utterly ridiculous. We have to write an ads_nentselx() and
    // ads_nentselpx() that returns the gsMarker.
    //
    // ads_ssget() is supposed to return the stack with ":N" mode, but
    // I am not sure if it works.
    //
    ads_name sset;
    short    marker;
    switch (ads_ssget(NULL, NULL, NULL, NULL, sset)) {

    case RTNORM:
    {
	// This code assumes that the user selected only one item and
	// that's an insert. This is just some test code, so minimal 
	// (zero) error checking is being done.

	struct resbuf *rb;
	if (ads_ssnamex(&rb, sset, 0) != RTNORM) {
	    ads_ssfree(sset);
	    return;
	}
	int            sel_method;
	ads_name       subname;
	AcGePoint3d    pickpnt;
	AcGeVector3d   pickvec;
	AcGeMatrix3d   xform;
	struct resbuf* containers;

	if (!extractEntityInfo(rb,
			  sel_method,
			  ename,
			  subname,
			  marker,
			  pickpnt,
			  pickvec,
			  containers)) {
	    ads_ssfree(sset);
	    return;
	}
	ads_ssfree(sset);
        break;
    }
    default:
	assert(Adesk::kFalse);
        break;
    }

    assert(marker != 0);
    if (marker == 0)
	return;

    ads_point pickpt;
    ads_matrix adsmat;
    struct resbuf *insStack;
    pickpt[0] = 14; pickpt[1] = 8; pickpt[2] = 0;
    if (ads_nentselp(NULL, ename, pickpt, TRUE, adsmat,
					       &insStack) != RTNORM)
	return;

    assert(insStack != NULL);

    struct resbuf *rb = insStack;

    // Just allocate space for 100 ids.
    //
    AcDbObjectId *idArray = new AcDbObjectId[100];
    int count = 0;
    AcDbObjectId objId;
    acdbGetObjectId(objId, ename);
    idArray[count++] = objId;

    while (rb != NULL) {
        ename[0] = rb->resval.rlname[0];
        ename[1] = rb->resval.rlname[1];
	acdbGetObjectId(objId, ename);
        idArray[count++] = objId;

	rb = rb->rbnext;
    }
    count--;
    assert(count <= 3);

    AcGeMatrix3d         xform;
    int                  numPaths;
    AcDbFullSubentPath*  subentPaths;
    AcDbBlockReference*  pInsert;
    AcGePoint3d          pickpnt;

    AOK(acdbOpenObject(pInsert, topInsertId, AcDb::kForRead));
    pInsert->getSubentPathsAtGsMarker(AcDb::kEdgeSubentType,
                                            marker,
                                            pickpnt,
                                            xform,
                                            numPaths,
                                            subentPaths, 
                                            count,
                                            idArray);
    assert(numPaths == 1);
    assert(subentPaths != NULL);

    AcDbFullSubentPath subPath;
    AOK(pInsert->highlight(subentPaths[0]));

    Adesk::Boolean yes = Adesk::kTrue, interrupted = Adesk::kFalse;
    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
	&& !interrupted
	&& yes == Adesk::kTrue) {
	AOK(pInsert->unhighlight(subentPaths[0]));
    } else if (interrupted)
	goto Done;

    delete []subentPaths;


    // Highlight the poly.
    //
    subPath.objectIds().append(idArray[3]);
    subPath.objectIds().append(idArray[2]);
    subPath.objectIds().append(idArray[1]);
    subPath.objectIds().append(idArray[0]);
    pInsert->highlight(subPath);
    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
	&& !interrupted
	&& yes == Adesk::kTrue) {
	AOK(pInsert->unhighlight(subPath));
    } else if (interrupted)
	goto Done;

    // Highlight the insert above poly (3rd in the list).
    //
    subPath.objectIds().removeAt(subPath.objectIds().length() - 1);
    pInsert->highlight(subPath);
    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
	&& !interrupted
	&& yes == Adesk::kTrue) {
	AOK(pInsert->unhighlight(subPath));
    } else if (interrupted)
	goto Done;

    // Highlight 2nd in the list.
    //
    subPath.objectIds().removeAt(subPath.objectIds().length() - 1);
    pInsert->highlight(subPath);
    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
	&& !interrupted
	&& yes == Adesk::kTrue) {
	AOK(pInsert->unhighlight(subPath));
    } else if (interrupted)
	goto Done;

    // Highlight the top one.
    //
    subPath.objectIds().removeAt(subPath.objectIds().length() - 1);
    pInsert->highlight(subPath);
    if (getYOrN("\nUnhighlight? [Y]", Adesk::kTrue, yes) == Acad::eOk 
	&& !interrupted
	&& yes == Adesk::kTrue) {
	AOK(pInsert->unhighlight(subPath));
    } else if (interrupted)
	goto Done;

    pInsert->close();

 Done:
    return;
}


static Acad::ErrorStatus
getEditOption(char* option)
{
    Acad::ErrorStatus es = Acad::eOk;
    int 	      as = RTNORM;

    char promptStr[256];

    ads_initget(0, "Grow Shrink More Less Thicken Flatten Name EXit X");

    switch (eMode) {

    case PolyEdit::kGrow:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<Grow>: ");
	break;

    case PolyEdit::kShrink:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<Shrink>: ");
	break;

    case PolyEdit::kMore:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<More>: ");
	break;

    case PolyEdit::kLess:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<Less>: ");
	break;

    case PolyEdit::kThicken:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<Thicken>: ");
	break;

    case PolyEdit::kFlatten:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<Flatten>: ");
	break;

    case PolyEdit::kName:

	sprintf(promptStr, 
                "\nGrow/Shrink/More/Less/Thicken/Flatten/Name/eXit/<Name>: ");
	break;
    }

    as = ads_getkword(promptStr, option);

    switch (as) {
    case RTNORM:

        if (strcmp(option, "X") == 0) {
            strcpy(option, "EXit");
	}
	break;
    case RTNONE:
	if (eMode == PolyEdit::kGrow) {
            strcpy(option, "Grow");
	} else if (eMode == PolyEdit::kShrink) {
            strcpy(option, "Shrink");
	} else if (eMode == PolyEdit::kMore) {
            strcpy(option, "More");
	} else if (eMode == PolyEdit::kLess) {
            strcpy(option, "Less");
	} else if (eMode == PolyEdit::kThicken) {
            strcpy(option, "Thicken");
	} else if (eMode == PolyEdit::kFlatten) {
            strcpy(option, "Flatten");
	} else if (eMode == PolyEdit::kName) {
            strcpy(option, "Name");
	}
	break;
    case RTCAN:
	interrupted = Adesk::kTrue;
	break;
    default:
	done = Adesk::kTrue;
	break;
    }

    return es;
}

static Acad::ErrorStatus
getYOrN(char* promptStr, Adesk::Boolean defaultY, Adesk::Boolean& yes)
{
    char option[256];
    yes = Adesk::kTrue;

    ads_initget(0, "Yes No");
    int as = ads_getkword(promptStr, option);

    switch (as) {
    case RTNORM:
        if (strcmp(option, "Yes") == 0)
	    yes = Adesk::kTrue;
	else
	    yes = Adesk::kFalse;
	break;

    case RTNONE:
        if (defaultY)
	    yes = Adesk::kTrue;
	else
	    yes = Adesk::kFalse;
	break;

    case RTCAN:
	interrupted = Adesk::kTrue;
	break;
    default:
	break;
    }

    return Acad::eOk;
}


// Increases the area of the polygon by one quarter of its original
// area. New area = 1.25 * old area

static Acad::ErrorStatus growPoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    double oldArea;
    if ((es = poly->getArea(oldArea)) != Acad::eOk) {
        return es;
    }

    double area = oldArea * 1.25;

    if ((es = setPolyArea(poly, area)) != Acad::eOk) {
        return es;
    }

    return es;
}


// Decrease the area of the polygon by twenty percent.
// New area = 0.8 * old area. That way "Grow" followed by a "Shrink"
// will bring the polygon back to its original size.

static Acad::ErrorStatus shrinkPoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    double oldArea;
    if ((es = poly->getArea(oldArea)) != Acad::eOk) {
        return es;
    }

    double area = oldArea * 0.8;

    if ((es = setPolyArea(poly, area)) != Acad::eOk) {
        return es;
    }

    return es;
}


// Increases the number of sides of the polygon by one.

static Acad::ErrorStatus morePoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    int numSides = poly->numSides() + 1;

    if ((es = poly->upgradeOpen()) != Acad::eOk) {
        return es;
    }
    if ((es = poly->setNumSides(numSides)) != Acad::eOk) {
        poly->downgradeOpen();
        return es;
    }
    if ((es = poly->downgradeOpen()) != Acad::eOk) {
        return es;
    }

    return es;
}

// Decreases the number fo sides of the polygon by one.

static Acad::ErrorStatus lessPoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    if (poly->numSides() == 3) {
        ads_printf("\nThat's it.");
	return es;
    }
    int numSides = poly->numSides()  - 1;

    if ((es = poly->upgradeOpen()) != Acad::eOk) {
        return es;
    }
    if ((es = poly->setNumSides(numSides)) != Acad::eOk) {
        poly->downgradeOpen();
        return es;
    }
    if ((es = poly->downgradeOpen()) != Acad::eOk) {
        return es;
    }

    return es;
}


// Extrudes the region to a height that is one-fourth the perimeter.

static Acad::ErrorStatus thickenPoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;
    int i;
    AcDbVoidPtrArray lines;
    AcDbVoidPtrArray regions;

    // Explode to a set of lines
    //
    if ((es = poly->explode(lines)) != Acad::eOk) {
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }

    // Create a region from the set of lines.
    //
    if ((es = AcDbRegion::createFromCurves(lines, regions)) != Acad::eOk) {
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }
    assert(regions.length() == 1);
    AcDbRegion* region = AcDbRegion::cast((AcRxObject*)regions[0]);
    assert(region != NULL);

    // Extrude the region to create a solid.
    //
    double height;
    if ((es = poly->getDistAtParam(6.28318530717958647692, height))
	!= Acad::eOk)
    {
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }
    height *= 0.25;

    AcDb3dSolid* solid = new AcDb3dSolid;
    assert(solid != NULL);
    if ((es = solid->extrude(region, height, 0.0)) != Acad::eOk) {
        delete solid;
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }

    // Save up the data to flatten the solid.
    //
    savedCenter     = poly->center();
    savedStartPoint = poly->startPoint();
    savedNumSides   = poly->numSides();
    savedNormal     = poly->normal();
    char* sname     = poly->name();
    strcpy(savedName, sname);
    free(sname);

    // Now we have a solid. Change the entity to be this solid

    if ((es = solid->setPropertiesFrom(poly)) != Acad::eOk) {
        delete solid;
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }

    ent = poly;
    if ((es = ent->upgradeOpen()) != Acad::eOk) {
        delete solid;
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }
    es = replaceEntity(ent, solid);
    if ((es != Acad::eOk) && (es != Acad::eObjectToBeDeleted)) {
        ent->downgradeOpen();
        delete solid; 
	for (i = 0; i < lines.length(); i++) {
	    delete (AcRxObject*)lines[i];
	}
	for (i = 0; i < regions.length(); i++) {
	    delete (AcRxObject*)regions[i];
	}
	return es;
    }
    es = ent->downgradeOpen();

    for (i = 0; i < lines.length(); i++) {
	delete (AcRxObject*)lines[i];
    }
    for (i = 0; i < regions.length(); i++) {
	delete (AcRxObject*)regions[i];
    }
    return es;
}


static Acad::ErrorStatus flattenPoly(AcDb3dSolid* solid)
{
    Acad::ErrorStatus es = Acad::eOk;

    AcDbPoly* poly = new AcDbPoly(savedCenter, savedStartPoint,
				  savedNumSides, savedNormal, savedName);

    // Change the entity to be the poly. 

    if ((es = poly->setPropertiesFrom(solid)) != Acad::eOk) {
        delete poly;
        return es;
    }

    ent = solid;
    if ((es = ent->upgradeOpen()) != Acad::eOk) {
        delete poly;
        return es;
    }
    es = replaceEntity(ent, poly);
    if((es != Acad::eOk) && (es != Acad::eObjectToBeDeleted)) {
        ent->downgradeOpen();
        delete poly; 
        return es;
    }
    if ((es = ent->downgradeOpen()) != Acad::eOk) {
        return es;
    }

    return es;
}


// Solicit a new name for the polygon

static Acad::ErrorStatus namePoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    char nameBuf[133];
    if (ads_getstring(Adesk::kTrue, "\nEnter new name: ", nameBuf)
	== RTNORM)
    {
        if ((es = poly->upgradeOpen()) != Acad::eOk) {
            return es;
        }
	if ((es = poly->setName(nameBuf)) != Acad::eOk) {
            poly->downgradeOpen();
            return es;
        }

        return poly->downgradeOpen();
    } else
	return Acad::eInvalidInput;
}


// Solicit a new text style for the polygon

static Acad::ErrorStatus stylePoly(AcDbPoly* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    char nameBuf[133];
    int as;
    if ((as = ads_getstring(Adesk::kTrue, "\nEnter new text style: ", 
                                                       nameBuf)) == RTNORM) {
        // NULL string.
        if (strlen(nameBuf) == 0)
            return Acad::eOk;
	AcDbObjectId tsId;
	if ((es = rx_getTextStyleId(nameBuf, acdbCurDwg(), tsId)) != Acad::eOk)
            return es;
        if ((es = poly->upgradeOpen()) != Acad::eOk) {
            return es;
        }
	if ((es = poly->setTextStyle(tsId)) != Acad::eOk) {
            poly->downgradeOpen();
            return es;
        }
        return poly->downgradeOpen();
    } else if (as == RTCAN)
        return Acad::eOk;
    else
	return Acad::eInvalidInput;
}


static Acad::ErrorStatus setPolyArea(AcDbPoly* poly, double area)
{
    Acad::ErrorStatus es = Acad::eOk;

    double theta	= 3.14159265358979323846 * 0.5 - 3.14159265358979323846 / poly->numSides();
    double circumRadius = sqrt
			((2.0 * area) / (poly->numSides() * sin(2.0 * theta)));

    if (circumRadius < 1.0e-10) {
        ads_printf("\nThat's it.");
	return Acad::eOk;
    }

    AcGeVector3d vec = poly->startPoint() - poly->center();
    vec.normalize();
    vec *= circumRadius;
    AcGePoint3d startPt = poly->center() + vec;

    if ((es = poly->upgradeOpen()) != Acad::eOk) {
        return es;
    }
    if ((es = poly->setStartPoint(startPt)) != Acad::eOk) {
        poly->downgradeOpen();
        return es;
    }
    if ((es = poly->downgradeOpen()) != Acad::eOk) {
        return es;
    }

    return es;
}


static Acad::ErrorStatus updatePoly(AcDbEntity* poly)
{
    Acad::ErrorStatus es = Acad::eOk;

    AcDbObjectId objId = poly->objectId();

    ads_name ename;
    if ((es = acdbGetAdsName(ename, objId)) != Acad::eOk) {
        return es;
    }

    ads_redraw(ename, RD_ERASE);
    ads_redraw(ename, RD_DRAW);

    return es;
}


//  Replace "thisOne" with "with" in the database.  As currently written,
//  "thisOne" must be open for write on entry.  "with" is non-database-
//  resident, and on return, it is open for write (and is in the
//  database).  On success, "thisOne" is set equal with "with".
//
static Acad::ErrorStatus replaceEntity(AcDbEntity*& thisOne, 
				       AcDbEntity*  with)
{
    Acad::ErrorStatus es = thisOne->handOverTo(with);
    if (es == Acad::eObjectToBeDeleted) {
	delete thisOne;
	thisOne = with;
    }
    return es;
}

/************************************************************************
 Function:      extractEntityInfo()

 Description:   This function searches through the given resbuf list
                describing the entity and returns entity information.  The
		list must conform to the format returned by ads_ssnamex.
		It is assumed that ads_ssnamex() is being called to retrieve
		one entity at a time, not the entire selection set.

 Inputs:        rb - The resbuf which we are seaching through. It is expected
                     that this point to the beginning of the list or sublist (RTLB).
		     This routine will assert if it doesn't.  rb is used to
		     traverse the list, but since the pointer itself is passed
		     by value, the caller's pointer will not change.

 Outputs:       sel_method - will be filled with the selection method.
                ename - will be filled with the name of the entity (corresponds
		        to the top level container block).  If ename is returned
			(0,0), then no ename info was in the list (indicates the
			list referred to polygon/window points only).
		subname - will be filled with the name of the subentity that
		          was picked (PICK method only).  If entity is not
			  complex (no subentities), or a method other than PICK
			  was used, then subname will equal ename.
	        marker - the GS marker for the selected entity.
		pickpt - will be filled with the pick point if selection method
		         concerns location; otherwise, it is not modified.
			 Caller must test selection method to determine how to
			 deal with the pickpt.
		pickvec - will be filled with the pick point's projection vector
		          if selection method concerns location; otherwise, it
			  is not modified.
			 Caller must test selection method to determine how to
			 deal with the pickvec.
		xform - the transformation matrix for the picked entity.
		containers - resbuf chain to be filled with containing block
		             info.  Will be NULL if no containing block info
			     is defined for the entity.

		Return value - CONTINUE if entity info was extracted, CANCEL if
		               there was an error.  If CANCEL is returned, then
			       the other outputs are not set.
  //
  // ads_ssnamex list format.  The restype of each resbuf is shown.
  //
  // For pick method:
  //    5016 -- start of entity
  //    5003 -- selection method
  //    5006 -- picked entity's name (could be subname)
  //    5003 -- GS marker
  //    5016 -- start of list for pick point
  //    5003 -- point description, not used.
  //    5009 -- pick location (WCS)
  //    5009 -- (only if view is not XY) normal vector for point
  //    5017 -- end of list for pick point
  //    5009 -- xform data (X vector in rotation matrix)
  //    5009 -- xform data (Y vector in rotation matrix)
  //    5009 -- xform data (Z vector in rotation matrix)
  //    5009 -- xform data (Translation vector)
  //    5006 -- (only if applicable) first block in ads_nentselp-style list
  //             of containing blocks.
  //    :    -- (only if applicable) etc. for rest of containing blocks.
  //    5017 -- end of entity
  //
  // For fence method:
  //    5016 -- start of entity
  //    5003 -- selection method
  //    5006 -- picked entity's name (could be subname)
  //    5003 -- GS marker
  //    5016 -- start of list for first fence intersection with entity.
  //    5003 -- point description, not used.
  //    5009 -- pick location (WCS)
  //    5009 -- (only if view is not XY) normal vector for point
  //    5017 -- end of list for first fence intersection with entity.
  //    :    -- etc. for rest of fence points
  //    5017 -- end of entity
  //
  // For crossing, window, crossing polygon, and window polygon methods:
  //    5016 -- start of entity
  //    5003 -- selection method
  //    5006 -- pick entity's name
  //    5003 -- GS marker
  //    5003 -- polygon/window ID.
  //    5017 -- end of entity
  //    5016 -- Start of polygon/window pick points
  //    5003 -- Polygon/Window ID.
  //    5016 -- Start of first point
  //    5003 -- point description, not used.
  //    5009 -- pick location (WCS)
  //    5009 -- (only if view is not XY) normal vector for point
  //    5017 -- end of first point.
  //    :    -- etc. for rest of polygon/window pick points.
  //    5017 -- end of polygon/window pick points.
  //
  // For all method and groups (no location or container information):
  //    5016 -- start of entity
  //    5003 -- selection method
  //    5006 -- picked entity's name
  //    5003 -- GS marker
  //    5017 -- end of entity
  //
  // Note that the crossing methods currently have a bug that returns the
  // the subentities in block entities instead of only the block entity.
  // Each subentity returned will have containing block information.
  //

 Author:        Peter Bergin
 Date Created:  2-94
***********************************************************************/

#define CONTINUE 1
#define CANCEL   0

static int extractEntityInfo(struct resbuf *rb,
			      int&      sel_method,
			      ads_name  ename,
			      ads_name  subname,
			      short&	marker,
			      AcGePoint3d&    pickpnt,
			      AcGeVector3d&   pickvec,
// AcGeMatrix3d& xform,
			      struct resbuf*& containers)
{
   if ( rb == NULL || rb->restype != RTLB )
   {
     assert(0);
     return 0;
   }
   
   // Get the selection method.
   //
   rb = rb->rbnext;  // Bump up to the selection method, always after RTLB.
   sel_method = rb->resval.rint;

   // Get the first ename (could be either the actual entity name or
   // subentity name).
   //
   rb = rb->rbnext;  // Bump up to the first name, always after sel method.
   ename[0] = rb->resval.rlname[0];
   ename[1] = rb->resval.rlname[1];
   subname[0] = rb->resval.rlname[0];
   subname[1] = rb->resval.rlname[1];

   // Get marker info.
   //
   rb = rb->rbnext;
   marker = rb->resval.rint;

   return 1;
}

#if 0 // This code is commented out since I would need all those data
      // structures, e.g. CI_Selection to use it. I am not really in dire
      // need of it right now; all I need is the gsMarker. -Keshav 2/10/95

{
   // Get the pick location and vector, only for PICK and FENCE.  For FENCE,
   // we take the first intersection with the entity as the pick location.
   //
   if ( sel_method == CI_Selection::PICK_METH ||
        sel_method == CI_Selection::FENCE_METH )
   {
      rb = rb->rbnext;  // Skip to RTLB for pick location.
      rb = rb->rbnext;  // Skip to point description info.
      rb = rb->rbnext;  // Skip to the pick location.
      pickpnt.set( rb->resval.rpoint[0],
		     rb->resval.rpoint[1],
		     rb->resval.rpoint[2] );

      rb = rb->rbnext;  // Will be normal vector if we're not in XY view.
                        // Otherwise, it'll be an RTLE for pick location.

      if ( rb->restype == RT3DPOINT )
      {
	pickvec.set( rb->resval.rpoint[0],
		       rb->resval.rpoint[1],
		       rb->resval.rpoint[2] );
	rb = rb->rbnext;  // Make it point to the RTLE for the pick location.
      }

      // Ignore xform data until we are required to support it.
      //
/*
      // Get xform data for PICK_METH.
      //
      if ( sel_method == CI_Selection::PICK_METH ) {
	rb = rb->rbnext;		// Skip to xform X vector
	ArVector3d xvect( rb->resval.rpoint[0],
		          rb->resval.rpoint[1],
		          rb->resval.rpoint[2] );
	rb = rb->rbnext;		// Skip to xform Y vector
	ArVector3d yvect( rb->resval.rpoint[0],
		          rb->resval.rpoint[1],
		          rb->resval.rpoint[2] );
	rb = rb->rbnext;		// Skip to xform Z vector
	ArVector3d zvect( rb->resval.rpoint[0],
		          rb->resval.rpoint[1],
		          rb->resval.rpoint[2] );
	rb = rb->rbnext;		// Skip to xform translation point
	ArPoint3d origin( rb->resval.rpoint[0],
			  rb->resval.rpoint[1],
		          rb->resval.rpoint[2] );

	xform.set( origin, xvect, yvect, zvect );
      }
*/
   }

   // Initialize containers.
   //
   containers = NULL;

   // Now set up for traversing through the rest of the list.  We'll be looking for
   // container information for "containers" and also so that ename
   // can be updated to the real entity, not the subentity.  Container information
   // will only be available if PICK was used.
   //
   if (sel_method == CI_Selection::PICK_METH)
   {
     int begin = 1;   // Since first resbuf was an RTLB.
     int end = 0;     // Haven't found an RTLE yet.

     struct resbuf* container_head = NULL;

     // Loop until begin and end's are all matched up.
     //
     while(begin != end)
     {
         rb = rb->rbnext;

         if(rb->restype == RTENAME)
	 {
	     // Update entity name. Will eventually point to ename of
	     // top level block.
	     //
	     ename[0] = rb->resval.rlname[0]; 
	     ename[1] = rb->resval.rlname[1];

	     // Make a copy of resbuf for containers.  This chain is
	     // assigned by the caller into a CI_Selection object. The chain
	     // will be cleaned up by the CI_Selection destructor.
	     //
	     struct resbuf* tmp = ads_newrb(RTENAME);
	     tmp->resval.rlname[0] = rb->resval.rlname[0];
	     tmp->resval.rlname[1] = rb->resval.rlname[1];
	     tmp->rbnext = NULL;
	     
	     if (containers == NULL)  // first time through.
	     {
		 container_head = tmp;
		 containers = tmp;
	     }
	     else
	     {
		 containers->rbnext = tmp;
		 containers = tmp;
	     }
	 }
         else if(rb->restype == RTLE){
	   end++;
         }
         else if(rb->restype == RTLB){
	   begin++;
         }
     }

     containers = container_head;
   }

   debug_3;

   return CONTINUE;
}
#endif /* 0 */
