#ifndef ACDB_POLY_H
#define ACDB_POLY_H

//
// $Id: poly.h,v 1.18 1995/12/20 17:05:13 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// POLY.H
//
// CREATED BY: Keshav Sahoo		       May 1994
//
// DESCRIPTION: 


#include "dbmain.h"
#include "dbcurve.h"
#include "geassign.h"
#include "gegbl.h"
#include "gegblabb.h"

#include "aced.h"

#include "acestext.h"


#define MESSAGE(x) acadErrorStatusText(x)

#define INTERNAL_STATUS_ERROR(x) \
    acrx_abort("!%s@%d: %s", __FILE__, __LINE__, MESSAGE(x))

#define AOK(what) { Acad::ErrorStatus stat = what; \
		  if (stat != Acad::eOk) INTERNAL_STATUS_ERROR(stat); }



class AcDbPoly: public  AcDbCurve
{
public:
    ACRX_DECLARE_MEMBERS(AcDbPoly);

    //*************************************************************************
    // Constructors and destructor
    //*************************************************************************

    AcDbPoly();
    virtual ~AcDbPoly();

    AcDbPoly(const AcGePoint3d&   center,
             const AcGePoint3d&   startPoint,
             int                  numSides, 
             const AcGeVector3d&  normal,
	     const char*	  pName);

    AcDbPoly(const AcGePoint3d&   center,
             const AcGePoint3d&   startPoint,
             int                  numSides, 
             const AcGeVector3d&  normal,
	     const char*	  pName,
	     AcDbObjectId         styleId);

    //*************************************************************************
    // Methods specific to AcDbPoly 
    //*************************************************************************

    AcGePoint3d        center()      const  {assertReadEnabled();
					     return mCenter;}
    AcGePoint3d        startPoint()  const  {assertReadEnabled();
					     return mStartPoint;}
    int 	       numSides()    const  {assertReadEnabled();
					     return mNumSides;}
    AcGeVector3d       normal()      const  {assertReadEnabled();
					     return mPlaneNormal;}
    char*	       name()	     const;
    AcDbObjectId       styleId()     const;

    Acad::ErrorStatus  setStartPoint (AcGePoint3d startPt)  
				     {assertWriteEnabled();
				      mStartPoint = startPt; return Acad::eOk;}
    Acad::ErrorStatus  setName	     (const char* pName);
    Acad::ErrorStatus  setNumSides   (int numSides);
    Acad::ErrorStatus  setTextStyle  (AcDbObjectId);

    AcGeVector3d       sideVector    (int  whichSide)             const;
    Acad::ErrorStatus  getCoordSys   (AcGeMatrix3d& mat)          const;
    Acad::ErrorStatus  getVertices   (AcGePoint3dArray& vertexArray) const;

    //*************************************************************************
    // Overridden methods from AcDbCurve
    //*************************************************************************

    virtual Adesk::Boolean    isClosed	   () const {assertReadEnabled();
						     return Adesk::kTrue;}  
    virtual Adesk::Boolean    isPeriodic   () const {assertReadEnabled();
						     return Adesk::kTrue;}   
    virtual Adesk::Boolean    isPlanar	   () const {assertReadEnabled();
						     return Adesk::kTrue;}

    virtual Acad::ErrorStatus getPlane  (AcGePlane&        plane, 
                                         AcDb::Planarity&  type) const 
				        {
					   assertReadEnabled();
					   plane.set(mCenter, mPlaneNormal);
					   type = AcDb::kPlanar;

					   return Acad::eOk;
				         }

    virtual Acad::ErrorStatus getStartParam  (double& startParam)     const
				             {assertReadEnabled();
					      startParam = 0.0;
					      return Acad::eOk;}
    virtual Acad::ErrorStatus getEndParam  (double& endParam)    const
					   {assertReadEnabled();
					    endParam = 6.28318530717958647692;
					    return Acad::eOk;}
   
    virtual Acad::ErrorStatus getStartPoint (AcGePoint3d& startPoint)  const
					    {assertReadEnabled();
					     startPoint = mStartPoint;
					     return Acad::eOk;}
    virtual Acad::ErrorStatus getEndPoint (AcGePoint3d& endPoint)    const
					  {assertReadEnabled();
					   endPoint = mStartPoint;
					   return Acad::eOk;}

    virtual Acad::ErrorStatus getPointAtParam  (double             param, 
                                                AcGePoint3d&       point) const;
    virtual Acad::ErrorStatus getParamAtPoint  (const AcGePoint3d& point,
                                                double&            param) const;
    virtual Acad::ErrorStatus getDistAtParam   (double             param,
                                                double&            dist)  const;
    virtual Acad::ErrorStatus getParamAtDist    (double             dist,
                                                double&            param) const; 
    virtual Acad::ErrorStatus getDistAtPoint  (const AcGePoint3d& point,
                                               double&            dist)  const;
    virtual Acad::ErrorStatus getPointAtDist  (double             dist,
                                               AcGePoint3d&       point) const;

    virtual Acad::ErrorStatus getFirstDeriv  (double          param, 
                                              AcGeVector3d&   firstDeriv) const;
    virtual Acad::ErrorStatus getFirstDeriv  (const AcGePoint3d& point, 
                                              AcGeVector3d&   firstDeriv) const;
    virtual Acad::ErrorStatus getSecondDeriv (double          param, 
                                              AcGeVector3d&   secondDeriv)const
					     { return Acad::eNotApplicable;}
    virtual Acad::ErrorStatus getSecondDeriv  (const AcGePoint3d& point, 
                                               AcGeVector3d&  secondDeriv)const
					     { return Acad::eNotApplicable;}

    virtual Acad::ErrorStatus getClosestPointTo (const AcGePoint3d& givenPoint,
                                                AcGePoint3d&       pointOnCurve,
                                                Adesk::Boolean     extend = 
                                                               Adesk::kFalse) 
                                                                      const;

    virtual Acad::ErrorStatus getClosestPointTo (const AcGePoint3d&  givenPoint,
                                                 const AcGeVector3d& normal,
                                                 AcGePoint3d&      pointOnCurve,
                                                 Adesk::Boolean      extend = 
                                                                Adesk::kFalse) 
                                                                      const;

    virtual Acad::ErrorStatus getOrthoProjectedCurve(const AcGePlane&  plane,
                                             AcDbCurve*&       projectedCurve) 
                                                                      const;

    virtual Acad::ErrorStatus getProjectedCurve(const AcGePlane&    plane,
                                             const AcGeVector3d& projDir,
                                             AcDbCurve*&       projectedCurve) 
                                                                      const;

    virtual Acad::ErrorStatus getOffsetCurves (double             offsetDist,
                                      AcDbVoidPtrArray&           offsetCurves)  const;
#if 0
    virtual Acad::ErrorStatus getSpline (AcDbSpline*&	 spline) const;
#endif

    virtual Acad::ErrorStatus getSplitCurves (const AcGeDoubleArray& parameters,
                                      AcDbVoidPtrArray&           curveSegments) const;

    virtual Acad::ErrorStatus getSplitCurves  (const AcGePoint3dArray& points,
                                      AcDbVoidPtrArray&           curveSegments) const;

    virtual Acad::ErrorStatus extend (double		 newParam)
				     {return Acad::eNotApplicable;}

    virtual Acad::ErrorStatus extend (Adesk::Boolean       extendStart,
                                      const AcGePoint3d&   toPoint)
			             {return Acad::eNotApplicable;}

    virtual Acad::ErrorStatus getArea (double&		   area) const; 

    //*************************************************************************
    // Overridden methods from AcDbEntity
    //*************************************************************************

#if 0
    virtual Acad::ErrorStatus	getGeomExtents(AcDbExtents& extents) const;
#endif

    virtual Acad::ErrorStatus getSubentPathsAtGsMarker(
                                 AcDb::SubentType      type,
                                 int                   gsMark,
                                 const AcGePoint3d&    pickPoint,
                                 const AcGeMatrix3d&   viewXform,
                                 int&                  numPaths,
                                 AcDbFullSubentPath*&  subentPaths,
                                 int                   numInserts = 0,
				 AcDbObjectId*         entAndInsertStack 
                                                       = NULL) const;

    virtual Acad::ErrorStatus getGsMarkersAtSubentPath(
                                 const AcDbFullSubentPath& subPath, 
                                 AcDbIntArray&         gsMarkers) const;

    virtual AcDbEntity*       subentPtr(const AcDbFullSubentPath& id) const;
 
    virtual Acad::ErrorStatus   getOsnapPoints(
                                 AcDb::OsnapMode       osnapMode,
                                 int                   gsSelectionMark,
                                 const AcGePoint3d&    pickPoint,
                                 const AcGePoint3d&    lastPoint,
                                 const AcGeMatrix3d&   viewXform,
  	                         AcGePoint3dArray&        snapPoints,
                                 AcDbIntArray&     geomIds) const;

    virtual Acad::ErrorStatus   getGripPoints(AcGePoint3dArray&     gripPoints,
                                              AcDbIntArray&  osnapModes,
                                              AcDbIntArray&  geomIds) const;

    virtual Acad::ErrorStatus   moveGripPointsAt(const AcDbIntArray& indices,
					       const AcGeVector3d&     offset);

    virtual Acad::ErrorStatus	getStretchPointsAt(
                                              AcGePoint3dArray&  stretchPoints)
								      const;
    virtual Acad::ErrorStatus   moveStretchPointsAt(
					      const AcDbIntArray& indices,
                                              const AcGeVector3d&     offset);

    virtual Acad::ErrorStatus   getTransformedCopy(const AcGeMatrix3d& mat,
                                               AcDbEntity*&        ent) const;

    virtual Acad::ErrorStatus	intersectWith(
				      const AcDbEntity*   ent,
				      AcDb::Intersect     intType,
				      AcGePoint3dArray&      points,
				      int                 thisGsMarker  = 0,
				      int                 otherGsMarker = 0)
									const;
 
    virtual Acad::ErrorStatus	intersectWith(
				      const AcDbEntity*   ent,
				      AcDb::Intersect     intType,
				      const AcGePlane&    projPlane,
				      AcGePoint3dArray&      points,
				      int                 thisGsMarker  = 0,
				      int                 otherGsMarker = 0)
									const;
 
    virtual void		list() const;

    virtual Acad::ErrorStatus   transformBy(const AcGeMatrix3d& xform);

    virtual Acad::ErrorStatus	explode(AcDbVoidPtrArray& entitySet) const;

    virtual Adesk::Boolean	worldDraw(AcGiWorldDraw*	mode);
    virtual void		viewportDraw(AcGiViewportDraw*	mode);


    //*************************************************************************
    // Overridden methods from AcDbObject
    //*************************************************************************

    virtual Acad::ErrorStatus   applyPartialUndo(AcDbDwgFiler* undoFiler, 
						 AcRxClass*    classObj);
#if 0
    virtual Acad::ErrorStatus	audit(Adesk::Boolean confirm = Adesk::kFalse);
#endif

    virtual Acad::ErrorStatus	dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus	dwgOutFields(AcDbDwgFiler* filer) const;

    virtual Acad::ErrorStatus	dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus	dxfOutFields(AcDbDxfFiler* filer) const;


private:

    AcGePoint3d   mCenter;
    AcGePoint3d   mStartPoint;
    int           mNumSides;
    AcGeVector3d  mPlaneNormal;

    // Note: the name and text style fields in this entity are here
    // mainly for exercising those types of objects.  Don't worry
    // too much about whether they make sense as part of this
    // particular entity.  The text style field is there to test
    // that hard id's prevent purging of the object pointed to.
    // It's also used to verify that purging works with zombified
    // entities.

    char*	  mpName;
    AcDbHardPointerId  mTextStyle;

    // This enum is meant for speficying operation type on a poly 
    // that do partial undo recording, for example, changing number of
    // sides of the poly.
    //
    enum PolyOpCodeForPartialUndo { kNoOp		= 0,
				    kSetNumSides        = 1 };

};




class AcDbPolyEditorReactor: public  AcEditorReactor
{
public:
    ACRX_DECLARE_MEMBERS(AcDbPolyEditorReactor);
    AcDbPolyEditorReactor(){};
    virtual ~AcDbPolyEditorReactor(){};

    virtual void commandWillStart(const char * cmdStr);
    virtual void commandEnded(const char * cmdStr);
    virtual void commandCancelled(const char * cmdStr);
    virtual void commandFailed(const char * cmdStr);
    virtual void beginClose(AcDbDatabase* pDwg);

    virtual void beginSave(AcDbDatabase* pDwg, const char* pIntendedName);
    virtual void saveComplete(AcDbDatabase* pDwg, const char* pActualName);
};



class AcDbPolyDBReactor: public  AcDbDatabaseReactor
{
public:
    ACRX_DECLARE_MEMBERS(AcDbPolyDBReactor);
    AcDbPolyDBReactor(){};
    virtual ~AcDbPolyDBReactor(){};

    virtual void objectAppended(const AcDbDatabase* pDb,
                                const AcDbObject*   pObj);
    virtual void objectUnAppended(const AcDbDatabase* dwg,
			          const AcDbObject* dbObj);
    virtual void objectReAppended(const AcDbDatabase* dwg,
			          const AcDbObject* dbObj);

};


class AcDbPolyObjReactor: public  AcDbObjectReactor
{
public:
    ACRX_DECLARE_MEMBERS(AcDbPolyObjReactor);
    AcDbPolyObjReactor(){};
    virtual ~AcDbPolyObjReactor(){};

    virtual void	      erased(const AcDbObject* pObj,
				     Adesk::Boolean pErasing = Adesk::kTrue);
    virtual void              unappended(const AcDbObject* dbObj);
    virtual void              reappended(const AcDbObject* dbObj);
};

#endif
